#!/bin/perl -w

############################################################################
# purgesearchresults.pl
#
# Description
#
# A script to purge search results from the database after a configurable
# period of time.
#
# Designed to be run on a regular basis by crontab or other scheduling
# system.
#
# Change History
#
# 27/06/05 Ian Bolton
# Created 
# 
# 02/07/05 Ian Bolton
# Fixed bugs + made database connect use default passwd file
# Now uses shell environment variable
#
# $Id: purgesearchresults.pl,v 1.3 2004/07/05 12:40:52 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

use DBI;

############################################################################
# Library Includes
############################################################################

# Need to look in the configuration file
push(@INC, "$ENV{'HOME'}/public_html/cgi-bin/common/cfg");

# Configuration
require 'config.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# try to open log file, if not exists then create
# name is in configuration
# if it does exist, then get stats on the file, compare with max time allowed to keep file and wipe if over that time

my (@filestats) = stat($CFG{'purge-search-log-file-name'});

# Open the LOG file
if ($#filestats <= 0)
{
	# If file does not exist then create
	open(LOGF, ">$CFG{'purge-search-log-file-name'}") or 
		&critical_error("Could not create new log file", 0);
}
else
{
	# If log file older than the number of hours stated in the config, then
	# overwrite
	my($filehoursold) = ((time - $filestats[8]) / 60) / 60;
	if ($filehoursold > $CFG{'purge-search-log-file-max-age'})
	{
		open(LOGF, ">$CFG{'purge-search-log-file-name'}") or 
		&critical_error("Could not create new log file", 0);
	}
	else
	{
		open(LOGF, ">>$CFG{'purge-search-log-file-name'}") or 
		&critical_error("Could not open log file for appending", 0);
	}
}

# Connect to DB
my ($dbh) = 0;
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&critical_error("$DBI::errstr", 0);
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}

# Go through each search result to locate potential candidates to remove
if ($dberror == 0)
{
	my($sthGetSearchResults) = $dbh->prepare("select searchid, generationtime from Search");
	
	if ($sthGetSearchResults == 0)
	{
		$dberror = 1;
		&critical_error("$DBI:errstr", $dbconnected);
	}
	else
	{
		$sthGetSearchResults->execute() or 
			&critical_error("$DBI:errstr", $dbconnected);
	}

	if ($dberror == 0)
	{
		while(my($searchid, $searchgentime) = 
			$sthGetSearchResults->fetchrow_array())
		{
			# If this record older than the minimum period then delete
			my($recordagemin) = (time - $searchgentime) / 60;
			if ($recordagemin > $CFG{'minimum-search-result-kept-min'})
			{
				my($sthDeleteSearchRecord) = $dbh->prepare("delete from Search where searchid=$searchid");
				my($sthDeleteSearchResults) = $dbh->prepare("delete from SearchResult where searchid=$searchid");

				if ($sthDeleteSearchRecord == 0 || $sthDeleteSearchResults == 0)
				{
					$dberror = 1;
					&critical_error("$DBI:errstr", $dbconnected);
				}
				else
				{
					my($rows) = 0;
					$rows = $sthDeleteSearchResults->execute();

					my($recordrow) = 0;
					$recordrow = $sthDeleteSearchRecord->execute();

					my(@timearray) = localtime(time);
					my($logstring) = sprintf("%02d/%02d/%04d %02d:%02d:%02d ", $timearray[4]+1, $timearray[3], $timearray[5]+1900, $timearray[2], $timearray[1], $timearray[0]);

					if ($rows > 0)
					{
						print LOGF $logstring;
						print LOGF qq(Deleted $rows search result(s) with ID \'$searchid\'\n);
					}
					else
					{
						print LOGF $logstring;
						print LOGF qq(ERROR: No search results were deleted with ID \'$searchid\'!\n);
					}
					if ($recordrow > 0)
					{
						my($stringage) = sprintf("%1.0f", $recordagemin);
						print LOGF $logstring;
						print LOGF qq(Deleted $recordrow search record with ID \'$searchid\'. Results were $stringage minutes old.\n);
					}
					else
					{
						print LOGF $logstring;
						print LOGF qq(ERROR: The search record with ID \'$searchid\' was not deleted!\n);
					}
					
				}
			}
		}
	}
}


# Close Log file
close(LOGF);

if ($dbconnected == 1)
{
	$dbh->disconnect() || &critical_error("Can't Disconnect!", 0);
}

sub critical_error()
{
	my($errmsg, $dbconnected) = @_;
	print "Error: $errmsg\n";
	if ($dbconnected == 1)
	{
		$dbh->disconnect();
	}
	exit;
}
