#!/bin/perl -wT

############################################################################
# deletecollection.cgi
#
# Description
#
# Allows the administrator to Delete a whole collection record
#
# Change History
#
# 09/06/05 Ian Bolton
# Created 
#
# $Id: deletecollection.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

# Find out what the previous link navbar should be
my ($prevlink) = "";
if (exists($FORM{'browse'}))
{
	if ($FORM{'browse'} == 1 && exists($FORM{'letter'}) && exists($FORM{'page'}))
	{
		$prevlink = qq(<a href="browse.cgi?letter=$FORM{'letter'}&page=$FORM{'page'}">Browse $FORM{'letter'} Titles</a> &gt;&gt;);
	}
}
elsif (exists($FORM{'searchtype'}) && exists($FORM{'searchID'}) &&
		exists($FORM{'page'}) && exists($FORM{'numresultsperpage'}))
{
	$prevlink = "";
}

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; $prevlink Delete Collection);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - Confirm Delete Collection",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});

print &parse_template("$CFG{'main-admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&dbdie("$DBI::errstr");
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}

# Check record ID and get title check
my ($sthGetIDTitle);
if ($dberror == 0)
{
	$sthGetIDTitle = $dbh->prepare("select id, title from Collection where id = $FORM{'recordID'}");
	
	if ($sthGetIDTitle == 0)
	{
		$dberror = 1;	
		&dbdie("$DBI::errstr");
	}
}

if ($dberror == 0)
{
	$sthGetIDTitle->execute() or 
		(&dbdie("$DBI::errstr") and ($dberror = 1));
}

my ($rowid, $rowtitle);
if ($dberror == 0)
{
	if (!(($rowid, $rowtitle) = $sthGetIDTitle->fetchrow_array()))
	{
		&dienice("Invalid RecordID passed into script!");
		$dberror = 1;
	}
}

# Check if this is the second call of the script
if (exists($FORM{'confirmdel'}) && $dberror == 0)
{
	
	# Delete it
	if ($FORM{'confirmdel'} eq "YES")
	{
		my (@sthArray) = ();
	
		# Prepare all the delete statements
		my $sthDelMain = $dbh->prepare("delete from Collection where id = ?");
		push(@sthArray, ($sthDelMain, "Collection"));
		my $sthDelConcepts = 
			$dbh->prepare("delete from CollectionConcept where collection = ?");
		push(@sthArray, ($sthDelConcepts, "CollectionConcept"));
		my $sthDelObjects = 
			$dbh->prepare("delete from CollectionObject where collection = ?");
		push(@sthArray, ($sthDelObjects, "CollectionObject"));
		my $sthDelNames = 
			$dbh->prepare("delete from CollectionName where collection = ?");
		push(@sthArray, ($sthDelNames, "CollectionName"));
		my $sthDelPlaces = 
			$dbh->prepare("delete from CollectionPlace where collection = ?");
		push(@sthArray, ($sthDelPlaces, "CollectionPlace"));
		my $sthDelTimes = 
			$dbh->prepare("delete from CollectionTime where collection = ?");
		push(@sthArray, ($sthDelTimes, "CollectionTime"));
		my $sthDelAudiences =
			$dbh->prepare("delete from CollectionAudience where collectionid=?");
		push(@sthArray, ($sthDelAudiences, "CollectionAudience"));
		my $sthDelCatalogues = 
			$dbh->prepare("delete from CollectionCatalogue where collection=?");
		push(@sthArray, ($sthDelCatalogues, "CollectionCatalogue"));
		my $sthDelLocation = 
			$dbh->prepare("delete from CollectionLocation where collectionid=?");
		push(@sthArray, ($sthDelLocation, "CollectionLocation"));
		my $sthDelStrength = 
			$dbh->prepare("delete from CollectionStrength where collectionid=?");
		push(@sthArray, ($sthDelStrength, "CollectionStrength"));
		my $sthDelLanguages = 
			$dbh->prepare("delete from CollectionLanguage where collection=?");
		push(@sthArray, ($sthDelLanguages, "CollectionLanguage"));
			

		if ($sthDelMain == 0 || $sthDelConcepts == 0 || 
			$sthDelObjects == 0 ||  $sthDelNames == 0 || 
			$sthDelPlaces == 0 || $sthDelTimes == 0 ||
			$sthDelAudiences == 0 || $sthDelCatalogues == 0 ||
			$sthDelLocation == 0 || $sthDelStrength == 0 ||
			$sthDelLanguages == 0)
		{
			$dberror = 1;
			&dbdie("$DBI::errstr");
		}

		# Execute them all, report errors but ignore so DB is in better state
		if ($dberror == 0)
		{

			for (my $i = 0; $i < scalar(@sthArray); )
			{
				my ($rows) = 0;
				$rows = $sthArray[$i]->execute($FORM{'recordID'}) or
					($dberror = 1);
				if ($rows > 0)
				{
					$VAR{'DELETE-RESULT'} .= 
						qq(Deleted $rows rows from $sthArray[$i+1]<br />);
				}
				$i += 2;
			}

			# HAVE TO BLANK OUT ANY COLLECTION THAT REFERENCE THIS COLLECTION
			# AS A SUPER COLLECTION. Also for Associatedcollection
			my($sthUpdateSuper) = $dbh->prepare("update Collection set superCollection=0 where superCollection=$FORM{'recordID'}");

			$rows = 0;
			if ($sthUpdateSuper == 0)
			{
				$dberror = 1;
			}
			else
			{
				$rows = $sthUpdateSuper->execute() or ($dberror = 1);
			}
			if ($rows > 0)
			{
				$VAR{'DELETE-RESULT'} .= 
					qq(Updated $rows super-collection References to this record<br />);
			}
			
			my($sthUpdateAssoc) = $dbh->prepare("update Collection set associatedCollection=0 where associatedCollection=$FORM{'recordID'}");

			$rows = 0;
			if ($sthUpdateAssoc == 0)
			{
				$dberror = 1;
			}
			else
			{
				$rows = $sthUpdateAssoc->execute() or ($dberror = 1);
			}
			if ($rows > 0)
			{
				$VAR{'DELETE-RESULT'} .= 
					qq(Updated $rows associated-collection References to this record<br />);
			}

			# Have to do something special to clean up the Person, and
			# CollectionPerson tables

			# Find out which person details exist for this collection
			my($collectperson, $ownerperson, $adminperson) = (0, 0, 0);
			my ($sthGetPerson) = $dbh->prepare("select collectorperson, ownerperson, adminperson from CollectionPerson where collectionid=$FORM{'recordID'}");

			my($criticalerror) = 0;
			if ($sthGetPerson == 0)
			{
				$dberror = 1;
			}
			else
			{
				$sthGetPerson->execute() or 
					(&dbdie("$DBI::errstr") and ($criticalerror = 1) and
					 ($dberror = 1));
			}

			if ($criticalerror == 0)
			{
				# should be just one row cause recordID is primary
				($collectperson, $ownerperson, $adminperson) = 
					$sthGetPerson->fetchrow_array();
				if ($collectperson == NULL)
				{
					$collectperson = 0;
				}
				if ($ownerperson == NULL)
				{
					$ownerperson = 0;
				}
				if ($adminperson == NULL)
				{
					$adminperson = 0;
				}
			}

			# Add none-zero person ids to a 'delete' array
			my(@persondelids) = ();
			if ($collectperson != 0)
			{
				push(@persondelids, $collectperson);
			}
			if ($ownerperson != 0)
			{
				push(@persondelids, $ownerperson);
			}
			if ($adminperson != 0)
			{
				push(@persondelids, $adminperson);
			}

			# Now delete all person records from array
			$rows = 0;
			foreach my $perid (@persondelids)
			{
				my($sthDelPerson) = $dbh->prepare("delete from Person where personID=$perid");
				if ($sthDelPerson == 0)
				{
					$dberror = 1;
				}
				else
				{
					$rows += $sthDelPerson->execute() or ($dberror = 1);
				}
			}

			if ($rows > 0)
			{
				$VAR{'DELETE-RESULT'} .= 
					qq(Deleted $rows rows from Person<br />);
			}

			# Finally, delete the CollectionPerson record
			my($sthDelCollectPerson) = $dbh->prepare("delete from CollectionPerson where collectionid=$FORM{'recordID'}");
			if ($sthDelCollectPerson == 0)
			{
				$dberror = 1;
			}
			else
			{
				$rows = $sthDelCollectPerson->execute() or ($dberror = 1);
				if ($rows > 0)
				{
					$VAR{'DELETE-RESULT'} .= 
						qq(Deleted $rows rows from CollectionPerson<br />);
				}
			}

			if ($dberror == 1)
			{
				$VAR{'DELETE-RESULT'} .= qq(Error(s) occurred so not all data may be have been deleted!<br />);
				$dberror = 0;
			}
			else
			{
				$VAR{'DELETE-RESULT'} .= qq(<br /><b>Whole Collection Record was successfully deleted!</b><br />);
			}
			
			if (!($prevlink eq "") && $FORM{'browse'} == 1)
			{
				$VAR{'DELETE-RESULT'} .= qq(<a href="browse.cgi?letter=$FORM{'letter'}&page=$FORM{'page'}">[Return to Browse Results]</a>);
			}
			elsif (!($prevlink eq "") && exists($FORM{'searchID'}))
			{
				$VAR{'DELETE-RESULT'} .= "";
			}
		}
		
	}
	
	# Do NOT delete
	if (!($FORM{'confirmdel'} eq "YES") || $dberror == 1)
	{
		$VAR{'DELETE-RESULT'} = 
			qq(Record <b>$rowtitle</b> was NOT deleted <br />);
		if (!($prevlink eq "") && $FORM{'browse'} == 1)
		{
			$VAR{'DELETE-RESULT'} .= qq(<a href="browse.cgi?letter=$FORM{'letter'}&page=$FORM{'page'}">[Return to Browse Results]</a>);
		}
		elsif (!($prevlink eq "") && exists($FORM{'searchID'}))
		{
			$VAR{'DELETE-RESULT'} .= "";
		}
	}
	
	# Parse confirm delete template
	print &parse_template("$CFG{'main-admin-templates-rel-dir'}/delete-collection-result-xhtml.template", %VAR);
}

# Display form
elsif ($dberror == 0)
{
	
	# Create hidden fields to maintain form state
	
	$VAR{'DELETE-HIDDEN-FIELDS'} = "";
	$VAR{'DELETE-HIDDEN-FIELDS'} .= 
		$cgi->hidden({name=>"recordID", value=>"$FORM{'recordID'}"});
	
	if (exists($FORM{'browse'}))
	{
		$VAR{'DELETE-HIDDEN-FIELDS'} .= 
			$cgi->hidden({name=>"browse", value=>"$FORM{'browse'}"});
	}
	
	if (exists($FORM{'letter'}))
	{
		$VAR{'DELETE-HIDDEN-FIELDS'} .= 
			$cgi->hidden({name=>"letter", value=>"$FORM{'letter'}"});
	}
	
	if (exists($FORM{'page'}))
	{
		$VAR{'DELETE-HIDDEN-FIELDS'} .= 
			$cgi->hidden({name=>"page", value=>"$FORM{'page'}"});
	}


	if (exists($FORM{'searchID'}))
	{
		$VAR{'DELETE-HIDDEN-FIELDS'} .= 
			$cgi->hidden({name=>"searchID", value=>"$FORM{'searchID'}"});
	}

	if (exists($FORM{'numresultsperpage'}))
	{
		$VAR{'DELETE-HIDDEN-FIELDS'} .= 
			$cgi->hidden({name=>"numresultsperpage", value=>"$FORM{'numresultsperpage'}"});
	}

	if (exists($FORM{'searchtype'}))
	{
		$VAR{'DELETE-HIDDEN-FIELDS'} .= 
			$cgi->hidden({name=>"searchtype", value=>"$FORM{'searchtype'}"});
	}

	
	$VAR{'DELETE-START-FORM'} = 
		$cgi->start_form({action=>"deletecollection.cgi", method=>"post"});
	$VAR{'DELETE-END-FORM'} = $cgi->end_form();
	$VAR{'DELETE-TITLE'} = $rowtitle;
	print &parse_template("$CFG{'main-admin-templates-rel-dir'}/delete-collection-xhtml.template", %VAR);
	
}

# Disconnect
if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}

# Page Footer
print &parse_template("$CFG{'main-admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();
