#!/bin/perl -wT

############################################################################
# resetdblists.cgi
#
# Description
#
# Reset the dropdown lists in the DB from the datafiles.
#
# Change History
#
# 17/06/05 Ian Bolton
# Created 
#
# $Id: resetdblists.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}


# Each script action expects this same script to be called twice
my ($iteration) = 1;
if (exists($FORM{'call'}))
{
	if ($FORM{'call'} == 2)
	{
		$iteration = 2;
	}
}

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; Reset DB Lists);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - Reset DB Lists",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});

print &parse_template("$CFG{'main-admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&dbdie("$DBI::errstr");
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}

my ($displayconfirmform) = 1;

# Display confirm reset operation form
if ($dberror == 0 && $iteration == 1)
{
	# NB: Form displayed by default - anything required here?
}

# If verifed then reset db,
# else either display form or display database not reset
elsif ($dberror == 0 && $iteration == 2)
{
	my($displayresultform) = 0;
	my(@resultrows) = ();
	
	# Read in each file, use to update database
	if ($FORM{'confirmrst'} eq "YES")
	{
		my(@lines) = ();
		my($filename) = "";
		my($tablename) = "";
		
		$displayconfirmform = 0;
		$displayresultform = 1;

		# Set the standard template header
		$VAR{'RESET-ACTION'} = qq(Reseting database lists...);
		$VAR{'RESET-HEAD-ROW'} = &parse_template("$CFG{'main-admin-templates-rel-dir'}/reset-lists-result-head-row-xhtml.template", %VAR);

		# Read each LANGAUGE drop down list item and clear the LANGUAGE table.
		$filename = "languages\.txt";
		$tablename = "Language";
		$dberror = &read_data_file("$CFG{'lists-dir'}/$filename", \@lines);
		if ($dberror == 0)
		{
			my ($delstatement) = "delete from $tablename where name like \'\%\'";
			$dberror = &clear_db_table($delstatement, $dbh);
		}

		# Parse each LANGUAGE data line and add to the database
		my ($rowsupdated) = 0;
		if ($dberror == 0)
		{
			foreach my $line (@lines)
			{
				my ($dataline) = &remove_comments($line);

				my($dbid, $language, $languagecode, $langtype) = 
					split(/\,/,$dataline);

				# Remove newline character from last element on line
				if ($langtype eq "")
				{
					chomp($languagecode);
				}
				else
				{
					chomp($langtype);
				}

				if (!($dbid eq "" || $language eq "" || $languagecode eq ""))
				{
					# Standarise language and language code
					$language = lc($language); 
					$language = ucfirst($language);
					$languagecode = uc($languagecode);

					my ($sthInsert) = $dbh->prepare("insert into Language(id,iso639,name) values(?,?,?)");
					
					if ($sthInsert == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr")
					}
					
					if ($dberror == 0)
					{
						$rowsupdated += $sthInsert->execute($dbid, 
											$languagecode, $language) or
							(($dberror = 1) && &dbdie("$DBI::errstr"));
						if ($rowsupdated <= 0)
						{
							$rowsupdated = 0;
						}
					}
				}
			}
		}

		# Fill in the template result row for LANGUAGE reset
		push(@resultrows, &get_result_row($dberror, $filename, 
							$tablename, $rowsupdated, \%VAR));

		# Reset error status to continue
		$dberror = 0;

		# Read each TYPE-Type drop down list item and clear the table.
		$filename = "typetype\.txt";
		$tablename = "TypeType";
		$dberror = &read_data_file("$CFG{'lists-dir'}/$filename", \@lines);
		if ($dberror == 0)
		{
			my ($delstatement) = "delete from $tablename where text like \'\%\'";
			$dberror = &clear_db_table($delstatement, $dbh);
		}

		# Parse each TYPE-Type data line and add to the database
		my ($rowsupdated) = 0;
		if ($dberror == 0)
		{
			foreach my $line (@lines)
			{
				my ($dataline) = &remove_comments($line);

				my($dbid, @termary) =  split(/\,/,$dataline);
				my($term) = join(',', @termary);
				chomp($term);

				if (!($dbid eq "" || $term eq ""))
				{

					my ($sthInsert) = $dbh->prepare("insert into $tablename(id,text) values(?,?)");
					
					if ($sthInsert == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr")
					}
					
					if ($dberror == 0)
					{
						$rowsupdated += $sthInsert->execute($dbid, $term) or
							(($dberror = 1) && &dbdie("$DBI::errstr"));
						if ($rowsupdated <= 0)
						{
							$rowsupdated = 0;
						}
					}
				}
			}
		}

		# Fill in the template result row for TYPE-Type reset
		push(@resultrows, &get_result_row($dberror, $filename, 
							$tablename, $rowsupdated, \%VAR));

		# Reset error status to continue
		$dberror = 0;

		# Read each TYPE-CuratorialEnv drop down list item and clear the table.
		$filename = "typecuratorialenv\.txt";
		$tablename = "TypeCuratorialEnvironment";
		$dberror = &read_data_file("$CFG{'lists-dir'}/$filename", \@lines);
		if ($dberror == 0)
		{
			my ($delstatement) = "delete from $tablename where text like \'\%\'";
			$dberror = &clear_db_table($delstatement, $dbh);
		}

		# Parse each TYPE-CuratorialEnv data line and add to the database
		my ($rowsupdated) = 0;
		if ($dberror == 0)
		{
			foreach my $line (@lines)
			{
				my ($dataline) = &remove_comments($line);

				my($dbid, @termary) =  split(/\,/,$dataline);
				my($term) = join(',', @termary);
				chomp($term);

				if (!($dbid eq "" || $term eq ""))
				{

					my ($sthInsert) = $dbh->prepare("insert into $tablename(id,text) values(?,?)");
					
					if ($sthInsert == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr")
					}
					
					if ($dberror == 0)
					{
						$rowsupdated += $sthInsert->execute($dbid, $term) or
							(($dberror = 1) && &dbdie("$DBI::errstr"));
						if ($rowsupdated <= 0)
						{
							$rowsupdated = 0;
						}
					}
				}
			}
		}

		# Fill in the template result row for TYPE-CuratorialEnv reset
		push(@resultrows, &get_result_row($dberror, $filename, 
							$tablename, $rowsupdated, \%VAR));

		# Reset error status to continue
		$dberror = 0;

		# Read each TYPE-Content drop down list item and clear the table.
		$filename = "typecontent\.txt";
		$tablename = "TypeContent";
		$dberror = &read_data_file("$CFG{'lists-dir'}/$filename", \@lines);
		if ($dberror == 0)
		{
			my ($delstatement) = "delete from $tablename where text like \'\%\'";
			$dberror = &clear_db_table($delstatement, $dbh);
		}

		# Parse each TYPE-Content data line and add to the database
		my ($rowsupdated) = 0;
		if ($dberror == 0)
		{
			foreach my $line (@lines)
			{
				my ($dataline) = &remove_comments($line);

				my($dbid, @termary) =  split(/\,/,$dataline);
				my($term) = join(',', @termary);
				chomp($term);

				if (!($dbid eq "" || $term eq ""))
				{

					my ($sthInsert) = $dbh->prepare("insert into $tablename(id,text) values(?,?)");
					
					if ($sthInsert == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr")
					}
					
					if ($dberror == 0)
					{
						$rowsupdated += $sthInsert->execute($dbid, $term) or
							(($dberror = 1) && &dbdie("$DBI::errstr"));
						if ($rowsupdated <= 0)
						{
							$rowsupdated = 0;
						}
					}
				}
			}
		}

		# Fill in the template result row for TYPE-Content reset
		push(@resultrows, &get_result_row($dberror, $filename, 
							$tablename, $rowsupdated, \%VAR));

		# Reset error status to continue
		$dberror = 0;

		# Read each TYPE-Policy drop down list item and clear the table.
		$filename = "typepolicy\.txt";
		$tablename = "TypePolicy";
		$dberror = &read_data_file("$CFG{'lists-dir'}/$filename", \@lines);
		if ($dberror == 0)
		{
			my ($delstatement) = "delete from $tablename where text like \'\%\'";
			$dberror = &clear_db_table($delstatement, $dbh);
		}

		# Parse each TYPE-Policy data line and add to the database
		my ($rowsupdated) = 0;
		if ($dberror == 0)
		{
			foreach my $line (@lines)
			{
				my ($dataline) = &remove_comments($line);

				my($dbid, @termary) =  split(/\,/,$dataline);
				my($term) = join(',', @termary);
				chomp($term);

				if (!($dbid eq "" || $term eq ""))
				{

					my ($sthInsert) = $dbh->prepare("insert into $tablename(id,text) values(?,?)");
					
					if ($sthInsert == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr")
					}
					
					if ($dberror == 0)
					{
						$rowsupdated += $sthInsert->execute($dbid, $term) or
							(($dberror = 1) && &dbdie("$DBI::errstr"));
						if ($rowsupdated <= 0)
						{
							$rowsupdated = 0;
						}
					}
				}
			}
		}

		# Fill in the template result row for TYPE-Policy reset
		push(@resultrows, &get_result_row($dberror, $filename, 
							$tablename, $rowsupdated, \%VAR));

		# Reset error status to continue
		$dberror = 0;

		# Read each Concept drop down list item and clear the table.
		$filename = "concepts\.txt";
		$tablename = "Concept";
		$dberror = &read_data_file("$CFG{'lists-dir'}/$filename", \@lines);
		if ($dberror == 0)
		{
			my ($delstatement) = "delete from $tablename where name like \'\%\'";
			$dberror = &clear_db_table($delstatement, $dbh);
		}

		# Parse each Concept data line and add to the database
		my ($rowsupdated) = 0;
		if ($dberror == 0)
		{
			foreach my $line (@lines)
			{
				my ($dataline) = &remove_comments($line);

				my($dbid, @termary) =  split(/\,/,$dataline);
				my($term) = join(',', @termary);
				chomp($term);

				if (!($dbid eq "" || $term eq ""))
				{

					my ($sthInsert) = $dbh->prepare("insert into $tablename(id,name) values(?,?)");
					
					if ($sthInsert == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr")
					}
					
					if ($dberror == 0)
					{
						$rowsupdated += $sthInsert->execute($dbid, $term) or
							(($dberror = 1) && &dbdie("$DBI::errstr"));
						if ($rowsupdated <= 0)
						{
							$rowsupdated = 0;
						}
					}
				}
			}
		}
		$dberror = &set_seq_nums($dberror, $tablename, $dbh);
		# Fill in the template result row for Concept reset
		push(@resultrows, &get_result_row($dberror, $filename, 
							$tablename, $rowsupdated, \%VAR));

		# Reset error status to continue
		$dberror = 0;

		# Read each Name(Personal) drop down list item and clear the table.
		$filename = "namepersonal\.txt";
		$tablename = "NamePersonal";
		$dberror = &read_data_file("$CFG{'lists-dir'}/$filename", \@lines);
		if ($dberror == 0)
		{
			my ($delstatement) = "delete from $tablename where name like \'\%\'";
			$dberror = &clear_db_table($delstatement, $dbh);
		}

		# Parse each Name(Personal) data line and add to the database
		my ($rowsupdated) = 0;
		if ($dberror == 0)
		{
			foreach my $line (@lines)
			{
				my ($dataline) = &remove_comments($line);

				my($dbid, @termary) =  split(/\,/,$dataline);
				my($term) = join(',', @termary);
				chomp($term);

				if (!($dbid eq "" || $term eq ""))
				{

					my ($sthInsert) = $dbh->prepare("insert into $tablename(id,name,sequence) values(?,?,?)");
					
					if ($sthInsert == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr")
					}
					
					if ($dberror == 0)
					{
						$rowsupdated += $sthInsert->execute($dbid, $term, $dbid * 10000) or
							(($dberror = 1) && &dbdie("$DBI::errstr"));
						if ($rowsupdated <= 0)
						{
							$rowsupdated = 0;
						}
					}
				}
			}
		}

		# Fill in the template result row for Name(Personal) reset
		push(@resultrows, &get_result_row($dberror, $filename, 
							$tablename, $rowsupdated, \%VAR));

		# Reset error status to continue
		$dberror = 0;

		# Read each Name(Corporate) drop down list item and clear the table.
		$filename = "namecorporate\.txt";
		$tablename = "NameCorporate";
		$dberror = &read_data_file("$CFG{'lists-dir'}/$filename", \@lines);
		if ($dberror == 0)
		{
			my ($delstatement) = "delete from $tablename where name like \'\%\'";
			$dberror = &clear_db_table($delstatement, $dbh);
		}

		# Parse each Name(Corporate) data line and add to the database
		my ($rowsupdated) = 0;
		if ($dberror == 0)
		{
			foreach my $line (@lines)
			{
				my ($dataline) = &remove_comments($line);

				my($dbid, @termary) =  split(/\,/,$dataline);
				my($term) = join(',', @termary);
				chomp($term);

				if (!($dbid eq "" || $term eq ""))
				{

					my ($sthInsert) = $dbh->prepare("insert into $tablename(id,name) values(?,?)");
					
					if ($sthInsert == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr")
					}
					
					if ($dberror == 0)
					{
						$rowsupdated += $sthInsert->execute($dbid, $term) or
							(($dberror = 1) && &dbdie("$DBI::errstr"));
						if ($rowsupdated <= 0)
						{
							$rowsupdated = 0;
						}
					}
				}
			}
		}
		$dberror = &set_seq_nums($dberror, $tablename, $dbh);

		# Fill in the template result row for Name(Corporate) reset
		push(@resultrows, &get_result_row($dberror, $filename, 
							$tablename, $rowsupdated, \%VAR));

		# Reset error status to continue
		$dberror = 0;

		# Read each Place drop down list item and clear the table.
		$filename = "places\.txt";
		$tablename = "Place";
		$dberror = &read_data_file("$CFG{'lists-dir'}/$filename", \@lines);
		if ($dberror == 0)
		{
			my ($delstatement) = "delete from $tablename where name like \'\%\'";
			$dberror = &clear_db_table($delstatement, $dbh);
		}

		# Parse each Place data line and add to the database
		my ($rowsupdated) = 0;
		if ($dberror == 0)
		{
			foreach my $line (@lines)
			{
				my ($dataline) = &remove_comments($line);

				my($dbid, @termary) =  split(/\,/,$dataline);
				my($term) = join(',', @termary);
				chomp($term);

				if (!($dbid eq "" || $term eq ""))
				{

					my ($sthInsert) = $dbh->prepare("insert into $tablename(id,name) values(?,?)");
					
					if ($sthInsert == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr")
					}
					
					if ($dberror == 0)
					{
						$rowsupdated += $sthInsert->execute($dbid, $term) or
							(($dberror = 1) && &dbdie("$DBI::errstr"));
						if ($rowsupdated <= 0)
						{
							$rowsupdated = 0;
						}
					}
				}
			}
		}
		$dberror = &set_seq_nums($dberror, $tablename, $dbh);

		# Fill in the template result row for Place reset
		push(@resultrows, &get_result_row($dberror, $filename, 
							$tablename, $rowsupdated, \%VAR));

		# Reset error status to continue
		$dberror = 0;

		# Read each Time drop down list item and clear the table.
		$filename = "times\.txt";
		$tablename = "Time";
		$dberror = &read_data_file("$CFG{'lists-dir'}/$filename", \@lines);
		if ($dberror == 0)
		{
			my ($delstatement) = "delete from $tablename where name like \'\%\'";
			$dberror = &clear_db_table($delstatement, $dbh);
		}

		# Parse each Time data line and add to the database
		my ($rowsupdated) = 0;
		if ($dberror == 0)
		{
			foreach my $line (@lines)
			{
				my ($dataline) = &remove_comments($line);

				my($dbid, @termary) =  split(/\,/,$dataline);
				my($term) = join(',', @termary);
				chomp($term);

				if (!($dbid eq "" || $term eq ""))
				{

					my ($sthInsert) = $dbh->prepare("insert into $tablename(id,name,sequence) values(?,?,?)");
					
					if ($sthInsert == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr")
					}
					
					if ($dberror == 0)
					{
						$rowsupdated += $sthInsert->execute($dbid, $term,
															$dbid*10000) or
							(($dberror = 1) && &dbdie("$DBI::errstr"));
						if ($rowsupdated <= 0)
						{
							$rowsupdated = 0;
						}
					}
				}
			}
		}

		# Fill in the template result row for Time reset
		push(@resultrows, &get_result_row($dberror, $filename, 
							$tablename, $rowsupdated, \%VAR));

	}
		
	# 
	if ($FORM{'confirmrst'} eq "NO")
	{
		$displayconfirmform = 0;
		$displayresultform = 1;
		
		$VAR{'RESULT-RES'} = qq(<br />The database drop down lists <b>were NOT reset</b>.<br />);
	}

	# Display results operation
	if ($displayresultform == 1)
	{
		$VAR{'RESULT-RES'} .= qq(<a href="main.cgi">[Return to Main Menu]</a>);

		$VAR{'RESET-BODY-ROWS'} = "@resultrows";

		print &parse_template("$CFG{'main-admin-templates-rel-dir'}/reset-lists-result-xhtml.template", %VAR);
	}
}

if ($displayconfirmform == 1)
{

	# Fill in delete form template tags
	$VAR{'RESET-START-FORM'} = $cgi->start_form({method=>'post',
		action=>'resetdblists.cgi'});
	$VAR{'RESETE-END-FORM'} = $cgi->end_form();
	
	# Indicate second iteration of script
	$VAR{'RESET-HIDDEN-FIELDS'} = $cgi->hidden({name=>'call', value=>"2"});
		
	print &parse_template("$CFG{'main-admin-templates-rel-dir'}/reset-lists-xhtml.template", %VAR);
	
}

# Disconnect
if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}

# Page Footer
print &parse_template("$CFG{'main-admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();

sub read_data_file()
{
	my($filename, $linesref) = @_;
	my($reterrval) = 0;
	
	# Read in language file
	open(LISTFILE, $filename) || ($reterrval = 1);
	if ($reterrval == 0)
	{
		@$linesref = <LISTFILE>;
		close(LISTFILE);
	}
	else
	{
		&dienice("Could not read in file \'$filename\'");
	}
	return ($reterrval);
}

sub clear_db_table()
{
	my($delstatement, $dbh) = @_;
	my($reterrval) = 0;

	my ($sthDel) = $dbh->prepare($delstatement);
	if ($sthDel == 0)
	{
		$reterrval = 1;
		&dbdie("$DBI::errstr");
	}
	else
	{
		$sthDel->execute() or 
			(($reterrval = 1) and (&dbdie("$DBI::errstr")));
	}
	return ($reterrval);
}

sub set_seq_nums()
{
	my($dberror, $tablename, $dbh) = @_;

		if ($dberror == 0)
		{
			my($sthSortData) = $dbh->prepare("select id from $tablename order by name");
			my($sthInsertSeq) = $dbh->prepare("update $tablename set sequence=? where id=?");
			if ($sthSortData == 0 || $sthInsertSeq == 0)
			{
				$dberror = 1;
				&dbdie("$DBI::errstr")
			}
			
			if ($dberror == 0)
			{
				$sthSortData->execute() or 
				(($dberror = 1) && &dbdie("$DBI::errstr"));
			}
					
			if ($dberror == 0)
			{
				my($nextseqnum) = 10000;
				while ( my($thisid) = $sthSortData->fetchrow_array() )
				{
					$sthInsertSeq->execute( $nextseqnum, $thisid ) or 
					 (($dberror = 1) && &dbdie("$DBI::errstr"));
					 
					 if ( $dberror == 1 )
					 {
					 	last;
					 }
					 $nextseqnum += 10000;
				}
			}
		}
	return($dbresult);
}

sub remove_comments()
{
	my($line) = @_;
	
	# Remove comments
	while ($line =~ /#.*/)
	{
		$line =~ s/#.*//;
	}
	
	return($line);
}

sub get_result_row()
{
	my($dberror, $filename, $tablename, $rowsupdated, $varhashref) = @_;
	
	if ($dberror == 1)
	{
			$varhashref->{'RESET-RESULT'} = 
				qq(<font color="#ff0000">ERROR</font>);
	}
	else
	{
			$varhashref->{'RESET-RESULT'} = qq(<font color="#00ff00">OK</font>);
	}

	$varhashref->{'ROWS-UPDATED'} = "$rowsupdated";
	$varhashref->{'RESET-FILENAME'} = "$filename";
	$varhashref->{'RESET-TABLENAME'} = "$tablename";

	return(&parse_template("$CFG{'main-admin-templates-rel-dir'}/reset-lists-result-body-row-xhtml.template", %$varhashref));
}
