#!/bin/perl -wT

############################################################################
# updatelocation.cgi
#
# Description
#
# Allow the administrator to update a list.
#
# Change History
#
# 03/10/04 Ian Bolton
# Created 
#
#
# $Id: updatelocation.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# Standard help image used on the form
$VAR{'HELP-IMAGE'} = "$CFG{'image-dir'}/helpQ.gif";

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

# Find out what the previous link navbar should be
my ($prevlink) = "";
if (!exists($FORM{'field'}) || $FORM{'field'} eq "" )
{
	$FORM{'field'} = "Time";
}


# This script does not need to distinguish between add and update because
# it can work it out for itself. Just set to the more common Update op.
my ($action) = "";
if (exists($FORM{'action'}))
{
	if ($FORM{'action'} eq "add")
	{
		$action = "add";
	}
	elsif ($FORM{'action'} eq "update")
	{
		$action = "update";
	}
	elsif ($FORM{'action'} eq "delete")
	{
		$action = "delete";
	}
}

# Set up the template for all <form> begin tags
$VAR{'UPDATEDBLIST-FORM-START'} = $cgi->start_form( {action=>'updatedblist.cgi',
													 method=>'post' } );
$VAR{'UPDATEDBLIST-FORM-START'} .= $cgi->hidden({name=>'field', value=>"$FORM{'field'}"});

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; <a href="listview.cgi">Update Dropdown Lists</a> &gt;&gt; $FORM{'field'} List);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - Update $FORM{'field'} List",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});

print &parse_template("$CFG{'main-admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&dbdie("$DBI::errstr");
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}

# See if we are to add any records to the DB
my( $validentry ) = 1;
$VAR{'UPDATEDBLIST-RESULT-MSG'} = "";
if ( ( $dberror == 0 ) && ( $action eq "add" ) )
{
	# neworder, newname
	# check if order blank or not given - same error
	# check same for name
	# if order <= 0 then = 1
	# updid
	# order$updid
	# name$updid

	if ( !exists($FORM{'neworder'}) || $FORM{'neworder'} !~ /^\d+$/ )
	{
		$validentry = 0;
		$VAR{'UPDATEDBLIST-RESULT-MSG'} = "Error: An invalid position number was entered! Please try again..";
	}
	elsif ( !exists($FORM{'newname'}) || $FORM{'newname'} eq "" )
	{
		$validentry = 0;
		$VAR{'UPDATEDBLIST-RESULT-MSG'} = "Error: invalid text was entered for the list item name! Please try again..";
	}
	elsif ( $FORM{'neworder'} <=0 )
	{
		$FORM{'neworder'} = 1;
	}
}

# validate update parameters
if ( ( $dberror == 0 ) && ( $action eq "update" ) )
{
	# updid
	# order$updid
	# name$updid
#print "<h2>order$updid $FORM{'order$updid'}</h2>";
	if ( !exists($FORM{'updid'}) || $FORM{'updid'} !~ /^\d+$/ )
	{
		$validentry = 0;
		$VAR{'UPDATEDBLIST-RESULT-MSG'} = "Error: No primary key was given!";
	}
	elsif ( !exists($FORM{"order$FORM{'updid'}"}) || $FORM{"order$FORM{'updid'}"} !~ /^\d+$/ )
	{
		$validentry = 0;
		$VAR{'UPDATEDBLIST-RESULT-MSG'} = "Error: An invalid position number was entered! Please try again..";
	}
	elsif ( !exists($FORM{"name$FORM{'updid'}"}) || $FORM{"name$FORM{'updid'}"} eq "" )
	{
		$validentry = 0;
		$VAR{'UPDATEDBLIST-RESULT-MSG'} = "Error: invalid text was entered for the list item name! Please try again..";
	}
	elsif ( $FORM{"order$FORM{'updid'}"} <=0 )
	{
		$FORM{"order$FORM{'updid'}"} = 1;
	}
}

if ( ( $dberror == 0 ) && ( ( $action eq "add" ) || ( $action eq "update" ) ) &&
	 ( $validentry == 1 ) )
{
	# go through list until end or less than. break. get next record.
	# if one exists then get half one value between new and previous,
	# else add a certain number of thousdan to last and put on end of list
	my ($sthGetTimes) = $dbh->prepare("select id, sequence from $FORM{'field'} order by sequence asc");
	my ($sthGetPriKey) = $dbh->prepare("select max(id) from $FORM{'field'}");

	if ($sthGetTimes == 0 || $sthGetPriKey == 0)
	{
		$dberror = 1;	
		&dbdie("$DBI::errstr");
	}

	if ($dberror == 0)
	{
		$sthGetTimes->execute() or 
			(&dbdie("$DBI::errstr") and ($dberror = 1));
	}

	if ($dberror == 0)
	{
		$sthGetPriKey->execute() or 
			(&dbdie("$DBI::errstr") and ($dberror = 1));
	}

	if ($dberror == 0)
	{
		my ($reccount) = 0;
		my( $timeidmax, $timesequence );
		my( $timeid ) = 0;
		my( $prevseqnum ) = 0;
		my( $nextseqnum ) = 0;
		my( $newseqnum );
		my( $ordernum );
		
		if ( $action eq	 "add" )
		{
			$ordernum = $FORM{'neworder'};
		}
		else
		{
			$ordernum = $FORM{"order$FORM{'updid'}"};
		}

		while ( ( $timeid, $timesequence ) = 
					$sthGetTimes->fetchrow_array() )
		{
			$reccount++;

			if ( ( $action eq "update" ) &&
				 ( $timeid == $FORM{'updid'} ) && 
			 	( $reccount < $ordernum) )
			{
				$ordernum++;
			}

			$nextseqnum = $timesequence;

			if ( $ordernum <= $reccount )
			{
				last;
			}

			$prevseqnum = $timesequence;
		}

		# If our position is the same then set same position
		if ( ( $action eq "update" ) &&
			 ( $timeid == $FORM{'updid'} ) )
		{
			$newseqnum = $nextseqnum;
		}

		# Add to the end of existing records, or 1st record
		elsif ( $reccount < $ordernum )
		{
			#add someit to old sequence number	
			$newseqnum = $prevseqnum + 6000;
			$ordernum = $reccount;
			if ( $ordernum == 0 )
			{
				$ordernum =1 ;
			}
		}
		else
		{
			# get half way value
			$newseqnum = ( $prevseqnum + $nextseqnum ) / 2;
		}

		if ( ! ( ( $timeidmax ) = 
					$sthGetPriKey->fetchrow_array() ) )
		{
			$timeidmax = 1;
		}

		# Add this record
		my ($sthAddUpdItem);
		if ( $action eq "add" )
		{
			$sthAddUpdItem = $dbh->prepare("insert into $FORM{'field'}(id, name, sequence) values(?,?,?)");
		}
		else
		{
			$sthAddUpdItem = $dbh->prepare("update $FORM{'field'} set name=?, sequence=? where id=?");
		}

		if ($sthAddUpdItem == 0)
		{
			$dberror = 1;	
			&dbdie("$DBI::errstr");
		}

		if ($dberror == 0 && $action eq "add")
		{
			$sthAddUpdItem->execute($timeidmax+3, $FORM{'newname'},
									$newseqnum) or 
				(&dbdie("$DBI::errstr") and ($dberror = 1));
			if ( $dberror == 0 )
			{
				$VAR{'UPDATEDBLIST-RESULT-MSG'} = "Successfully added new record into position $ordernum";
			}
		}

		if ($dberror == 0 && $action eq "update")
		{
			$sthAddUpdItem->execute($FORM{"name$FORM{'updid'}"},
									$newseqnum, $FORM{'updid'}) or 
				(&dbdie("$DBI::errstr") and ($dberror = 1));
			if ( $dberror == 0 )
			{
				$VAR{'UPDATEDBLIST-RESULT-MSG'} = "Successfully updated record in position $ordernum";
			}
		}
	}	
}

#delete
if ( $dberror == 0 && $action eq "delete" )
{
	if (exists($FORM{'delid'}) && $FORM{'delid'} =~ /^\d+$/)
	{
		my($sthDel) = $dbh->prepare("delete from $FORM{'field'} where id=$FORM{'delid'}");
		
		if ( $sthDel == 0 )
		{
			$dberror = 1;	
			&dbdie("$DBI::errstr");
		}

		if ($dberror == 0)
		{
			$sthDel->execute() or 
				( ( $dberror = 1 ) && &dbdie("$DBI::errstr") );

			if ( $dberror == 0 )
			{
				$VAR{'UPDATEDBLIST-RESULT-MSG'} = "Successfully deleted one record from list";
			}
		}
	}
}

# Set message font colour
if ( $validentry == 0 )
{
	$VAR{'UPDATEDBLIST-MSG-FONT-COLOR'} = "#ff0000";
}
else
{
	$VAR{'UPDATEDBLIST-MSG-FONT-COLOR'} = "";
}	

# Generate the page
if ($dberror == 0)
{
	my ($displayaddeditform) = 1;

	# Need to load add/edit form template values from DB
	my ($sthGetTimes) = $dbh->prepare("select id, name, sequence from $FORM{'field'} order by sequence asc");
	
	if ($sthGetTimes == 0)
	{
		$dberror = 1;	
		&dbdie("$DBI::errstr");
	}

	if ($dberror == 0)
	{
		$sthGetTimes->execute() or 
			(&dbdie("$DBI::errstr") and ($dberror = 1));
	}
			
	if ($dberror == 0)
	{
		# should be just one row cause collectionID is primary
		my ($reccount) = 0;
		$VAR{'UPDATEDBLIST-RESULT-ROWS'} = "";
		while ( my( $timeid, $timename, $timesequence ) = 
					$sthGetTimes->fetchrow_array() )
		{
				$reccount++;
					# Set the form templates for this row
				$VAR{'UPDATEDDBLIST-VAL-ORDER'} = $reccount;
				$VAR{'UPDATEDBLIST-INPUT-ORDER'} = "order$timeid";
				$VAR{'UPDATEDBLIST-VAL-NAME'} = $timename;
				$VAR{'UPDATEDBLIST-INPUT-NAME'} = "name$timeid";
				$VAR{'UPDATEDBLIST-PRIKEY'} = $timeid;
				$VAR{'UPDATEDBLIST-RESULT-ROWS'} .= &parse_template("$CFG{'main-admin-templates-rel-dir'}/update-dblist-form-result-row-xhtml.template", %VAR);
		}
	}

	if ($dberror)
	{
		$displayaddeditform = 0;
	}
	
	# Set standard template params and display add/edit form
	if ($displayaddeditform == 1)
	{
		print &parse_template("$CFG{'main-admin-templates-rel-dir'}/update-dblist-form-xhtml.template", %VAR);
	}
}

# Disconnect
if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}

# Page Footer
print &parse_template("$CFG{'main-admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();
