############################################################################
# Config.pl
#
# Description
# ===========
#
# The configuration file for the Tap Into Bath Dynamic content.
#
# Configuration Variable Descriptions
# ===================================
#
# TO FOLLOW
#
# Change History
# ==============
#
# 26/05/04 Ian Bolton
# Created
#
# 02/07/04 Ian Bolton
# Updated stylesheet paths + added many other configurable items
#
# 05/07/04 Ian Bolton
# Database name now specified in this config
#
# 08/07/04 Ian Bolton
# Changed the wording of Audience tickbox text and added two more
# tickbox elements - 16+ and Professional
#
# $Id: config.pl,v 1.9 2004/07/09 00:29:17 ibolton Exp $
#
############################################################################
sub get_config()
{
	my (%config) = ();

	# Standard directory and file locations
	$config{'home-dir'} = '/s/tap';

	$config{'database-file-path'} = "$config{'home-dir'}" . '/.my.cnf';
	
	$config{'webmaster-email'} = 'tapintobath@bath.ac.uk';

	$config{'image-dir'} = '../../images';

	$config{'public-stylesheet-rel-path'} = '../../stylesheets/public/tapintobath.css';

	$config{'public-templates-rel-dir'} = '../../templates/public';

	$config{'admin-stylesheet-rel-path'} = '../../stylesheets/admin/tapintobath.css';

	$config{'admin-templates-rel-dir'} = '../../templates/entry';
	$config{'main-admin-templates-rel-dir'}= '../../templates/admin';

	$config{'lists-dir'} = '../common/lists';

	# Name of database
	$config{'database-name'} = 'tapintobath';

	# Search help text
	$config{'search-help-method'} = 'A word search will match any (OR) or all (AND) words entered, which are separated by space. A phrase search will match the exact text string entered only.';

	# Element help text
	$config{'help-title'} = 'The title of the collection.';
	$config{'help-description'} = 'A free text description of the collection. Usual length is one paragraph.';
	$config{'help-physical-char'} = 'The format or type of materials held in the collection (e.g. books, serials).';
	$config{'help-size'} = 'The size of the collection (e.g. 3,000 books).';
	$config{'help-audience'} = 'The target audience of the collection (e.g. undergraduate).';
	$config{'helpconcept'} = 'Subject terms (LCSH) associated with the collection';
	$config{'helpobject'} = 'Specific objects (LCSH) associated with a collection (e.g. footwear, dresses, hats)';
	$config{'helpname'} = 'Names of people and organisations associated with the collection (e.g. Isaac Pitman)';
	$config{'helpplace'} = 'Names of places associated with a collection';
	$config{'helptime'} = 'Time period associated with the collection (e.g. Roman Period)';
	$config{'helpstrength'} = 'An indication of specific strengths of a collection';
	$config{'helpcatalogue'} = 'Enter a catalogue that is available for this collection, and what form it takes.';

	# Advanced data form help text
	$config{'help-adv-acc-date-range'} = 'Period over which collection has been accumulated or collected (e.g. 1963 onwards).';
	$config{'help-adv-cnt-date-range'} = 'The earliest and latest dates of items held in the collection.';
	$config{'help-adv-supercoll'} = 'This collection is part of another collection already held in the database.';
	$config{'help-adv-associatedcoll'} = 'This collection has some connection to another collection already held in the database.';
	$config{'help-adv-associatedpub'} = 'Publication directly connected to the collection.';
	$config{'help-adv-accrualstatus'} = 'Is this collection still being added to? How does this collection add material - by purchase and/or donation?';
	$config{'help-adv-legalstatus'} = 'The legal status of the collection.';
	$config{'help-adv-custodialhist'} = 'Details of who has had custody of the collection, prior to its current ownership or administration.';
	$config{'help-adv-accessctrl'} = 'Who can use this collection (Note that opening hours, admission charges, etc are in Location section).';
	$config{'help-adv-note'} = 'Any information not held in other fields.';

	# Person form help text
	$config{'help-person-name'} = 'The name of the person.';
	$config{'help-person-organisation'} = 'An organisation associated with this person.';
	$config{'help-person-role'} = 'The role (job title) of this person within the organisation.';
	$config{'help-person-telnum'} = 'The telephone number associated with this person.';
	$config{'help-person-faxnum'} = 'The fax number associated with this person.';
	$config{'help-person-email'} = 'The email address associated with this person.';
	$config{'help-person-history'} = 'Further details of named person or organisation.';

	# Location form help text
	$config{'help-location-name'} = 'The name of the building in which the collection is located, if appropriate.';
	$config{'help-location-address'} = 'Full postal address of the building.';
	$config{'help-location-postcode'} = 'Postcode of building.';
	$config{'help-location-country'} = 'The country in which this collection is located.';
	$config{'help-location-locator'} = 'URL of home page for collection.';
	$config{'help-location-access'} = 'Hours of opening, admission charges, accessibility for disabled users, etc.';
	$config{'help-location-seealso'} = 'Any addition location information.';

	# Type data form help text
	$config{'help-type-form'} = 'The type of the collection. The field uses a set of predefined groups from the collection description schema.';

	# Name form help text
	$config{'help-name-form-personal'} = 'Personal names of people associated with this collection.';
	$config{'help-name-form-corporate'} = 'Corporate names of people or organisations associated with this collection.';

	# Places form help text
	$config{'help-places-form-place'} = 'Names of places associated with this Collection.';

	# Times form help text
	$config{'help-times-form-time'} = 'Time periods associated with this Collection.';

	# Element data entry error messages
	$config{'error-title'} = 'The Title must be entered.';
	$config{'error-description'} = 'The Description must be entered.';
	$config{'error-physical-char'} = 'At least one Characteristic must be entered.';
	$config{'error-size'} = 'The collection Size must be entered.';
	$config{'error-audience'} = 'At least one Audience checkbox must be selected.';
	$config{'error-concept'} = 'At least one Concept must be selected from the list.';
	$config{'error-invalid-locator'} = 'A valid web URL must begin with http://';

	# The Person contact information form
	$config{'error-invalid-email'} = 'Please enter a valid Email address, or leave blank.';
	
	$config{'errorobject'} = 'An object must be entered.';
	$config{'errorname'} = 'A name is required!';
	$config{'errorplace'} = 'A place is required!';
	$config{'errortime'} = 'A time is required!';
	
	$config{'errorstrength'} = 'Invalid Strength!';

	$config{'errorcatalogue'} = 'A catalogue must be entered.';

	# Number of Administrator Browse results per page
	$config{'num-browse-results-per-page'} = 2;

	# Number of public search results per page
	$config{'num-search-results-per-page'} = 5;

	# Character limit on the size of the description displayed
	# in the public search results.
	$config{'public_search_description_sizelimit'} = 200;

	# The text of the Audience Tick boxes
	$config{'audience-general-tickbox-text'} = 'General Public';
	$config{'audience-school-tickbox-text'} = 'Schools';
	$config{'audience-undergrad-tickbox-text'} = 'Undergraduate';
	$config{'audience-research-tickbox-text'} = 'Research';
	$config{'audience-sixteen-tickbox-text'} = '16+';
	$config{'audience-professional-tickbox-text'} = 'Professional';

	# The search purge log file name
	$config{'purge-search-log-file-name'} = "$config{'home-dir'}" . '/database/purge/purgelog.txt';

	# The maximum period to keep the search purge log file, in hrs
	$config{'purge-search-log-file-max-age'} = 24;

	# The minimum period in which to keep an individual search result
	# in minutes
	$config{'minimum-search-result-kept-min'} = 10;

	return(%config);
}
1;
