############################################################################
# Utility.pl
#
# Description
# ===========
#
# Contains a collection of general purpose utility functions.
#
# Change History
# ==============
#
# 27/05/04 Ian Bolton
# Created
#
# 02/07/04 Ian Bolton
# Removed unnecessary functions now that the data entry interface has
# changed.
#
# 09/07/04 Ian Bolton
# Added function get_record_title().
#
############################################################################

############################################################################
# Functions
############################################################################

############################################################################
# Function
# ========
#
# get_mod_time()
#
# Description
# ===========
#
# Returns the Modification time of the calling script in the format,
# "Wed, 25 May 2004".
#
# Change History
# ==============
#
# 27/05/05 Ian Bolton
# Created
#
############################################################################
sub get_mod_time()
{
	my (@filestats) = stat($0);
	my (@lastmodtime) = localtime($filestats[9]);
	my (@shortdays) = qw(Sun Mon Tue Wed Thu Fri Sat);
	my (@shortmonths) = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);

	return sprintf("%3s, %2d %3s %04d", $shortdays[$lastmodtime[6]],
	 $lastmodtime[3], $shortmonths[$lastmodtime[4]], ($lastmodtime[5]+1900));
}

############################################################################
# Function
# ========
#
# get_template_values()
#
# Description
# ===========
#
# Sets the values that can appear in XHTML templates.
#
# Change History
# ==============
#
# 27/05/05 Ian Bolton
# Created
#
############################################################################
sub get_template_values()
{
	my (%CFG) = @_;
	my (%VAR);
	
	# Set template parsing variables
	$VAR{'WEBMASTER-EMAIL'} = $CFG{'webmaster-email'};
	$VAR{'CFG-IMAGE-DIR'} = $CFG{'image-dir'};
	$VAR{'MODIFICATION-TIME'} = &get_mod_time();
	
	return(%VAR);
}

sub dienice()
{
	my($msg) = @_;
	print "<h2>Error</h2>\n";
	print $msg;
	print "<br />";
}

sub dbdie()
{
	my($dberrmsg) = @_;
	my($package, $filename, $line) = caller;
	my($errmsg) = "Database error: $dberrmsg<br />
		called from $package $filename line $line";
	&dienice($errmsg);
}


sub generate_browse_list()
{
	my ($link, $page) = @_;
	my ($listresult) = "";
	
	if ($link eq "")
	{
		$link = "searchorbrowse.cgi";
	}

	foreach my $letter (A..Z)
	{
		$listresult .= qq(<a href="$link?letter=$letter&page=$page">[$letter]</a> );
	}

	return ($listresult);
}

############################################################################
# Function
# ========
#
# get_record_title()
#
# Description
# ===========
#
# Retrieves the title string from the collection database for the record ID
# passed in as an argument. Ignores any errors.
#
# Change History
# ==============
#
# 09/07/04 Ian Bolton
# Created
#
############################################################################
sub get_record_title()
{
	my($recid, $cfgref) = @_;
	my($rectitle) = "";
	my($error) = 0;
	my($dbconnected) = 0;

	my($tmpdbh) = DBI->connect("dbi:mysql:$cfgref->{'database-name'}" . ";mysql_read_default_file=$cfgref->{'database-file-path'}");

	if ($tmpdbh == 0)
	{
		$error = 1;
	}
	else
	{
		$dbconnected = 1;
	}

	if ($error == 0)
	{
		my($gettitlesth) = $tmpdbh->prepare("select title from Collection where id=$recid");
		if ($gettitlesth == 0)
		{
			$error = 1;
		}
		else
		{
			$gettitlesth->execute() or $error = 1;
		}

		if ($error == 0)
		{
			if (!($rectitle = $gettitlesth->fetchrow_array()))
			{
				$rectitle = "";
			}
		}
	}

	if ($dbconnected == 1)
	{
		$tmpdbh->disconnect();
	}

	return($rectitle);
}
1;
