#!/bin/perl -wT

############################################################################
# addeditdelcatalogue.cgi
#
# Description
#
# Allows the administrator to Add, Edit, or Delete a collection Catalogue
# in the DB
#
# Change History
#
# 22/06/05 Ian Bolton
# Created 
#
# 01/07/07 Ian Bolton
# Updated links to record manager for ease of navigation.
#
# $Id: addeditdelcatalogue.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# Standard help image used on the form
$VAR{'HELP-IMAGE'} = "$CFG{'image-dir'}/helpQ.gif";

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

# Find out what the previous link navbar should be
my ($prevlink) = "";
if (exists($FORM{'recordID'}))
{
	$prevlink = qq(<a href="recordmanager.cgi?recordID=$FORM{'recordID'}#catalogue">Collection Record Manager</a> &gt;&gt;);
}

# Script allows three different actions
my ($action) = "";
if (exists($FORM{'action'}))
{
	if ($FORM{'action'} eq "add")
	{
		$action = "Add";
	}
	elsif ($FORM{'action'} eq "delete")
	{
		$action = "Delete";
	}
	elsif ($FORM{'action'} eq "update")
	{
		$action = "Update";
	}
}

# Each script action expects this same script to be called twice
my ($iteration) = 1;
if (exists($FORM{'call'}))
{
	if ($FORM{'call'} == 2)
	{
		$iteration = 2;
	}
}

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; <a href="searchorbrowse.cgi">Search or Browse</a> &gt;&gt; $prevlink $action Catalogue);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - $action Catalogue",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});

print &parse_template("$CFG{'admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&dbdie("$DBI::errstr");
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}


# Are we Adding or Updating a record
if ($dberror == 0 && ($action eq "Add" || $action eq "Update"))
{
	my ($displayaddeditform) = 1;
	
	# Set submit button text template
	if ($action eq "Update")
	{
		$VAR{'CATALOGUE-SUBMIT-VALUE'} = "Update Database";
	}
	else
	{
		$VAR{'CATALOGUE-SUBMIT-VALUE'} = "Add to Database";
	}
	
	# On first iteration display the add/edit form with or without values
	# set from the database
	if ($iteration == 1)
	{
		
		# Need to load add/edit form template values from DB
		if ($action eq "Update")
		{

			my ($sthGetCatalogue) = $dbh->prepare("select catalogue from CollectionCatalogue where catalogueid=$FORM{'catalogueID'} and collection=$FORM{'recordID'}");
	
			if ($sthGetCatalogue == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");
			}

			if ($dberror == 0)
			{
				$sthGetCatalogue->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}
			
			if ($dberror == 0)
			{
				if (my($cataloguetext) =
					$sthGetCatalogue->fetchrow_array())
				{
					$VAR{'CATALOGUE-ENTRY-VALUE'} = "$cataloguetext";
				}
				else
				{
					# Ignore invalid DB error?
				}
			}
			
		# end $update
		}

	# end $iteration1
	}

	# On second iteration, validate form values then add/update the DB OR
	# if not valid display the add/edit form with submitted values and red
	# error row to indicate error.
	elsif ($iteration == 2)
	{
		# Validate Catalogue
		my ($validform) = 1;
		if (!exists($FORM{'entry'}) || $FORM{'entry'} eq "")
		{
			$validform = 0;
			$VAR{'ERROR-ROW-TEXT'} = $CFG{'errorcatalogue'};
			$VAR{'CATALOGUE-FIELD-ERROR-ROW'} = &parse_template("$CFG{'admin-templates-rel-dir'}/error-row-xhtml.template", %VAR);
		}
		
		# If form valid add/update DB and do not display add/update form
		if ($validform == 1)
		{
			$displayaddeditform = 0;
			
			# Update
			my($sthAddUpd);
			if ($action eq "Update")
			{
				$sthAddUpd = $dbh->prepare("update CollectionCatalogue set catalogue=? where catalogueid=$FORM{'catalogueID'} and collection=$FORM{'recordID'}");
			}
			# Add
			else
			{
				$sthAddUpd = $dbh->prepare("insert into CollectionCatalogue(collection, catalogue) values($FORM{'recordID'},?)");
			}
			
			if ($sthAddUpd == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");	
			}
			
			my ($rows) = 0;
			if ($dberror == 0)
			{
				$rows = $sthAddUpd->execute($FORM{'entry'}) or 
						(($dberror = 1) && &dbdie("$DBI::errstr"));
				if ($rows <= 0)
				{
					$rows = 0;
				}
			}
			
			# Set output result row template
			if ($dberror == 0)
			{
				if ($action eq "Update")
				{
					$VAR{'ADD-UPD-DEL-RESULT'} = qq(Updated $rows row(s) from CollectionCatalogue<br />);
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Record was Updated Successfully!</b><br />);
				}
				else
				{
					$VAR{'ADD-UPD-DEL-RESULT'} = qq(Added $rows row to CollectionCatalogue<br />);
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Record was Added Successfully!</b><br />);
				}
			}
			else
			{
				$VAR{'ADD-UPD-DEL-RESULT'} = qq(<b>Error(s) occurred so not all data may have been $action!</b><br />);
			}
			
			$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<a href="recordmanager.cgi?recordID=$FORM{'recordID'}#catalogue">[Return to Collection Record Manager]</a>);
			
			# Output results template
			print &parse_template("$CFG{'admin-templates-rel-dir'}/add-upd-del-results-xhtml.template", %VAR);
		}
		
		# Form not valid, so save all fields by setting their values as before
		else
		{
			$VAR{'CATALOGUE-ENTRY-VALUE'} = $FORM{'entry'};
		}
	# end iteration 2
	}
	
	if ($dberror)
	{
		$displayaddeditform = 0;
	}
	
	# Set standard template params and display add/edit form
	if ($displayaddeditform == 1)
	{
		my ($formstr) = "";
		$formstr = $cgi->start_form({action=>'addeditdelcatalogue.cgi', 
									 method=>'post'});
		$formstr .= $cgi->hidden({name=>'recordID', 
								  value=>"$FORM{'recordID'}"});
		$formstr .= $cgi->hidden({name=>'action', 
								  value=>"$FORM{'action'}"});
								  
		# Always indicate second iteration of script
		$formstr .= $cgi->hidden({name=>'call', value=>"2"});

		$formstr .= $cgi->hidden({name=>'catalogueID',
								  value=>"$FORM{'catalogueID'}"});
		$VAR{'CATALOGUE-FORM-START'} = $formstr;
		
		# Field help text
		$VAR{'CATALOGUE-HELP'} = $CFG{'helpcatalogue'};
		
		print &parse_template("$CFG{'admin-templates-rel-dir'}/add-edit-catalogue-form-xhtml.template", %VAR);
	}
# end add || update
}

# This element is to be deleted
elsif ($dberror == 0 && $action eq "Delete")
{
	my ($displaydeleteform) = 1;
	
	# Display the confirm record delete form
	if ($iteration == 1)
	{
		# Form displayed by default. Anything else for this section?

	# end iteration 1
	}
	
	# Was dialogue confirmed yes? if so delete from DB, 
	# otherwise just output nothing was deleted.
	# if no checkboxes selected then display form
	elsif ($iteration == 2)
	{
		my ($displayresultform) = 0;
		
		# Attempt to delete this record
		if ($FORM{'confirmdel'} eq "YES")
		{
			$displaydeleteform = 0;
			
			# prepare delete statement
			my ($sthDelete) = $dbh->prepare("delete from CollectionCatalogue where catalogueid=$FORM{'catalogueID'} and collection=$FORM{'recordID'}");

			if ($sthDelete == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");	
			}
			
			# execute delete statement
			if ($dberror == 0)
			{
				my ($rows) = 0;
				$rows = $sthDelete->execute() or 
						(($dberror = 1) && &dbdie("$DBI::errstr"));
				if ($rows <= 0)
				{
					$rows = 0;
				}
				
				if ($dberror == 0)
				{
					$displayresultform = 1;
					$VAR{'ADD-UPD-DEL-RESULT'} = 
						qq(Deleted $rows row(s) from CollectionCatalogue<br />);
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Record was Deleted Successfully!</b><br />);
				}
			}
		}
		
		# Either user said NO or an error prevented data deletion
		if ($FORM{'confirmdel'} eq "NO" || $dberror == 1)
		{
			$displaydeleteform = 0;
			$displayresultform = 1;

			$VAR{'ADD-UPD-DEL-RESULT'} = 
				qq(Record <b>Catalogue (ID $FORM{'recordID'},$FORM{'catalogueID'})</b> was NOT deleted <br />);
		}

		# Display results of the delete operation
		if ($displayresultform == 1)
		{
			$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<a href="recordmanager.cgi?recordID=$FORM{'recordID'}#catalogue">[Return to Collection Record Manager]</a>);
			
			print &parse_template("$CFG{'admin-templates-rel-dir'}/add-upd-del-results-xhtml.template", %VAR);
		}
		
	# end iteration 2
	}
	
	if ($displaydeleteform == 1)
	{
		my ($formstr) = "";

		# Fill in delete form template tags
		$VAR{'DELETE-START-FORM'} = $cgi->start_form({method=>'post',
			action=>'addeditdelcatalogue.cgi'});
		$VAR{'DELETE-END-FORM'} = $cgi->end_form();
		
		$formstr .= $cgi->hidden({name=>'recordID', 
								  value=>"$FORM{'recordID'}"});
		$formstr .= $cgi->hidden({name=>'action', 
								  value=>"$FORM{'action'}"});
								  
		# Always indicate second iteration of script
		$formstr .= $cgi->hidden({name=>'call', value=>"2"});

		$formstr .= $cgi->hidden({name=>'catalogueID', 
								  value=>"$FORM{'catalogueID'}"});

		$VAR{'DELETE-HIDDEN-FIELDS'} = "$formstr";

		$VAR{'DELETE-TITLE'} = "Catalogue (ID $FORM{'recordID'},$FORM{'catalogueID'})";
		
		print &parse_template("$CFG{'admin-templates-rel-dir'}/delete-collection-xhtml.template", %VAR);
	}	
		
# end action del
}

# Disconnect
if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}

# Page Footer
print &parse_template("$CFG{'admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();
