#!/bin/perl -wT

############################################################################
# addupdatebasic.cgi
#
# Description
#
# Allows the administrator to set the basic information associated with
# a particular collection. The title, description, physical characteristic,
# size, audience, and concepts.
#
# The Add and Update actions are permitted.
#
# Cannot be deleted without removing the whole collection from the DB.
#
# Change History
#
# 20/06/05 Ian Bolton
# Created
#
# 01/07/07 Ian Bolton
# Updated links to record manager for ease of navigation.
#
# 08/07/07 Ian Bolton
# Added the handling of two new elements for the audience field - 16+ and
# profession.
#
# 03/10/04 Ian Bolton
# Added support for new concept table with sequence numbers
#
# $Id: addupdatebasic.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# Standard help image used on the form
$VAR{'HELP-IMAGE'} = "$CFG{'image-dir'}/helpQ.gif";

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

# Find out what the previous link navbar should be
my ($prevlink) = "";
if (exists($FORM{'recordID'}))
{
	$prevlink = qq(<a href="searchorbrowse.cgi">Search or Browse</a> &gt;&gt; <a href="recordmanager.cgi?recordID=$FORM{'recordID'}#basic">Collection Record Manager</a> &gt;&gt;);
}
else
{
	# Must be creating a new Collection, so set ID to NULL for now
	$FORM{'recordID'} = NULL;
}

# Script allows Add and Update Collection information.
# Delete via another script
my ($action) = "";
if (exists($FORM{'action'}))
{
	if ($FORM{'action'} eq "add")
	{
		$action = "Add";
	}
	elsif ($FORM{'action'} eq "delete")
	{
		$action = "Delete";
	}
	elsif ($FORM{'action'} eq "update")
	{
		$action = "Update";
	}
}

# Each script action expects this same script to be called twice
my ($iteration) = 1;
if (exists($FORM{'call'}))
{
	if ($FORM{'call'} == 2)
	{
		$iteration = 2;
	}
}

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; $prevlink $action Basic Collection Information);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - $action Basic Collection Information",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});

print &parse_template("$CFG{'admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&dbdie("$DBI::errstr");
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}

# Build hash of which concepts are already associated with this collection,
# if any! Alongwith their id's
my(%dbconcepts);
if ($dberror == 0)
{
	my($sthGetAssocConcepts) = $dbh->prepare("select Concept.name, Concept.id from Concept,CollectionConcept where Concept.id=CollectionConcept.concept and CollectionConcept.collection=$FORM{'recordID'}");
	
	if ($sthGetAssocConcepts == 0)
	{
		$dberror = 1;
		&dbdie("$DBI::errstr");
	}
	else
	{
		$sthGetAssocConcepts->execute() or 
			(($dberror=1) and (&dbdie("$DBI::errstr")));
	}
	
	if ($dberror == 0)
	{
	
		while( my($concept, $conceptid) = 
			$sthGetAssocConcepts->fetchrow_array())
		{
			$dbconcepts{$conceptid} = $concept;
		}
	}
}

# Are we Adding or Updating a record
if ($dberror == 0 && ($action eq "Add" || $action eq "Update"))
{
	my ($displayaddeditform) = 1;
	
	# Set submit button text template
	if ($action eq "Update")
	{
		$VAR{'BASIC-FORM-SUBMIT-VALUE'} = "Update Database";
	}

	else
	{
		$VAR{'BASIC-FORM-SUBMIT-VALUE'} = "Add to Database";
	}
	
	# On first iteration display the add/edit form with or without values
	# set from the database
	if ($iteration == 1)
	{
		# Fill in the listbox values if we are adding or editing

		# Fill in the concept list box with subject terms
		my($sthGetConcepts) = $dbh->prepare("select id, name from Concept order by sequence");
		if ($sthGetConcepts == 0)
		{
			$dberror = 1;	
			&dbdie("$DBI::errstr");
		}
		if ($dberror == 0)
		{
			$sthGetConcepts->execute() or 
				(&dbdie("$DBI::errstr") and ($dberror = 1));
		}
		if ($dberror == 0)
		{
			my ($numrows) = 0;
			while (my($recid,$rectitle) = 
				$sthGetConcepts->fetchrow_array())
			{
				$numrows++;
				my($conceptoptstr) = qq(<option value="$recid" );

				# See if the record id should be selected
				# by default in the concept list
				if (exists($dbconcepts{$recid}))
				{
					$conceptoptstr .= "selected>";
				}
				else
				{
					$conceptoptstr .= ">";
				}
				$conceptoptstr .= "$rectitle";

				$VAR{'BASIC-OPTIONS-CONCEPTS'} .= 
					qq($conceptoptstr</option>);
			}

			if ($numrows == 0)
			{
				$dberror = 1;
				&dbdie("There are no Concepts loaded into the database! Please reset the database.");
			}
		}
		
		# Need to load add/edit form template values from DB
		if ($action eq "Update" && $dberror == 0)
		{
			
			my ($sthGetBasic) = $dbh->prepare("select title, description, physicalCharacteristics, size from Collection where id=$FORM{recordID}");
	
			if ($sthGetBasic == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");
			}

			if ($dberror == 0)
			{
				$sthGetBasic->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}
			
			if ($dberror == 0)
			{
				# should be just one row cause recordID is primary
				if (my($title, $description, $physchar, $size) = 
						$sthGetBasic->fetchrow_array())
				{
					# Set the form templates
					$VAR{'BASIC-VAL-TITLE'} = $title;
					$VAR{'BASIC-VAL-DESCRIPTION'} = $description;

					$VAR{'BASIC-VAL-PHYSCHAR'} = $physchar;
					$VAR{'BASIC-VAL-SIZE'} = $size;

					# Fill in the Audience check box values
					my($sthGetAudiences) = $dbh->prepare("select generalinterest, schoolvisit, undergrad, research, sixteenplus, professional from CollectionAudience where collectionid=$FORM{'recordID'}");

					if ($sthGetAudiences == 0)
					{
						$dberror = 1;	
						&dbdie("$DBI::errstr");
					}

					if ($dberror == 0)
					{
						$sthGetAudiences->execute() or 
							(&dbdie("$DBI::errstr") and ($dberror = 1));
					}

					if ($dberror == 0)
					{
						if (my($general, $school, $undergrad, $research, 
							   $sixteenplus, $professional) = 
							$sthGetAudiences->fetchrow_array())
						{
							# If any flag is 1, then set the checkbox to select
							if ($general == 1)
							{
								$VAR{'BASIC-VAL-SEL-AUDIENCE-GENERAL'} = 
									"checked";
							}
							if ($school == 1)
							{
								$VAR{'BASIC-VAL-SEL-AUDIENCE-SCHOOL'} = 
									"checked";
							}
							if ($undergrad == 1)
							{
								$VAR{'BASIC-VAL-SEL-AUDIENCE-UNDERGRAD'} = 
									"checked";
							}
							if ($research == 1)
							{
								$VAR{'BASIC-VAL-SEL-AUDIENCE-RESEARCH'} = 
									"checked";
							}
							if ($sixteenplus == 1)
							{
								$VAR{'BASIC-VAL-SEL-AUDIENCE-SIXTEENPLUS'} = 
									"checked";
							}
							if ($professional == 1)
							{
								$VAR{'BASIC-VAL-SEL-AUDIENCE-PROFESSIONAL'} = 
									"checked";
							}
						}
						else
						{
							# No audience record exists for this collection.
							# should be an error, but none critical.
							# A well designed program can get around this!
							# by adding instead of updating on 2nd iteration
						}
					}
				}
				else
				{
					$dberror = 1;
					&dbdie("Could not find Basic record with ID $FORM{'recordID'}");
				}
			}
			
		# end $update
		}

	# end $iteration1
	}
	
	# On second iteration, validate form values then add/update the DB OR
	# if not valid display the add/edit form with submitted values and red
	# error row to indicate error.
	elsif ($iteration == 2)
	{
		# Validate form fields
		my ($validform) = 1;

		if (!exists($FORM{'title'}) || $FORM{'title'} eq "")
		{
			$validform = 0;
			$VAR{'ERROR-ROW-TEXT'} = $CFG{'error-title'};
			$VAR{'BASIC-ERRORROW-TITLE'} = &parse_template("$CFG{'admin-templates-rel-dir'}/error-row-xhtml.template", %VAR);
		}

		if (!exists($FORM{'description'}) || $FORM{'description'} eq "")
		{
			$validform = 0;
			$VAR{'ERROR-ROW-TEXT'} = $CFG{'error-description'};
			$VAR{'BASIC-ERRORROW-DESCRIPTION'} = &parse_template("$CFG{'admin-templates-rel-dir'}/error-row-xhtml.template", %VAR);
		}

		if (!exists($FORM{'physchar'}) || $FORM{'physchar'} eq "")
		{
			$validform = 0;
			$VAR{'ERROR-ROW-TEXT'} = $CFG{'error-physical-char'};
			$VAR{'BASIC-ERRORROW-PHYSCHAR'} = &parse_template("$CFG{'admin-templates-rel-dir'}/error-row-xhtml.template", %VAR);
		}

		if (!exists($FORM{'size'}) || $FORM{'size'} eq "")
		{
			$validform = 0;
			$VAR{'ERROR-ROW-TEXT'} = $CFG{'error-size'};
			$VAR{'BASIC-ERRORROW-SIZE'} = &parse_template("$CFG{'admin-templates-rel-dir'}/error-row-xhtml.template", %VAR);
		}

		if ( (my($numsel) = ($cgi->param(audience))) <= 0 )
		{
			$validform = 0;
			$VAR{'ERROR-ROW-TEXT'} = $CFG{'error-audience'};
			$VAR{'BASIC-ERRORROW-AUDIENCE'} = &parse_template("$CFG{'admin-templates-rel-dir'}/error-row-xhtml.template", %VAR);
		}

		if ( (my($numsel) = ($cgi->param(concepts))) <= 0 )
		{
			$validform = 0;
			$VAR{'ERROR-ROW-TEXT'} = $CFG{'error-concept'};
			$VAR{'BASIC-ERRORROW-CONCEPTS'} = &parse_template("$CFG{'admin-templates-rel-dir'}/error-row-xhtml.template", %VAR);
		}

		# FURTHER PROCESSING that is required if the form is valid or not

		# If the form is invalid, then save the Audience check box values
		# If the form is valid, then convert submitted values into DB values
		my ($audgeneral, $audschool, $audundergrad, $audresearch,
			$audsixteenplus, $audprofessional) = (0,0,0,0,0,0);
		foreach my $selectedaudiencebox ($cgi->param(audience))
		{
			if ($selectedaudiencebox eq "general")
			{
				$VAR{'BASIC-VAL-SEL-AUDIENCE-GENERAL'} = "checked";

				$audgeneral = 1;
			}
			if ($selectedaudiencebox eq "school")
			{
				$VAR{'BASIC-VAL-SEL-AUDIENCE-SCHOOL'} = "checked";
				$audschool = 1;
			}
			if ($selectedaudiencebox eq "undergraduate")
			{
				$VAR{'BASIC-VAL-SEL-AUDIENCE-UNDERGRAD'} = "checked";
				$audundergrad = 1;
			}
			if ($selectedaudiencebox eq "research")
			{
				$VAR{'BASIC-VAL-SEL-AUDIENCE-RESEARCH'} = "checked";
				$audresearch = 1;
			}
			if ($selectedaudiencebox eq "sixteenplus")
			{
				$VAR{'BASIC-VAL-SEL-AUDIENCE-SIXTEENPLUS'} = "checked";
				$audsixteenplus = 1;
			}
			if ($selectedaudiencebox eq "professional")
			{
				$VAR{'BASIC-VAL-SEL-AUDIENCE-PROFESSIONAL'} = "checked";
				$audprofessional = 1;
			}
		}

		# If form valid add/update DB and do not display add/update form
		if ($validform == 1)
		{
			$displayaddeditform = 0;

			# Add/Update the Basic Collection fields in the Database
			
			# Prepare all submitted form values for database
			my ($title, $description, $physchar, $size) = 
					("", "", "", "");

			$title = $FORM{'title'};
			$description = $FORM{'description'};
			$physchar = $FORM{'physchar'};
			$size = $FORM{'size'};

			# Update
			my($sthAddUpd);
			if ($action eq "Update")
			{
				$sthAddUpd = $dbh->prepare("update Collection set title=?, description=?, physicalCharacteristics=?, size=? where id=$FORM{'recordID'}");
			}
			# Add
			else
			{
				$sthAddUpd = $dbh->prepare("insert into Collection(title,description,physicalCharacteristics,size) values(?,?,?,?)");
			}
			
			if ($sthAddUpd == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");	
			}
			
			my ($rows) = 0;
			if ($dberror == 0)
			{
				$rows = $sthAddUpd->execute($title, $description, 
											$physchar, $size) or 
						(($dberror = 1) && &dbdie("$DBI::errstr"));
				if ($rows <= 0)
				{
					$rows = 0;
				}
			}
			
			# Set output result row template
			if ($dberror == 0)
			{
				if ($action eq "Update")
				{
					$VAR{'ADD-UPD-DEL-RESULT'} = qq(Updated $rows row(s) from Collection<br />);
				}
				else
				{
					$VAR{'ADD-UPD-DEL-RESULT'} = qq(Added $rows row to Collection<br />);
				}
			}

			# If we just add a new Collection, then we need to get the RecordID
			# to allow us to add audience and collection.
			if ($action eq "Add" && $dberror == 0)
			{
				my($sthGetNewID) = 
					$dbh->prepare('select last_insert_id() from Collection');

				if ($sthGetNewID == 0)
				{
					$dberror = 1;
					&dbdie("$DBI:errstr");
				}
				else
				{
					$sthGetNewID->execute() or
						(($dberror = 1) && &dbdie("$DBI::errstr"));
				}

				if ($dberror == 0)
				{
					if (my($newrecid) = $sthGetNewID->fetchrow_array())
					{
						$FORM{'recordID'} = $newrecid;
					}
					else
					{
						$dberror = 1;
						&dbdie("Internal Error: Just added a new row, but can't get the record ID from the database!");
					}
				}
			}

			# Add/Update the Audience DB fields now
			
			# Forget add/edit here. Just exeucte a query to find out if there
			# is a collection-audience entry for this collection.
			# if not then set insert query, if so set update query and execute.
			my($sthGetAudiences) = $dbh->prepare("select collectionid from CollectionAudience where collectionid=$FORM{'recordID'}");

			if ($sthGetAudiences == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");
			}

			if ($dberror == 0)
			{
				$sthGetAudiences->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}

			my($actualaction) = "";
			if ($dberror == 0)
			{
				if (my($testid) = $sthGetAudiences->fetchrow_array())
				{
					# record exists so prepare update statement
					$actualaction = "Update";
					$sthAddUpd = $dbh->prepare("update CollectionAudience set generalinterest=?, schoolvisit=?, undergrad=?, research=?, sixteenplus=?, professional=? where collectionid=$FORM{'recordID'}");
				}
				else
				{
					# record not exist so prepare insert statement
					$actualaction = "Add";
					$sthAddUpd = $dbh->prepare("insert into CollectionAudience(collectionid, generalinterest, schoolvisit, undergrad, research, sixteenplus, professional) values($FORM{'recordID'}, ?,?,?,?,?,?)");
				}
			}

			if ($sthAddUpd == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");	
			}
			
			$rows = 0;
			if ($dberror == 0)
			{
				$rows = $sthAddUpd->execute($audgeneral, $audschool,
										    $audundergrad, $audresearch,
										    $audsixteenplus, $audprofessional)
						or (($dberror = 1) && &dbdie("$DBI::errstr"));
				if ($rows <= 0)
				{
					$rows = 0;
				}
			}

			if ($dberror == 0)
			{
				if ($actualaction eq "Update")
				{
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(Updated $rows row(s) from CollectionAudience<br />);
				}
				else
				{
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(Added $rows row to CollectionAudience<br />);
				}
			}

			# Add/Update the Concepts associated with this collection in the DB

			# ADD all the selected concept entries from list box,
			# which are not already in the database
			my(%selected);
			my($totaladded) = 0;
			foreach my $selectedconceptid ($cgi->param(concepts))
			{
				# Make into hash to save time later
				$selected{$selectedconceptid} = 1;
		
				# Add if selected does not exist in DB
				if (!exists($dbconcepts{$selectedconceptid}))
				
				{
					my ($sthAdd) = 
					$dbh->prepare("insert into CollectionConcept values(?,?)");

					if ($sthAdd == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr");
					}
					else
					{
						$totaladded += $sthAdd->execute($FORM{'recordID'},
													    $selectedconceptid) or
								 (($dberror = 1) and &dbdie("$DBI::errstr"));
					}
				}
			}
	
			# Set the details of added records in the templates
			$VAR{'ADD-UPD-DEL-RESULT'} .= 
				qq(Added $totaladded row(s) to CollectionConcept<br />);

	
			# Delete all Concept entries that are in DB, but not selected
			# from list in previous call of form
			my ($totaldeleted) = 0;
			foreach my $dbconceptid (keys %dbconcepts)
			{
				if (!exists($selected{$dbconceptid}))
				{
					my ($sthDel) = $dbh->prepare("delete from CollectionConcept where collection=$FORM{'recordID'} and concept=$dbconceptid");
			
					if ($sthDel == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr");
					}
					else
					{
						$totaldeleted += $sthDel->execute() or
								 (($dberror = 1) and &dbdie("$DBI::errstr"));
					}
				}
			}
	
			# Set the details of deleted records in the templates
			$VAR{'ADD-UPD-DEL-RESULT'} .= 
				qq(Deleted $totaldeleted row(s) from CollectionConcept<br />);

			if ($dberror == 1)
			{
				if ($action eq "Update")
				{
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Error(s) occurred so not all data may have been Updated!</b><br />);
				}
				else
				{
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Error(s) occurred so not all data may have been Added!</b><br />);
				}
			}
			else
			{
				if ($action eq "Update")
				{
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Collection Record was successfully Updated!</b><br />);
				}
				else
				{
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Collection Record was successfully Added!</b><br />);
				}
			}
			
			if ($action eq "Update")
			{
				$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<a href="recordmanager.cgi?recordID=$FORM{'recordID'}#basic">[Return to Collection Record Manager]</a>);
			}
			else
			{
				$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<a href="recordmanager.cgi?recordID=$FORM{'recordID'}">[Proceed to Collection Record Manager]</a>);
			}
			
			# Output results template
			print &parse_template("$CFG{'admin-templates-rel-dir'}/add-upd-del-results-xhtml.template", %VAR);
		}
		
		# Form not valid, so save all fields by setting their values as before
		else
		{
			# Save the text and text area simple fields
			$VAR{'BASIC-VAL-TITLE'} = $FORM{'title'};
			$VAR{'BASIC-VAL-DESCRIPTION'} = $FORM{'description'};
			$VAR{'BASIC-VAL-PHYSCHAR'} = $FORM{'physchar'};
			$VAR{'BASIC-VAL-SIZE'} = $FORM{'size'};

			# Build hash of all the concept ID's that have been select in 
			# the last iteration and selection of form
			my(%selectedconceptids) = ();
			foreach my $selectedconceptid ($cgi->param(concepts))
			{
				$selectedconceptids{$selectedconceptid}=1;
			}

			# Fill in the concept list box to save subject terms
			my($sthGetConcepts) = $dbh->prepare("select id, name from Concept order by sequence");

			if ($sthGetConcepts == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");
			}
			if ($dberror == 0)
			{
				$sthGetConcepts->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}
			if ($dberror == 0)
			{
				my ($numrows) = 0;
				while (my($recid,$rectitle) = 
						$sthGetConcepts->fetchrow_array())
				{
					$numrows++;
					my($conceptoptstr) = qq(<option value="$recid" );

					# See if the record id should be selected
					# by default in the concept list - if it was
					# selected by previous call of form submit
					if (exists($selectedconceptids{$recid}))
					{
						$conceptoptstr .= "selected>";
					}
					else
					{
						$conceptoptstr .= ">";
					}
					$conceptoptstr .= "$rectitle";

					$VAR{'BASIC-OPTIONS-CONCEPTS'} .= 
						qq($conceptoptstr</option>);
				}

				if ($numrows == 0)
				{
					$dberror = 1;
					&dbdie("There are no Concepts loaded into the database! Please reset the database.");
				}
			}
		}
	# end iteration 2
	}
	
	if ($dberror)
	{
		$displayaddeditform = 0;
	}
	
	# Set standard template params and display add/edit form
	if ($displayaddeditform == 1)
	{
		my ($formstr) = "";
		$formstr = $cgi->start_form({action=>'addupdatebasic.cgi', 
									 method=>'post'});

		# Only set the recordID param if we are not Adding a collection
		if (!($action eq "Add"))
		{
			$formstr .= $cgi->hidden({name=>'recordID', 
									  value=>"$FORM{'recordID'}"});
		}

		$formstr .= $cgi->hidden({name=>'action', 
								  value=>"$FORM{'action'}"});
								  
		# Always indicate second iteration of script
		$formstr .= $cgi->hidden({name=>'call', value=>"2"});

		$VAR{'BASIC-FORM-START-HIDDEN'} = $formstr;

		# Load check box text from the config
		$VAR{'BASIC-VAL-AUDIENCE-GENERAL'} = 
			$CFG{'audience-general-tickbox-text'};
		$VAR{'BASIC-VAL-AUDIENCE-SCHOOL'} = 
			$CFG{'audience-school-tickbox-text'};
		$VAR{'BASIC-VAL-AUDIENCE-UNDERGRAD'} = 
			$CFG{'audience-undergrad-tickbox-text'};
		$VAR{'BASIC-VAL-AUDIENCE-RESEARCH'} = 
			$CFG{'audience-research-tickbox-text'};
		$VAR{'BASIC-VAL-AUDIENCE-SIXTEENPLUS'} = 
			$CFG{'audience-sixteen-tickbox-text'};
		$VAR{'BASIC-VAL-AUDIENCE-PROFESSIONAL'} = 
			$CFG{'audience-professional-tickbox-text'};

		# Field help texts
		$VAR{'BASIC-HELP-TITLE'} = $CFG{'help-title'};
		$VAR{'BASIC-HELP-DESCRIPTION'} = $CFG{'help-description'};
		$VAR{'BASIC-HELP-PHYSCHAR'} = $CFG{'help-physical-char'};
		$VAR{'BASIC-HELP-SIZE'} = $CFG{'help-size'};
		$VAR{'BASIC-HELP-AUDIENCE'} = $CFG{'help-audience'};
		$VAR{'BASIC-HELP-CONCEPTS'} = $CFG{'helpconcept'};

		print &parse_template("$CFG{'admin-templates-rel-dir'}/add-update-basic-form-xhtml.template", %VAR);
	}
# end add || update
}

# This element is to be deleted
elsif ($dberror == 0 && $action eq "Delete")
{
	# Do nothing for this script. See addeditdelstrength.cgi for impl.
}

# Disconnect
if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}

# Page Footer
print &parse_template("$CFG{'admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();
