#!/bin/perl -wT

############################################################################
# addupdateperson.cgi
#
# Description
#
# Allows the administrator to set the person information for an administra
# tor, collector, owner and associated with a particular collection.
#
# The Add and Update actions are permitted.
#
# Cannot be deleted without removing the whole collection from the DB, once
# the table is created - it will be just set blank.
#
# Change History
#
# 22/06/05 Ian Bolton
# Created 
#
# 01/07/07 Ian Bolton
# Updated links to record manager for ease of navigation.
#
# $Id: addupdateperson.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# Standard help image used on the form
$VAR{'HELP-IMAGE'} = "$CFG{'image-dir'}/helpQ.gif";

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

# Find out what the previous link navbar should be
my ($prevlink) = "";
if (exists($FORM{'recordID'}))
{
	$prevlink = qq(<a href="searchorbrowse.cgi">Search or Browse</a> &gt;&gt; <a href="recordmanager.cgi?recordID=$FORM{'recordID'}#person">Collection Record Manager</a> &gt;&gt;);
}
else
{
	# Must be creating a new Collection, so set ID to NULL for now
	$FORM{'recordID'} = NULL;
}

# Script allows Add and Update Collection information.
# Delete via another script
my ($action) = "";
if (exists($FORM{'action'}))
{
	if ($FORM{'action'} eq "add")
	{
		$action = "Add";
	}
	elsif ($FORM{'action'} eq "delete")
	{
		$action = "Delete";
	}
	elsif ($FORM{'action'} eq "update")
	{
		$action = "Update";
	}
}

# Either 'administrator, collector, or owner'
my ($personType) = "";
my ($ucfpersonType) = "";
if (exists($FORM{'personType'}))
{
	$personType = $FORM{'personType'};
	$ucfpersonType = ucfirst($personType);
}

# Each script action expects this same script to be called twice
my ($iteration) = 1;
if (exists($FORM{'call'}))
{
	if ($FORM{'call'} == 2)
	{
		$iteration = 2;
	}
}

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; $prevlink $action $ucfpersonType Person Information);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - $action $ucfpersonType Person Information",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});

print &parse_template("$CFG{'admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&dbdie("$DBI::errstr");
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}

# Are we Adding or Updating a record
if ($dberror == 0 && ($action eq "Add" || $action eq "Update"))
{
	my ($displayaddeditform) = 1;
	
	# Set submit button text template
	if ($action eq "Update")
	{
		$VAR{'PERSON-FORM-SUBMIT-VALUE'} = "Update Database";
	}

	else
	{
		$VAR{'PERSON-FORM-SUBMIT-VALUE'} = "Add to Database";
	}
	
	# On first iteration display the add/edit form with or without values
	# set from the database
	if ($iteration == 1)
	{
		# Fill in the listbox values if we are adding or editing
		
		# Need to load add/edit form template values from DB
		if ($action eq "Update" && $dberror == 0)
		{
			
			my ($sthGetPersonInfo) = $dbh->prepare("select name, organisation, role, telnum, faxnum, email, biography from Person where personID=$FORM{'personID'}");
	
			if ($sthGetPersonInfo == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");
			}

			if ($dberror == 0)
			{
				$sthGetPersonInfo->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}
			
			if ($dberror == 0)
			{
				# should be just one row cause personID is primary
				if (my($pername, $perorganisation, $perrole, $pertelnum,
						$perfaxnum, $peremail, $perbiography) = 
						$sthGetPersonInfo->fetchrow_array())
				{
					# Set the form templates
					$VAR{'PERSON-VAL-NAME'} = $pername;
					$VAR{'PERSON-VAL-ORGANISATION'} = $perorganisation;
					$VAR{'PERSON-VAL-ROLE'} = $perrole;
					$VAR{'PERSON-VAL-TELNUM'} = $pertelnum;
					$VAR{'PERSON-VAL-FAXNUM'} = $perfaxnum;
					$VAR{'PERSON-VAL-EMAIL'} = $peremail;
					$VAR{'PERSON-VAL-HISTORY'} = $perbiography;
				}
				else
				{
					$dberror = 1;
					&dbdie("Could not find Person record with ID \'$FORM{'personID'}\'");
				}
			}
			
		# end $update
		}

	# end $iteration1
	}
	
	# On second iteration, validate form values then add/update the DB OR
	# if not valid display the add/edit form with submitted values and red
	# error row to indicate error.
	elsif ($iteration == 2)
	{
		# Validate form fields
		my ($validform) = 1;

		# If email field was submitted, and not blank, then verify
		# it is a valid email address.
		if (exists($FORM{'email'}) && !($FORM{'email'} eq ""))
		{
			if ($FORM{'email'} !~ /^\w+[\.\w\-]+\@[\.\w\-]+\.[\w\-]+$/)
			{
				$validform = 0;
				$VAR{'ERROR-ROW-TEXT'} = $CFG{'error-invalid-email'};
				$VAR{'PERSON-ERRORROW-EMAIL'} = &parse_template("$CFG{'admin-templates-rel-dir'}/error-row-xhtml.template", %VAR);
			}
		}


		# If form valid add/update DB and do not display add/update form
		if ($validform == 1)
		{
			$displayaddeditform = 0;

			# Add/Update the Basic Collection fields in the Database
			
			# Prepare all submitted form values for database
			my ($pername, $perorganisation, $perrole, $pertelnum,
				$perfaxnum, $peremail, $perbiography) = 
					("", "", "", "", "", "", "");

			$pername = $FORM{'name'};
			$perorganisation = $FORM{'organisation'};
			$perrole = $FORM{'role'};
			$pertelnum = $FORM{'telnum'};
			$perfaxnum = $FORM{'faxnum'};
			$peremail = $FORM{'email'};
			$perbiography = $FORM{'history'};

			# Update
			my($sthAddUpd);
			if ($action eq "Update")
			{
				$sthAddUpd = $dbh->prepare("update Person set name=?, organisation=?, role=?, telnum=?, faxnum=?, email=?, biography=? where personID=$FORM{'personID'}");
			}
			# Add
			else
			{
				$sthAddUpd = $dbh->prepare("insert into Person(name, organisation, role, telnum, faxnum, email, biography) values(?,?,?,?,?,?,?)");
			}
			
			if ($sthAddUpd == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");	
			}
			
			my ($rows) = 0;
			if ($dberror == 0)
			{
				$rows = $sthAddUpd->execute($pername, $perorganisation,
											$perrole, $pertelnum,
											$perfaxnum, $peremail, 
											$perbiography) or 
						(($dberror = 1) && &dbdie("$DBI::errstr"));
				if ($rows <= 0)
				{
					$rows = 0;
				}
			}
			
			# Set output result row template
			if ($dberror == 0)
			{
				if ($action eq "Update")
				{
					$VAR{'ADD-UPD-DEL-RESULT'} = qq(Updated $rows row(s) from Person<br />);
				}
				else
				{
					$VAR{'ADD-UPD-DEL-RESULT'} = qq(Added $rows row to Person<br />);
				}
			}

			# If we just add a new Person, then we need to get the auto_inc
			# primary key, which to associated with the collection in
			# CollectionPerson
			if ($action eq "Add" && $dberror == 0)
			{
				my($sthGetNewID) = 
					$dbh->prepare('select last_insert_id() from Person');

				if ($sthGetNewID == 0)
				{
					$dberror = 1;
					&dbdie("$DBI:errstr");
				}
				else
				{
					$sthGetNewID->execute() or
						(($dberror = 1) && &dbdie("$DBI::errstr"));
				}

				if ($dberror == 0)
				{
					if (my($newrecid) = $sthGetNewID->fetchrow_array())
					{
						$FORM{'personID'} = $newrecid;
					}
					else
					{
						$dberror = 1;
						&dbdie("Internal Error: Just added a new row, but can't get the Person ID from the database!");
					}
				}

				# Examine the personType parameter, to decide if this is
				# an admin, collector, or owner type.
				# Then update DB CollectionPerson table, OR insert new
				# if there is not already a record for this collection.
				# Remember, there is only one record for each collection.
				my($collectorperson, $ownerperson, $adminperson) = (0,0,0);
				my($updatelinktable) = 0;
				if ($dberror == 0)
				{
					$sthCheckCollPerson = $dbh->prepare("select collectorperson, ownerperson, adminperson from CollectionPerson where collectionid=$FORM{'recordID'}");

					if ($sthCheckCollPerson == 0)
					{
						$dberror = 1;
						&dbdie("$DBI:errstr");
					}
					else
					{
						$sthCheckCollPerson->execute() or
							(($dberror = 1) && &dbdie("$DBI::errstr"));
					}

					if ($dberror == 0)
					{
						if (($collectorperson, $ownerperson, $adminperson) =
								 $sthCheckCollPerson->fetchrow_array())
						{
							$updatelinktable = 1;
						}

						# If any values are NULL, then set back to zero
						if ($collectorperson == NULL)
						{
							$collectorperson = 0;
						}
						if ($ownerperson == NULL)
						{
							$ownerperson = 0;
						}
						if ($adminperson == NULL)
						{
							$adminperson = 0;
						}
					}
				}

				if ($dberror == 0 && exists($FORM{'personType'}))
				{
					if ($FORM{'personType'} eq "owner")
					{
						$ownerperson = $FORM{'personID'};
					}
					elsif ($FORM{'personType'} eq "administrator")
					{
						$adminperson = $FORM{'personID'};
					}
					elsif ($FORM{'personType'} eq "collector")
					{
						$collectorperson = $FORM{'personID'};
					}
				}

				my($sthAddUpdPerColl);
				if ($updatelinktable == 1)
				{
					$sthAddUpdPerColl = $dbh->prepare("update CollectionPerson set collectorperson=?, ownerperson=?, adminperson=? where collectionid=$FORM{'recordID'}");
				}
				# Add
				else
				{
					$sthAddUpdPerColl = $dbh->prepare("insert into CollectionPerson(collectionid, collectorperson, ownerperson, adminperson) values($FORM{'recordID'},?,?,?)");
				}
			
				if ($sthAddUpdPerColl == 0)
				{
					$dberror = 1;	
					&dbdie("$DBI::errstr");	
				}
			
				$rows = 0;
				if ($dberror == 0)
				{
					$rows = $sthAddUpdPerColl->execute($collectorperson, 
								$ownerperson, $adminperson) or 
									(($dberror = 1) && &dbdie("$DBI::errstr"));
					if ($rows <= 0)
					{
						$rows = 0;
					}
				}
			
				# Set output result row template
				if ($dberror == 0)
				{
					if ($updatelinktable == 1)
					{
						$VAR{'ADD-UPD-DEL-RESULT'} .= qq(Updated $rows row(s) from CollectionPerson<br />);
					}
					else
					{
						$VAR{'ADD-UPD-DEL-RESULT'} .= qq(Added $rows row to CollectionPerson<br />);
					}
				}
			}

			if ($dberror == 1)
			{
				if ($action eq "Update")
				{
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Error(s) occurred so not all data may have been Updated!</b><br />);
				}
				else
				{
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Error(s) occurred so not all data may have been Added!</b><br />);
				}
			}
			else
			{
				if ($action eq "Update")
				{
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Record was successfully Updated!</b><br />);
				}
				else
				{
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Record was successfully Added!</b><br />);
				}
			}
			
			$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<a href="recordmanager.cgi?recordID=$FORM{'recordID'}#person">[Return to Collection Record Manager]</a>);
			
			# Output results template
			print &parse_template("$CFG{'admin-templates-rel-dir'}/add-upd-del-results-xhtml.template", %VAR);
		}
		
		# Form not valid, so save all fields by setting their values as before
		else
		{
			# Save the text and text area fields
			$VAR{'PERSON-VAL-NAME'} = $FORM{'name'};
			$VAR{'PERSON-VAL-ORGANISATION'} = $FORM{'organisation'};
			$VAR{'PERSON-VAL-ROLE'} = $FORM{'role'};
			$VAR{'PERSON-VAL-TELNUM'} = $FORM{'telnum'};
			$VAR{'PERSON-VAL-FAXNUM'} = $FORM{'faxnum'};
			$VAR{'PERSON-VAL-EMAIL'} = $FORM{'email'};
			$VAR{'PERSON-VAL-HISTORY'} = $FORM{'history'};
		}
	# end iteration 2
	}
	
	if ($dberror)
	{
		$displayaddeditform = 0;
	}
	
	# Set standard template params and display add/edit form
	if ($displayaddeditform == 1)
	{
		my ($formstr) = "";
		$formstr = $cgi->start_form({action=>'addupdateperson.cgi', 
									 method=>'post'});

		# Only set the recordID param if we are not Adding a collection
		$formstr .= $cgi->hidden({name=>'recordID', 
								  value=>"$FORM{'recordID'}"});

		$formstr .= $cgi->hidden({name=>'action', 
								  value=>"$FORM{'action'}"});

		$formstr .= $cgi->hidden({name=>'personID', 
								  value=>"$FORM{'personID'}"});

		$formstr .= $cgi->hidden({name=>'personType', 
								  value=>"$FORM{'personType'}"});
								  
		# Always indicate second iteration of script
		$formstr .= $cgi->hidden({name=>'call', value=>"2"});

		$VAR{'PERSON-FORM-START-HIDDEN'} = $formstr;

		# Field help texts
		$VAR{'PERSON-HELP-NAME'} = $CFG{'help-person-name'};
		$VAR{'PERSON-HELP-ORGANISATION'} = $CFG{'help-person-organisation'};
		$VAR{'PERSON-HELP-ROLE'} = $CFG{'help-person-role'};
		$VAR{'PERSON-HELP-TELNUM'} = $CFG{'help-person-telnum'};
		$VAR{'PERSON-HELP-FAXNUM'} = $CFG{'help-person-faxnum'};
		$VAR{'PERSON-HELP-EMAIL'} = $CFG{'help-person-email'};
		$VAR{'PERSON-HELP-HISTORY'} = $CFG{'help-person-history'};

		print &parse_template("$CFG{'admin-templates-rel-dir'}/add-update-person-form-xhtml.template", %VAR);
	}
# end add || update
}

# This element is to be deleted
elsif ($dberror == 0 && $action eq "Delete")
{
	# Do nothing for this script. See addeditdelstrength.cgi for impl.
}

# Disconnect
if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}

# Page Footer
print &parse_template("$CFG{'admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();
