#!/bin/perl -wT

############################################################################
# adminperformsearch.cgi
#
# Description
#
# Allows the administrator to search through all the collections in the DB.
#
# Change History
#
# 29/06/05 Ian Bolton
# Created 
#
# $Id: adminperformsearch.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

# Need to use the POSIX ceiling function in this script
use POSIX qw(ceil);

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; <a href="searchorbrowse.cgi">Search or Browse</a> &gt;&gt; <a href="adminsearch.cgi">Admin Search</a> &gt;&gt; Search Results);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - Search Results",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});


print &parse_template("$CFG{'admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

# Two major actions of this script. 'search' for or 'retrieve' results.
my ($action) = "";
if (exists($FORM{'action'}))
{
	if ($FORM{'action'} eq "search")
	{
		$action = "Search";
	}
	elsif($FORM{'action'} eq "retrieve")
	{
		$action = "Retrieve";
	}
}

# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&dbdie("$DBI::errstr");
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}

# Search for results
if ($dberror == 0 && $action eq "Search")
{

	# Find out which search fields were submitted, and add to data structure
	# BASIC OR ADVANCED1
	my(@search_array) = ();
	my(@operation_array) = ();
	if (exists($FORM{'searchfieldselect0'}) && !($FORM{'searchtext0'} eq ""))
	{
		push(@search_array, ($cgi->param('searchfieldselect0'), &escape_search_text($FORM{'searchtext0'}), $FORM{'searchmethodselect0'}));
		push(@operation_array, "none");
	}
	if (exists($FORM{'searchfieldselect1'}) && !($FORM{'searchtext1'} eq ""))
	{
		push(@search_array, ($cgi->param('searchfieldselect1'), &escape_search_text($FORM{'searchtext1'}), $FORM{'searchmethodselect1'}));
		push(@operation_array, $FORM{'andorfield1'});
	}
	if (exists($FORM{'searchfieldselect2'}) && !($FORM{'searchtext2'} eq ""))
	{
		push(@search_array, ($cgi->param('searchfieldselect2'), &escape_search_text($FORM{'searchtext2'}), $FORM{'searchmethodselect2'}));
		push(@operation_array, $FORM{'andorfield2'});
	}
	if (exists($FORM{'searchfieldselect3'}) && !($FORM{'searchtext3'} eq ""))
	{
		push(@search_array, ($cgi->param('searchfieldselect3'), &escape_search_text($FORM{'searchtext3'}), $FORM{'searchmethodselect3'}));
		push(@operation_array, $FORM{'andorfield3'});
	}

	# Look up the search phrases from the advanced form term drop downs
	# ADVANCED2

	if (exists($FORM{'titleselect'}) && !($FORM{'titleselect'} eq "none"))
	{
		if ($FORM{'titleselect'} =~ /^\d+$/)
		{
			my($sthGetTitleByID) = $dbh->prepare("select title from Collection where id=$FORM{'titleselect'}");
			if ($sthGetTitleByID == 0)
			{
				$dberror = 1;
				&dbdie("$DBI::errstr");
			}
			else
			{
				$sthGetTitleByID->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}

			if ($dberror == 0)
			{
				if (my($name) = $sthGetTitleByID->fetchrow_array())
				{
					push(@search_array, ("title", 
						&escape_search_text($name), "phrase"));
					push(@operation_array, "none");
				}
			}
		}
	}

	if (exists($FORM{'conceptselect'}) && !($FORM{'conceptselect'} eq "none"))
	{
		if ($FORM{'conceptselect'} =~ /^\d+$/)
		{
			my($sthGetConceptByID) = $dbh->prepare("select name from Concept where id=$FORM{'conceptselect'}");
			if ($sthGetConceptByID == 0)
			{
				$dberror = 1;
				&dbdie("$DBI::errstr");
			}
			else
			{
				$sthGetConceptByID->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}

			if ($dberror == 0)
			{
				if (my($name) = $sthGetConceptByID->fetchrow_array())
				{
					push(@search_array, ("concept", 
						&escape_search_text($name), "phrase"));
					push(@operation_array, $FORM{'andorconcept'});
				}
			}
		}
	}

	if (exists($FORM{'nameperselect'}) && !($FORM{'nameperselect'} eq "none"))
	{
		if ($FORM{'nameperselect'} =~ /^\d+$/)
		{
			my($sthGetNamePerByID) = $dbh->prepare("select name from NamePersonal where id=$FORM{'nameperselect'}");
			if ($sthGetNamePerByID == 0)
			{
				$dberror = 1;
				&dbdie("$DBI::errstr");
			}
			else
			{
				$sthGetNamePerByID->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}

			if ($dberror == 0)
			{
				if (my($name) = $sthGetNamePerByID->fetchrow_array())
				{
					push(@search_array, ("name", 
						&escape_search_text($name), "phrase"));
					push(@operation_array, $FORM{'andornameper'});
				}
			}
		}
	}

	if (exists($FORM{'namecorselect'}) && !($FORM{'namecorselect'} eq "none"))
	{
		if ($FORM{'namecorselect'} =~ /^\d+$/)
		{
			my($sthGetNameCorByID) = $dbh->prepare("select name from NameCorporate where id=$FORM{'namecorselect'}");
			if ($sthGetNameCorByID == 0)
			{
				$dberror = 1;
				&dbdie("$DBI::errstr");
			}
			else
			{
				$sthGetNameCorByID->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}

			if ($dberror == 0)
			{
				if (my($name) = $sthGetNameCorByID->fetchrow_array())
				{
					push(@search_array, ("name", 
						&escape_search_text($name), "phrase"));
					push(@operation_array, $FORM{'andornamecor'});
				}
			}
		}
	}

	if (exists($FORM{'placeselect'}) && !($FORM{'placeselect'} eq "none"))
	{
		if ($FORM{'placeselect'} =~ /^\d+$/)
		{
			my($sthGetPlaceByID) = $dbh->prepare("select name from Place where id=$FORM{'placeselect'}");
			if ($sthGetPlaceByID == 0)
			{
				$dberror = 1;
				&dbdie("$DBI::errstr");
			}
			else
			{
				$sthGetPlaceByID->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}

			if ($dberror == 0)
			{
				if (my($name) = $sthGetPlaceByID->fetchrow_array())
				{
					push(@search_array, ("place", 
						&escape_search_text($name), "phrase"));
					push(@operation_array, $FORM{'andorplace'});
				}
			}
		}
	}

	if (exists($FORM{'timeselect'}) && !($FORM{'timeselect'} eq "none"))
	{
		if ($FORM{'timeselect'} =~ /^\d+$/)
		{
			my($sthGetTimeByID) = $dbh->prepare("select name from Time where id=$FORM{'timeselect'}");
			if ($sthGetTimeByID == 0)
			{
				$dberror = 1;
				&dbdie("$DBI::errstr");
			}
			else
			{
				$sthGetTimeByID->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}

			if ($dberror == 0)
			{
				if (my($name) = $sthGetTimeByID->fetchrow_array())
				{
					push(@search_array, ("time", 
						&escape_search_text($name), "phrase"));
					push(@operation_array, $FORM{'andortime'});
				}
			}
		}
	}
				
	# Prepare statement to search collections
	my($sthCollection) = $dbh->prepare("select id, title, description from Collection");
	
	if ($sthCollection == 0)
	{
		$dberror = 1;
		&dbdie("$DBI:errstr");
	}
	else
	{
		$sthCollection->execute() or 
			(&dbdie("$DBI::errstr") and ($dberror = 1));
	}

	# Build a strength rating for every collection
	my(%strengthrating) = ();
	my(%highlighttitle) = ();
	my(%highlightdescription) = ();
	my($atLeastOneResult) = 0;
	while ( ($dberror == 0) && (my($recid, $rectitle, $recdescription) =
								$sthCollection->fetchrow_array()) )
	{
		# Zero the hash ID rating for this record
		$strengthratingbyid{$recid} = 0;

		# For each item in the array of search fields build part strength
		# and add to the array
		my(@strengthfield) = ();
		for (my $i = 0; $i < ($#search_array+1); $i=$i+3)
		{
			my($fieldtype) = $search_array[$i];
			my($searchtext) = $search_array[$i+1];
			my($method) = $search_array[$i+2];

			# Initialise the strength rating for this field
			my($fieldstrengthrating) = 0;

			# Build an array of keywords based on search method
			my(@keywords) = ();
			if ($method eq "phrase")
			{
				# For a phrase, the whole text becomes single keyword!
				push(@keywords, $searchtext);
			}
			elsif ($method eq "wordsAND" || $method eq "wordsOR")
			{
				# Split into individual words by space
				push(@keywords, split(/ /, $searchtext));
			}
			else
			{
				$dberror = 1;
				&dienice("Did not recognise search method \'$method\' passed by script!");
				last;
			}

			# Search the Collection ID (Admin only)
			if ($fieldtype eq "id")
			{
				# Expand keywords that have number ranges
				my(@newkeywords) = ();
				foreach my $idtext (@keywords)
				{
					if ($idtext =~ /^(\d+)-(\d+)$/)
					{
						my($startrange, $endrange) = ($1, $2);
						for(my($range) = $startrange; ($range <= $endrange); $range++)
						{
							push(@newkeywords, $range);
						}
					}
					else
					{
						push(@newkeywords, $idtext);
					}
				}

				$fieldstrengthrating = 
					&perform_keyword_search("$recid", $method, \@newkeywords);
			}

			# Search the Collection title
			elsif ($fieldtype eq "title")
			{
				$fieldstrengthrating = 
					&perform_keyword_search($rectitle, $method, \@keywords);
				if ($fieldstrengthrating > 0)
				{
					$highlighttitle{$recid} = 
						&highlight_keyword($rectitle, \@keywords);
				}
			}

			# Search the Collection description
			elsif($fieldtype eq "description")
			{
				$fieldstrengthrating = 
					&perform_keyword_search($recdescription, 
											$method, \@keywords);
				if ($fieldstrengthrating > 0)
				{
					$highlightdescription{$recid} = 
						&highlight_keyword($recdescription, \@keywords);
				}
			}

			# Search the Concepts for this collection
			elsif($fieldtype eq "concept")
			{
				my($sthGetConcepts) = $dbh->prepare("select Concept.name from Concept, CollectionConcept where Concept.id=CollectionConcept.concept and CollectionConcept.collection=$recid");

				if ($sthGetConcepts == 0)
				{
					$dberror = 1;
					&dbdie("$DBI:errstr");
				}
				else
				{
					$sthGetConcepts->execute() or 
						(&dbdie("$DBI::errstr") and ($dberror = 1));
				}

				if ($dberror == 0)
				{
					# Treat repeatable fields as just one long field, so add
					# to rating
					while (my($fieldtext) = $sthGetConcepts->fetchrow_array())
					{
						$fieldstrengthrating += 
								&perform_keyword_search($fieldtext, 
														$method, \@keywords);
					}
				}
			}

			# Search the Objects for this collection
			elsif($fieldtype eq "object")
			{
				my($sthGetObjects) = $dbh->prepare("select CollectionObject.object from CollectionObject where CollectionObject.collection=$recid");

				if ($sthGetObjects == 0)
				{
					$dberror = 1;
					&dbdie("$DBI:errstr");
				}
				else
				{
					$sthGetObjects->execute() or 
						(&dbdie("$DBI::errstr") and ($dberror = 1));
				}

				if ($dberror == 0)
				{
					# Treat repeatable fields as just one long field, so add
					# to rating
					while (my($fieldtext) = $sthGetObjects->fetchrow_array())
					{
						$fieldstrengthrating += 
								&perform_keyword_search($fieldtext, 
														$method, \@keywords);
					}
				}
			}

			# Search the Names for this collection
			elsif($fieldtype eq "name")
			{
				my($sthGetNamesPer) = $dbh->prepare("select NamePersonal.name from NamePersonal, CollectionName where NamePersonal.id=CollectionName.namepersonal and CollectionName.collection=$recid");
				my($sthGetNamesCor) = $dbh->prepare("select NameCorporate.name from NameCorporate, CollectionName where NameCorporate.id=CollectionName.namecorporate and CollectionName.collection=$recid");

				if ($sthGetNamesPer == 0 || $sthGetNamesCor == 0)
				{
					$dberror = 1;
					&dbdie("$DBI:errstr");
				}
				else
				{
					($sthGetNamesPer->execute() and $sthGetNamesCor->execute())
					or (&dbdie("$DBI::errstr") and ($dberror = 1));
				}

				if ($dberror == 0)
				{
					# Treat repeatable fields as just one long field, so add
					# to rating
					while (my($fieldtext) = $sthGetNamesPer->fetchrow_array())
					{
						$fieldstrengthrating += 
								&perform_keyword_search($fieldtext, 
														$method, \@keywords);
					}
					while (my($fieldtext) = $sthGetNamesCor->fetchrow_array())
					{
						$fieldstrengthrating += 
								&perform_keyword_search($fieldtext, 
														$method, \@keywords);
					}
				}
			}

			# Search the Places for this collection
			elsif($fieldtype eq "place")
			{
				my($sthGetPlace) = $dbh->prepare("select Place.name from Place, CollectionPlace where Place.id=CollectionPlace.place and CollectionPlace.collection=$recid");

				if ($sthGetPlace == 0)
				{
					$dberror = 1;
					&dbdie("$DBI:errstr");
				}
				else
				{
					$sthGetPlace->execute() or 
						(&dbdie("$DBI::errstr") and ($dberror = 1));
				}

				if ($dberror == 0)
				{
					# Treat repeatable fields as just one long field, so add
					# to rating
					while (my($fieldtext) = $sthGetPlace->fetchrow_array())
					{
						$fieldstrengthrating += 
								&perform_keyword_search($fieldtext, 
														$method, \@keywords);
					}
				}
			}

			# Search the Times for this collection
			elsif($fieldtype eq "time")
			{
				my($sthGetTimes) = $dbh->prepare("select Time.name from Time, CollectionTime where Time.id=CollectionTime.time and CollectionTime.collection=$recid");

				if ($sthGetTimes == 0)
				{
					$dberror = 1;
					&dbdie("$DBI:errstr");
				}
				else
				{
					$sthGetTimes->execute() or 
						(&dbdie("$DBI::errstr") and ($dberror = 1));
				}

				if ($dberror == 0)
				{
					# Treat repeatable fields as just one long field, so add
					# to rating
					while (my($fieldtext) = $sthGetTimes->fetchrow_array())
					{
						$fieldstrengthrating += 
								&perform_keyword_search($fieldtext, 
														$method, \@keywords);
					}
				}
			}
			
			# UNKNOWN field type: error
			else
			{
				$dberror = 1;
				&dienice("Unknown field type \'$fieldtype\' passed into script");
				last;
			}

			# Push the strength rating for this particular field to the array
			push(@strengthfield, $fieldstrengthrating);
		}

		# Add up the strengths, and apply the boolean field ops to 
		# get overall rating
		my($store) = 0;
		if ($dberror == 0)
		{
			for (my $fieldindex = 0; $fieldindex < ($#strengthfield + 1); $fieldindex++)
			{
				$strengthratingbyid{$recid} += $strengthfield[$fieldindex];
				if ($fieldindex == 0)
				{
					# Set the store flag to TRUE or FALSE depending on the
					# value of the first field (which has no op)
					$store = $strengthfield[0];
				}
				else
				{
					# Apply the BOOLEAN operation between field to the flag
					if ($operation_array[$fieldindex] eq "AND")
					{
						$store = $store && $strengthfield[$fieldindex];
					}
					elsif ($operation_array[$fieldindex] eq "OR")
					{
						$store = $store || $strengthfield[$fieldindex];
					}
					else
					{
						$dberror = 1;
						&dienice("Unknow Boolean operation \'$operation_array[$fieldindex]\' passed into script!");
						last;
					}
				}
			}
		}

		# If this record did not match search criteria then delete all hash
		# elements that are related to it
		if ($dberror == 0 && $store != 0)
		{
			$atLeastOneResult = 1;
		}
		else
		{
			delete $strengthratingbyid{$recid};
			if (exists($highlightdescription{$recid}))
			{
				delete $highlightdescription{$recid};
			}
			if (exists($highlighttitle{$recid}))
			{
				delete $highlighttitle{$recid};
			}
		}
	}

	# If there was at least one result, then create the search record
	# container, and retrieve the primary searchID
	if ($atLeastOneResult == 1 && $dberror == 0)
	{
		my($sthInsertSearchContainer) = $dbh->prepare("insert into Search(generationtime, searchtype) values(?,?)");
		my($sthGetSearchID) = $dbh->prepare('select last_insert_id() from Search');
		
		if ($sthInsertSearchContainer == 0 || $sthGetSearchID == 0)
		{
			$dberror = 1;
			&dbdie("$DBI:errstr");
		}
		else
		{
			my($searchtypestr) = "";
			if ($FORM{'searchtype'} eq "basic")
			{
				$searchtypestr = 'BASIC';
			}
			else
			{
				$searchtypestr = 'ADVANCED';
			}

			$sthInsertSearchContainer->execute(time, $searchtypestr) or 
				(&dbdie("$DBI::errstr") and ($dberror = 1));

			$sthGetSearchID->execute() or 
				(&dbdie("$DBI::errstr") and ($dberror = 1));
		}

		if ($dberror == 0)
		{
			if (!($FORM{'searchID'} = $sthGetSearchID->fetchrow_array()))
			{
				$dberror = 1;
				&dbdie("Could not retrieve searchID from the database!");
			}
		}
	}

	# For each record that matched the search criteria, then 
	# add its rating to the database for retrieval later
	if ($atLeastOneResult == 1 && $dberror == 0)
	{

		my($sthInsertSearchRes) = $dbh->prepare("insert into SearchResult values($FORM{'searchID'}, ?, ?, ?, ?)");

		if ($sthInsertSearchRes == 0)
		{
			$dberror = 1;
			&dbdie("$DBI:errstr");
		}

		if ($dberror == 0)
		{
			foreach my $match (keys %strengthratingbyid)
			{

				# print "$match: $strengthratingbyid{$match}<br />";

				my($highlighttitleinsertstr) = "";
				if (exists($highlighttitle{$match}))
				{
					$highlighttitleinsertstr = $highlighttitle{$match};
				}
				my($highlightdescinsertstr) = "";
				if (exists($highlightdescription{$match}))
				{
					$highlightdescinsertstr = $highlightdescription{$match};
				}

				$sthInsertSearchRes->execute($match, 
												$strengthratingbyid{$match},
												$highlighttitleinsertstr, 
												$highlightdescinsertstr) or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));

				if ($dberror == 1)
				{
					last;
				}
			}
		}
	}

	# If no results then set searchID to invalid value, just so no
	# rows are retrieved, and the no records found error is displayed
	if ($atLeastOneResult == 0)
	{
		$FORM{'searchID'} = 0;
	}

	# Next action is to retrive and display results from DB
	if ($dberror == 0)
	{
		$action = "Retrieve";
		$FORM{'page'} = 1;
	}
}

# Retrieve results from the database
if ($dberror == 0 && $action eq "Retrieve" && exists($FORM{'searchID'}))
{
	my($sthGetNumResults) = $dbh->prepare("select count(*) from SearchResult where searchid=$FORM{'searchID'}");
	my($sthGetSearchResults) = $dbh->prepare("select Collection.id, Collection.title, Collection.description, SearchResult.hightitle, SearchResult.highdescription from Collection, SearchResult where Collection.id=SearchResult.collectionid and SearchResult.searchid=$FORM{'searchID'} order by SearchResult.strength desc, Collection.title");

	if ($sthGetSearchResults == 0 || $sthGetNumResults == 0)
	{
		$dberror = 1;
		&dbdie("$DBI:errstr");
	}

	if ($dberror == 0)
	{
		($sthGetSearchResults->execute() and $sthGetNumResults->execute()) or 
			(&dbdie("$DBI::errstr") and ($dberror = 1));
	}

	# Get the number of results
	my($numresults) = 0;
	if ($dberror == 0)
	{
		if (!($numresults = $sthGetNumResults->fetchrow_array()))
		{
			$numresults = 0;
		}
	}

	my($displayresults) = 1;
	if ($dberror == 0)
	{

		# Determine which results to show
		if ($numresults == 0)
		{
			$VAR{'BROWSE-RESULTS'} = qq(<em>No search results were found in the database.</em><br />);
			$VAR{'BROWSE-RESULTS'} .= qq(<br /><a href="adminsearch.cgi">[Return to Search Form]</a><br />);
		}
		else
		{
			# get # of results per page, and page number
			my ($numperpage);
			if (exists($FORM{'numresultsperpage'}) && $FORM{'numresultsperpage'} =~ /^\d+$/)
			{
				$numperpage = $FORM{'numresultsperpage'};
			}
			else
			{
				$numperpage = $CFG{'num-search-results-per-page'};
			}
			if (!exists($FORM{'page'}) || $FORM{'page'} !~ /^\d+$/)
			{
				$FORM{'page'} = 1;
			}

			if ($numperpage > 0)
			{
				my ($totalpages);
				my ($startindex);
				my ($endindex);
		
				$totalpages = $numresults / $numperpage;
				$totalpages = POSIX::ceil($totalpages);

				# Ensure page parameter does not exceed total pages
				if ($FORM{'page'} > $totalpages)
				{
					$FORM{'page'} = $totalpages;
				}
			
				# For safety, ensure page is not negative
				if ($FORM{'page'} < 1)
				{
					$FORM{'page'} = 1;
				}

				# Get row start and end indexes for this page
				$startindex = $numperpage * ($FORM{'page'} - 1);
				$endindex = $startindex + $numperpage;
			
				if ($endindex > $numresults)
				{
					$endindex = $numresults;
				}
			
				$VAR{'BROWSE-RESULT-ROWS'} = "";
				my($rowindex) = 0;
				while (my($resid, $restitle1, $resdescription1, 
							$restitle2, $resdescription2) = 
								$sthGetSearchResults->fetchrow_array())
				{
					# See if this is last record to display
					if ($rowindex == $endindex)
					{
						last;
					}

					# Find first record to display on this page
					if ($rowindex++ < $startindex)
					{
						next;
					}

					# Get the highlighted version of the title and description
					# if exists
					my($chosentitle) = "";
					if ($restitle2 eq "")
					{
						$chosentitle = "$restitle1";
					}
					else
					{
						$chosentitle = "$restitle2";
					}
					my($chosendescription) = "";
					if ($resdescription2 eq "")
					{
						$chosendescription = "$resdescription1";
					}
					else
					{
						$chosendescription = "$resdescription2";
					}
					$VAR{'BROWSE-RESULT-ROW-ID'} = "$resid";
					$VAR{'BROWSE-RESULT-ROW-TITLE'} = "$chosentitle";
					$VAR{'BROWSE-RESULT-ROW-UPDATE-LINK'} = qq(<a href="recordmanager.cgi?recordID=$resid&searchtype=$FORM{'searchtype'}&page=$FORM{'page'}&searchID=$FORM{'searchID'}&numresultsperpage=$numperpage">[Update]</a>);
					$VAR{'BROWSE-RESULT-ROW-DELETE-LINK'} = qq(<a href="deletecollection.cgi?recordID=$resid&searchtype=$FORM{'searchtype'}&page=$FORM{'page'}&searchID=$FORM{'searchID'}&numresultsperpage=$numperpage">[Delete]</a>);
					$VAR{'BROWSE-RESULT-ROW-DESCRIPTION'} = 
						"$chosendescription";

					$VAR{'BROWSE-RESULT-ROWS'} .= &parse_template("$CFG{'admin-templates-rel-dir'}/browse-content-result-row-xhtml.template", %VAR);
				}

				$VAR{'BROWSE-RESULT-PAGE'} = $FORM{'page'};
				$VAR{'BROWSE-RESULT-TOTAL-PAGE'} = $totalpages;

				# - Link to previous results page			
				my ($prevlink);
				if ($FORM{'page'} > 1)
				{
					my ($newpage) = $FORM{'page'} - 1;
					$prevlink = qq(<a href="adminperformsearch.cgi?searchID=$FORM{'searchID'}&searchtype=$FORM{'searchtype'}&action=retrieve&page=$newpage&numresultsperpage=$numperpage">[Previous]</a>);
				}
				else
				{
					$prevlink = qq([Previous]);
				}
				$VAR{'BROWSE-RESULT-PREV-LINK'} = $prevlink;
			
				# - Link to next results page
				my ($nextlink);
				if ($FORM{'page'} < $totalpages)
				{
					my ($newpage) = $FORM{'page'} + 1;
					$nextlink = qq(<a href="adminperformsearch.cgi?searchID=$FORM{'searchID'}&searchtype=$FORM{'searchtype'}&action=retrieve&page=$newpage&numresultsperpage=$numperpage">[Next]</a>);
				}
				else
				{
					$nextlink = qq([Next]);
				}
				$VAR{'BROWSE-RESULT-NEXT-LINK'} = $nextlink;

				# TODO: parse other templates, such as page number links
				$VAR{'BROWSE-RESULTS'} = &parse_template("$CFG{'admin-templates-rel-dir'}/browse-content-results-xhtml.template", %VAR);
			}
			else
			{
				# Display nothing!
				&dienice("Configuration Error: Cannot have < 1 search result per page!");
				$displayresults = 0;
			}
		}
	}

	if ($displayresults == 1 && $dberror == 0)
	{
		# parse template
		print &parse_template("$CFG{'admin-templates-rel-dir'}/browse-content-xhtml.template", %VAR);
	}
}

# Disconnect
if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}


# Page Footer
print &parse_template("$CFG{'admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();

# Escape all special characters with a backslash
sub escape_search_text()
{
	my($escapestr) = @_;	

	$escapestr =~ s/\\/\\\\/g;
	$escapestr =~ s/\./\\\./g;
	$escapestr =~ s/\+/\\\+/g;
	$escapestr =~ s/\?/\\\?/g;
	$escapestr =~ s/\*/\\\*/g;
	$escapestr =~ s/\^/\\\^/g;
	$escapestr =~ s/\$/\\\$/g;
	$escapestr =~ s/\@/\\\@/g;
	$escapestr =~ s/\(/\\\(/g;
	$escapestr =~ s/\)/\\\)/g;
	$escapestr =~ s/\[/\\\[/g;
	$escapestr =~ s/\]/\\\]/g;
	$escapestr =~ s/\|/\\\|/g;

	return($escapestr);
}

sub perform_keyword_search()
{
	my($searchtext, $method, $keywordref) = @_;
	my($rating) = 0;
	my(@keywords) = @{$keywordref};

	# Apply text search to all keywords
	foreach my $word (@keywords)
	{
		my($occurences) = 0;
		while($searchtext =~ /(\W|^)$word(\W|$)/ig)
		{
    		$occurences++;
		}
		
		if ($occurences == 0 && $method eq "wordsAND")
		{
			# This word must exists according to search criteria,
			# but it does not so this field adds nothing to the
			# accumulated rating
			$rating = 0;
			last;
		}
		else
		{
			# Add 1 for every occurence.
			$rating += $occurences;
		}
	}	
	return($rating);
}

sub highlight_keyword()
{
	my($text, $keywordref) = @_;
	my(@keywords) = @{$keywordref};

	foreach my $word (@keywords)
	{
		$text =~ s/(\W|^)($word)(\W|$)/$1<b>$2<\/b>$3/ig;
	}
	
	return($text);
}
