#!/bin/perl -wT

############################################################################
# adminsearch.cgi
#
# Description
#
# The script to display the advanced admin search form
#
# Change History
#
# 28/05/05 Ian Bolton
# Created
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

# Database
use DBI;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; <a href="searchorbrowse.cgi">Search or Browse</a> &gt;&gt; Admin Search);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - Search",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});


print &parse_template("$CFG{'admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&dbdie("$DBI::errstr");
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}

# Fill in term list boxes
if ($dberror == 0)
{
	# Fill in the Title list box with collection titles
	my($sthGetTitles) = $dbh->prepare("select id, title from Collection order by title");
	if ($sthGetTitles == 0)
	{
		$dberror = 1;	
		&dbdie("$DBI::errstr");
	}
	if ($dberror == 0)
	{
		$sthGetTitles->execute() or 
			(&dbdie("$DBI::errstr") and ($dberror = 1));
	}
	if ($dberror == 0)
	{
		my ($numrows) = 0;
		while (my($recid,$rectitle) = 
			$sthGetTitles->fetchrow_array())
		{
			$numrows++;
			my($conceptoptstr) = qq(<option value="$recid">);

			$conceptoptstr .= "$rectitle";

			$VAR{'ADVANCED-SEARCH-TITLE-OPTIONS'} .= 
				qq($conceptoptstr</option>);
		}
	}

	# Fill in the concept list box with subject terms
	my($sthGetConcepts) = $dbh->prepare("select id, name from Concept order by sequence");
	if ($sthGetConcepts == 0)
	{
		$dberror = 1;	
		&dbdie("$DBI::errstr");
	}
	if ($dberror == 0)
	{
		$sthGetConcepts->execute() or 
			(&dbdie("$DBI::errstr") and ($dberror = 1));
	}
	if ($dberror == 0)
	{
		my ($numrows) = 0;
		while (my($recid,$rectitle) = 
			$sthGetConcepts->fetchrow_array())
		{
			$numrows++;
			my($conceptoptstr) = qq(<option value="$recid">);

			$conceptoptstr .= "$rectitle";

			$VAR{'ADVANCED-SEARCH-CONCEPT-OPTIONS'} .= 
				qq($conceptoptstr</option>);
		}
	}

	# Fill in the Name Personal list box with subject terms
	my($sthGetNamePer) = $dbh->prepare("select id, name from NamePersonal order by sequence");
	if ($sthGetNamePer == 0)
	{
		$dberror = 1;	
		&dbdie("$DBI::errstr");
	}
	if ($dberror == 0)
	{
		$sthGetNamePer->execute() or 
			(&dbdie("$DBI::errstr") and ($dberror = 1));
	}
	if ($dberror == 0)
	{
		my ($numrows) = 0;
		while (my($recid,$rectitle) = 
			$sthGetNamePer->fetchrow_array())
		{
			$numrows++;
			my($conceptoptstr) = qq(<option value="$recid">);

			$conceptoptstr .= "$rectitle";

			$VAR{'ADVANCED-SEARCH-NAME-PER-OPTIONS'} .= 
				qq($conceptoptstr</option>);
		}
	}

	# Fill in the Name Corporate list box with subject terms
	my($sthGetNameCor) = $dbh->prepare("select id, name from NameCorporate order by sequence");
	if ($sthGetNameCor == 0)
	{
		$dberror = 1;	
		&dbdie("$DBI::errstr");
	}
	if ($dberror == 0)
	{
		$sthGetNameCor->execute() or 
			(&dbdie("$DBI::errstr") and ($dberror = 1));
	}
	if ($dberror == 0)
	{
		my ($numrows) = 0;
		while (my($recid,$rectitle) = 
			$sthGetNameCor->fetchrow_array())
		{
			$numrows++;
			my($conceptoptstr) = qq(<option value="$recid">);

			$conceptoptstr .= "$rectitle";

			$VAR{'ADVANCED-SEARCH-NAME-COR-OPTIONS'} .= 
				qq($conceptoptstr</option>);
		}
	}

	# Fill in the place list box with subject terms
	my($sthGetConcepts) = $dbh->prepare("select id, name from Place order by sequence");
	if ($sthGetConcepts == 0)
	{
		$dberror = 1;	
		&dbdie("$DBI::errstr");
	}
	if ($dberror == 0)
	{
		$sthGetConcepts->execute() or 
			(&dbdie("$DBI::errstr") and ($dberror = 1));
	}
	if ($dberror == 0)
	{
		my ($numrows) = 0;
		while (my($recid,$rectitle) = 
			$sthGetConcepts->fetchrow_array())
		{
			$numrows++;
			my($conceptoptstr) = qq(<option value="$recid">);

			$conceptoptstr .= "$rectitle";

			$VAR{'ADVANCED-SEARCH-PLACE-OPTIONS'} .= 
				qq($conceptoptstr</option>);
		}
	}

	# Fill in the time list box with subject terms
	my($sthGetConcepts) = $dbh->prepare("select id, name from Time order by sequence");
	if ($sthGetConcepts == 0)
	{
		$dberror = 1;	
		&dbdie("$DBI::errstr");
	}
	if ($dberror == 0)
	{
		$sthGetConcepts->execute() or 
			(&dbdie("$DBI::errstr") and ($dberror = 1));
	}
	if ($dberror == 0)
	{
		my ($numrows) = 0;
		while (my($recid,$rectitle) = 
			$sthGetConcepts->fetchrow_array())
		{
			$numrows++;
			my($conceptoptstr) = qq(<option value="$recid">);

			$conceptoptstr .= "$rectitle";

			$VAR{'ADVANCED-SEARCH-TIME-OPTIONS'} .= 
				qq($conceptoptstr</option>);
		}
	}
}


# Disconnect
if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}

# Page Content
print &parse_template("$CFG{'admin-templates-rel-dir'}/adminsearchform-xhtml.template", %VAR);


# Page Footer
print &parse_template("$CFG{'admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();
