#!/bin/perl -wT

############################################################################
# browse.cgi
#
# Description
#
# Allows the administrator to browse through all the collections in the DB
# by alphabetical order.
#
# Change History
#
# 08/06/05 Ian Bolton
# Created 
#
# $Id: browse.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

# Need to use the POSIX ceiling function in this script
use POSIX qw(ceil);

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; <a href="searchorbrowse.cgi">Search or Browse</a> &gt;&gt; Browse $FORM{'letter'} Titles);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - Browse $FORM{'letter'} Titles",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});


print &parse_template("$CFG{'admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

my ($validscript) = 0; 

# Check for Valid Input
if (!exists($FORM{'letter'}))
{
	&dienice("Invalid letter parameter passed into Script");
}
elsif (!exists($FORM{'page'}))
{
	&dienice("Invalid result page parameter passed into Script");
}
else
{
	if (!($FORM{'letter'} =~ /^[A-Za-z]$/))
	{
		&dienice("Invalid letter parameter passed into Script");
	}
	elsif (!($FORM{'page'} =~ /^\d+$/))
	{
		&dienice("Invalid result page parameter passed into Script");
	}
	else
	{
		$validscript = 1;
	}
}


# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
if ($validscript == 1)
{
	$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
	if ($dbh == 0)
	{
		&dbdie("$DBI::errstr");
	}
	else
	{
		$dberror = 0;
		$dbconnected = 1;
	}
}

# Prepare statement to get all matching rows, ordered by title
my ($sthBrowse);
if ($dberror == 0)
{
	my($strLikeUC) = uc($FORM{'letter'});
	my($strLikeLC) = lc($FORM{'letter'});
	$sthBrowse = $dbh->prepare("select id, title, description from Collection where title like '$strLikeUC\%' or title like '$strLikeLC\%' order by title");
	
	if ($sthBrowse == 0)
	{
		$dberror = 1;	
		&dbdie("$DBI::errstr");
	}
}

if ($dberror == 0)
{
	$sthBrowse->execute() or 
		(&dbdie("$DBI::errstr") and ($dberror = 1));
}

if ($dberror == 0)
{
	my (@orderedIds);
	my (%rowtitles) = ();
	my (%rowdescs) = ();
	my ($numresults) = 0;
	
	while (my ($rowid, $rowtitle, $rowdesc) = $sthBrowse->fetchrow_array())
	{
		$numresults++;
		push(@orderedIds, $rowid);
		$rowtitles{$rowid} = $rowtitle;
		$rowdescs{$rowid} = $rowdesc;
	}
	
	# Determine which results to show
	if ($numresults == 0)
	{
		$VAR{'BROWSE-RESULTS'} = qq(<em>No records exist with $FORM{'letter'} titles</em>);
	}
	else
	{
		#num-browse-results-per-page
		# get # of results per page, and page number
		my ($numperpage) = $CFG{'num-browse-results-per-page'};
		if ($numperpage > 0)
		{
			my ($totalpages);
			my ($startindex);
			my ($endindex);
		
			$totalpages = $numresults / $numperpage;
			$totalpages = POSIX::ceil($totalpages);

			# Ensure page parameter does not exceed total pages
			if ($FORM{'page'} > $totalpages)
			{
				$FORM{'page'} = $totalpages;
			}
			
			# For safety, ensure page is not negative
			if ($FORM{'page'} < 1)
			{
				$FORM{'page'} = 1;
			}

			# Get array (@orderedIds) start and end indexes for this page
			$startindex = $numperpage * ($FORM{'page'} - 1);
			$endindex = $startindex + $numperpage;
			
			if ($endindex > $numresults)
			{
				$endindex = $numresults;
			}
			
			$VAR{'BROWSE-RESULT-ROWS'} = "";
			for (my $resindex = $startindex; $resindex < $endindex; $resindex++)
			{
				my ($recid) = $orderedIds[$resindex];
				$VAR{'BROWSE-RESULT-ROW-ID'} = $recid;
				$VAR{'BROWSE-RESULT-ROW-TITLE'} = $rowtitles{$recid};
				$VAR{'BROWSE-RESULT-ROW-DESCRIPTION'} = $rowdescs{$recid};

				$VAR{'BROWSE-RESULT-ROW-UPDATE-LINK'} = qq(<a href=recordmanager.cgi?recordID=$recid&browse=1&letter=$FORM{'letter'}&page=$FORM{'page'}>[Update]</a>);
				
				$VAR{'BROWSE-RESULT-ROW-DELETE-LINK'} = qq(<a href=deletecollection.cgi?recordID=$recid&browse=1&letter=$FORM{'letter'}&page=$FORM{'page'}>[Delete]</a>);

				$VAR{'BROWSE-RESULT-ROWS'} .= &parse_template("$CFG{'admin-templates-rel-dir'}/browse-content-result-row-xhtml.template", %VAR);
			}

			# Set up other result rows container template variables
			$VAR{'BROWSE-RESULT-PAGE'} = $FORM{'page'};
			$VAR{'BROWSE-RESULT-TOTAL-PAGE'} = $totalpages;

			# - Link to previous results page			
			my ($prevlink);
			if ($FORM{'page'} > 1)
			{
				my ($newpage) = $FORM{'page'} - 1;
				$prevlink = qq(<a href=browse.cgi?letter=$FORM{'letter'}&page=$newpage>[Previous]</a>);
			}
			else
			{
				$prevlink = qq([Previous]);
			}
			$VAR{'BROWSE-RESULT-PREV-LINK'} = $prevlink;
			
			# - Link to next results page
			my ($nextlink);
			if ($FORM{'page'} < $totalpages)
			{
				my ($newpage) = $FORM{'page'} + 1;
				$nextlink = qq(<a href=browse.cgi?letter=$FORM{'letter'}&page=$newpage>[Next]</a>);
			}
			else
			{
				$nextlink = qq([Next]);
			}
			$VAR{'BROWSE-RESULT-NEXT-LINK'} = $nextlink;

			$VAR{'BROWSE-RESULTS'} = &parse_template("$CFG{'admin-templates-rel-dir'}/browse-content-results-xhtml.template", %VAR);
		
		}
		else
		{
			# Display nothing!
			&dienice("Configuration Error: Cannot have < 1 search result per page!");
			$validscript = 0;
		}
	}
	
#	foreach $item (@orderedIds)
#	{
#		print "Item: $item $rowtitles{$item}<br />";
#	}
}

if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}



if ($dberror == 0 && $validscript == 1)
{
	
	# Generate Alphabetical browse list
	$VAR{'BROWSE-LIST'} = &generate_browse_list('browse.cgi', 1);

	print &parse_template("$CFG{'admin-templates-rel-dir'}/browse-content-xhtml.template", %VAR);
}

# Page Footer
print &parse_template("$CFG{'admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();
