#!/bin/perl -wT

############################################################################
# recordmanager.cgi
#
# Description
#
# Administration interface collection record management script
#
#
# Change History
#
# 27/05/04 Ian Bolton
# Created
#
# 18/06/04 Ian Bolton
# Changed how the Basic Data and Repeatable fields are read and displayed.
#
# 22/06/04 Ian Bolton
# Added Collection Object, and Catalogue handling
#
# 23/06/04 Ian Bolton
# Fixed bugs
# Added management of Collection Location Details
#
# 01/07/04 Ian Bolton
# Made strength list separated by comma rather than space.
# Languages now displayed as list, rather than one line.
# Sub-collections now printed as a bullet list.
# Changed capital letters to lower case on contact info display.
# Name of current record displayed in title.
#
# 08/07/04 Ian Bolton
# The Audience element can now also be 16+ or professional.
#
# 09/07/04 Ian Bolton
# Removed the class=small from the strength terms.
#
# 03/07/04 Ian Bolton
# Associated collection is now a text field rather than a link.
#
# 03/07/04 Ian Bolton Updated the ordering of some fields.
#
# $Id: recordmanager.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Use the Database Independant Lib
use DBI;

# Uncomment at end to show compliance
#use strict;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
%CFG = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
%VAR = &get_template_values(%CFG);

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

# Find out what the previous link navbar should be
my ($prevlink) = "";
if (exists($FORM{'browse'}))
{
	if ($FORM{'browse'} == 1 && exists($FORM{'letter'}) && exists($FORM{'page'}))
	{
		$prevlink = qq(<a href="browse.cgi?letter=$FORM{'letter'}&page=$FORM{'page'}">Browse $FORM{'letter'} Titles</a> &gt;&gt;);
	}
}
elsif (exists($FORM{'searchtype'}) && exists($FORM{'searchID'}) &&
		exists($FORM{'page'}) && exists($FORM{'numresultsperpage'}))
{
	$prevlink = qq(<a href="adminsearch.cgi">Admin Search</a> &gt;&gt; <a href="adminperformsearch.cgi?searchID=$FORM{'searchID'}&action=retrieve&page=$FORM{'page'}&searchtype=$FORM{'searchtype'}&numresultsperpage=$FORM{'numresultsperpage'}">Search Results</a> &gt;&gt;);
}

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; <a href="searchorbrowse.cgi">Search or Browse</a> &gt;&gt; $prevlink Collection Record Manager);

my ($criticalerror) = 0;
my ($dbConnected) = 0;
my ($dbh);

# Connect to MySQL Database
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	$criticalerror = 1;
	&output_header("Error", \%CFG, \%VAR, $cgi);
	&dbdie("$DBI::errstr");
}
else
{
	$dbConnected = 1;
}

# Check Record ID
if (!exists($FORM{'recordID'}))
{
	$criticalerror = 1;
	&output_header("Error", \%CFG, \%VAR, $cgi);
	print $cgi->h2("Error: No Collection Record ID given to script!");
	print $cgi->br();
}
elsif ($FORM{'recordID'} eq "")
{
	$criticalerror = 1;
	&output_header("Error", \%CFG, \%VAR, $cgi);
	print $cgi->h2("Error: A Blank Collection Record ID was given to script!");
	print $cgi->br();
}

# Check if ID passed into the script is valid
if ($criticalerror == 0)
{
	my($sthGetID) = $dbh->prepare("select id, title from Collection where id=?");
	
	if ($sthGetID == 0)
	{
		$criticalerror = 1;
		&output_header("Error", \%CFG, \%VAR, $cgi);
		&dbdie("$DBI::errstr");
	}
		
	if ($criticalerror == 0)
	{
		$sthGetID->execute($FORM{recordID}) or ($criticalerror = 1);

		if ($criticalerror == 0)
		{
			my($id, $titletext);
			if (!(($id, $titletext) = $sthGetID->fetchrow_array()))
			{
				$criticalerror = 1;
				&output_header("Error", \%CFG, \%VAR, $cgi);
				print $cgi->h2("Error: An Invalid Collection Record ID was passed into the Script!");
				print $cgi->br();
			}
			else
			{
				&output_header("$titletext", \%CFG, \%VAR, $cgi);
			}
		}
		else
		{
			&output_header("Error", \%CFG, \%VAR, $cgi);
			&dbdie("$DBI::errstr");
		}
	}	
}

############################################################################
# Output Basic Information
############################################################################
if ($criticalerror == 0)
{
	my ($sthGetBasic) = $dbh->prepare("select title, description, physicalCharacteristics, size from Collection where id=$FORM{recordID}");
	
	if ($sthGetBasic == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetBasic->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in Basic fields templates
	if ($criticalerror == 0)
	{
		my($title, $desc, $physchar, $size) = $sthGetBasic->fetchrow_array();
		

		$VAR{'DB-TITLE'}=$title;
		$VAR{'DB-DESCRIPTION'}=$desc;
		$VAR{'DB-PHYSCHAR'}=$physchar;
		$VAR{'DB-SIZE'}=$size;
		$VAR{'DB-BASIC-UPDATE'}="<a href=addupdatebasic.cgi?action=update&recordID=$FORM{'recordID'}>[Update Basic Details]</a>";
	}
	
	# Get Basic Audience field string
	my ($sthGetAudience) = $dbh->prepare("select generalinterest, schoolvisit, undergrad, research, sixteenplus, professional from CollectionAudience where collectionid=$FORM{'recordID'}");

	if ($sthGetAudience == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}

	if ($criticalerror == 0)
	{
		$sthGetAudience->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}

	if ($criticalerror == 0)
	{
		my($tmpstr) = "";
		if (my($general,$school,$undergrad,$research,
			   $sixteenplus, $professional) = 
				$sthGetAudience->fetchrow_array())
		{

			if ($general == 1)
			{
				$tmpstr .= "$CFG{'audience-general-tickbox-text'}, ";
			}

			if ($school == 1)
			{
				$tmpstr .= "$CFG{'audience-school-tickbox-text'}, ";
			}

			if ($undergrad == 1)
			{
				$tmpstr .= "$CFG{'audience-undergrad-tickbox-text'}, ";
			}

			if ($research == 1)
			{
				$tmpstr .= "$CFG{'audience-research-tickbox-text'}, ";
			}

			if ($sixteenplus == 1)
			{
				$tmpstr .= "$CFG{'audience-sixteen-tickbox-text'}, ";
			}

			if ($professional == 1)
			{
				$tmpstr .= "$CFG{'audience-professional-tickbox-text'}, ";
			}

			# Check there is not a blank string, or print error string
			# because there should be at least one audience selection
			# or DB is in an Invalid state
			if ($tmpstr eq "")
			{
				$tmpstr = qq(<b><em>Error: Audience record exists but no selections have been made. DB in invalid state!</em></b>);
			}
			else
			{
				# remove extra space and comma
				chop($tmpstr); chop($tmpstr);
#				$tmpstr = qq(<span class="small">$tmpstr</span>);
			}
		}
		else
		{
			$tmpstr = qq(<b><em>Error: No Audience record could be retrieved for Collection ID \'$FORM{'recordID'}\'</em></b>);
		}
		$VAR{'DB-AUDIENCE'} = qq($tmpstr)
	}

	# Get Concepts associated with this collection
	my ($sthGetConcepts) = $dbh->prepare("select Concept.name from Concept,CollectionConcept where Concept.id=CollectionConcept.concept and CollectionConcept.collection=$FORM{'recordID'} order by Concept.sequence");

	if ($sthGetConcepts == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}

	if ($criticalerror == 0)
	{
		$sthGetConcepts->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}

	if ($criticalerror == 0)
	{
		my($tmpstr) = "";
		my($numconcepts) = 0;
		while (my($conceptname) = 
				$sthGetConcepts->fetchrow_array())
		{
			$numconcepts++;
			$tmpstr .= "$conceptname, ";
		}

		# Check there is at least one record, else print error
		# as there should be at least one concept associated
		if ($numconcepts == 0 || $tmpstr eq "")
		{
			$tmpstr = qq(<b><em>Error: Could not find any concepts associated with this collection!</em></b>);
		}
		else
		{
			# remove extra space and comma
			chop($tmpstr); chop($tmpstr);
#			$tmpstr = qq(<span class="small">$tmpstr</span>);
		}
		$VAR{'DB-CONCEPTS'} = qq($tmpstr);
	}
}

my ($callresult);

# Output Objects
if ($criticalerror == 0)
{
	my ($addDelEditFilename) = 
		qq(addeditdelobject.cgi?recordID=$FORM{'recordID'});
	my ($sthObject) = $dbh->prepare("select objectid, object from CollectionObject where collection=$FORM{'recordID'} order by object");

	my (@rowdata) = ();

	if ($sthObject == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthObject->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		my ($num) = 0;
		while (my($objectid, $objecttext) =
			$sthObject->fetchrow_array())
		{
			$num++;

			$VAR{'TOP-LEFT'} = "$objecttext";
			
			$VAR{'TOP-RIGHT'}=qq(<a href="$addDelEditFilename&action=update&objectID=$objectid">[Update]</a>/<a href="$addDelEditFilename&action=delete&objectID=$objectid">[Delete]</a>);
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}
		
		if ($num == 0)
		{
			$VAR{'TOP-RIGHT'}="";
			$VAR{'TOP-LEFT'}="<em>No Objects are Associated with this Collection</em>";
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}
		
		$VAR{'BOT-MID'}=
			qq(<a href="$addDelEditFilename&action=add">[Add New Object]</a>);
		push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-bot-xhtml.template", %VAR));
		
		$VAR{'DB-OBJECTS'} = "@rowdata";
	}
}


# Output Names
if ($criticalerror == 0)
{
	my ($sthGetName) = $dbh->prepare("select namepersonal, namecorporate from CollectionName where collection=$FORM{'recordID'}");

	my (@rowdata) = ();

	if ($sthGetName == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetName->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		my ($numnames) = 0;
		while (my($namepersonal, $namecorporate) =
				$sthGetName->fetchrow_array())
		{
			$numnames++;
			$VAR{'TOP-RIGHT'}="";

			# Check for invalid information in the database
			if ($namepersonal != NULL && $namecorporate != NULL)
			{
				$VAR{'TOP-LEFT'} = qq(<b><em>Error: Personal and Corporate Names are set. DB is in invalid state.</em></b>);
			}
			elsif ($namepersonal == NULL && $namecorporate == NULL)
			{
				$VAR{'TOP-LEFT'} = qq(<b><em>Error: Personal and Corporate Names are NULL. DB is in invalid state.</em></b>);
			}

			# If Name(personal) is set, then look up string from
			# drop down list table - and display in column 1
			elsif ($namepersonal != NULL)
			{
				my($sthGetNamePersonal) = $dbh->prepare("select name from NamePersonal where id=$namepersonal");
				if ($sthGetNamePersonal == 0)
				{
					$criticalerror = 1;
					&dbdie("$DBI::errstr");
				}
				else
				{
					$sthGetNamePersonal->execute() or 
						(&dbdie("$DBI::errstr") and ($criticalerror = 1));
				}

				if ($criticalerror == 0)
				{
					if (my($nametext) = $sthGetNamePersonal->fetchrow_array())
					{
						$VAR{'TOP-LEFT'} = "$nametext";
					}
					else
					{
						$VAR{'TOP-LEFT'} = qq(<b><em>Error: Could not get text for Personal Name ID $namepersonal. DB is in invalid state.</em></b>);
					}	
				}
			}


			# If Name(Corporate) is set, then look up string from
			# drop down list table - and display in column 1
			elsif ($namecorporate != NULL)
			{
				my($sthGetNameCorporate) = $dbh->prepare("select name from NameCorporate where id=$namecorporate");
				if ($sthGetNameCorporate == 0)
				{
					$criticalerror = 1;
					&dbdie("$DBI::errstr");
				}
				else
				{
					$sthGetNameCorporate->execute() or 
						(&dbdie("$DBI::errstr") and ($criticalerror = 1));
				}

				if ($criticalerror == 0)
				{
					if (my($nametext) = $sthGetNameCorporate->fetchrow_array())
					{
						$VAR{'TOP-LEFT'} = "$nametext";
					}
					else
					{
						$VAR{'TOP-LEFT'} = qq(<b><em>Error: Could not get text for Corporate Name ID $namecorporate. DB is in invalid state.</em></b>);
					}	
				}
			}

			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}

		if ($numnames == 0)
		{
			$VAR{'TOP-RIGHT'}="";
			$VAR{'TOP-LEFT'} = 
				qq(<em>No Names are Associated with this Collection</em>);
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}
		
		$VAR{'BOT-MID'}=
			qq(<a href="setnames.cgi?recordID=$FORM{'recordID'}">[Select Names]</a>);
		push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-bot-xhtml.template", %VAR));
		
		$VAR{'DB-NAMES'} = "@rowdata";
	}
}

# Output Places
if ($criticalerror == 0)
{
	my ($sthGetPlace) = $dbh->prepare("select Place.name from Place,CollectionPlace where Place.id=CollectionPlace.place and CollectionPlace.collection=$FORM{'recordID'} order by Place.sequence");

	my (@rowdata) = ();

	if ($sthGetPlace == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetPlace->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		my ($numrows) = 0;
		while (my($placename) =
				$sthGetPlace->fetchrow_array())
		{
			$numrows++;
			$VAR{'TOP-RIGHT'}="";
			$VAR{'TOP-LEFT'} = "$placename";
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}

		if ($numrows == 0)
		{
			$VAR{'TOP-RIGHT'}="";
			$VAR{'TOP-LEFT'} = 
				qq(<em>No Places are Associated with this Collection</em>);
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}
		
		$VAR{'BOT-MID'}=
			qq(<a href="setplaces.cgi?recordID=$FORM{'recordID'}">[Select Places]</a>);
		push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-bot-xhtml.template", %VAR));
		
		$VAR{'DB-PLACES'} = "@rowdata";
	}
}

# Output Times
if ($criticalerror == 0)
{
	my ($sthGetTime) = $dbh->prepare("select Time.name from Time,CollectionTime where Time.id=CollectionTime.time and CollectionTime.collection=$FORM{'recordID'} order by Time.sequence");

	my (@rowdata) = ();

	if ($sthGetTime == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetTime->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		my ($numrows) = 0;
		while (my($timename) =
				$sthGetTime->fetchrow_array())
		{
			$numrows++;
			$VAR{'TOP-RIGHT'}="";
			$VAR{'TOP-LEFT'} = "$timename";
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}

		if ($numrows == 0)
		{
			$VAR{'TOP-RIGHT'}="";
			$VAR{'TOP-LEFT'} = 
				qq(<em>No Times are Associated with this Collection</em>);
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}
		
		$VAR{'BOT-MID'}=
			qq(<a href="settimes.cgi?recordID=$FORM{'recordID'}">[Select Times]</a>);
		push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-bot-xhtml.template", %VAR));
		
		$VAR{'DB-TIMES'} = "@rowdata";
	}
}

# Output Collection Strength rows
if ($criticalerror == 0)
{
	my ($addDelEditFilename) = 
		qq(addeditdelstrength.cgi?recordID=$FORM{'recordID'});
	my ($sthStrength) = $dbh->prepare("select strengthid, localinterest, regionalinterest, local, regional, national, international, text from CollectionStrength where collectionid=$FORM{'recordID'} order by text");
	my (@rowdata) = ();

	if ($sthStrength == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthStrength->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		my ($num) = 0;
		while (my($strengthID, $localint, $regionalint, 
					$localsig,$regionalsig,$nationalsig,
					$internationalsig,$valuestr) =
			$sthStrength->fetchrow_array())
		{
			$num++;
			$VAR{'TOP-LEFT'} = '<strong>' . "$valuestr" . '</strong>: ';
			
			# Add to string depending on different strength categories
			my($tmpstr) = "";
			if ($localint == 1) {$tmpstr .= "Local interest, ";}
			if ($regionalint == 1) {$tmpstr .= "Regional interest, ";}
			if ($localsig == 1)	{$tmpstr .= "Local significance, ";}
			if ($regionalsig == 1) {$tmpstr .= "Regional significance, ";}
			if ($nationalsig == 1) {$tmpstr .= "National significance, ";}
			if ($internationalsig == 1) {
				$tmpstr .= "International significance, ";}
			for (my($i) = 0; $i < 2; $i++) { chop($tmpstr); }
			$VAR{'TOP-LEFT'} .= $tmpstr;
			
			$VAR{'TOP-RIGHT'}=qq(<a href="$addDelEditFilename&action=update&strengthID=$strengthID">[Update]</a>/<a href="$addDelEditFilename&action=delete&strengthID=$strengthID">[Delete]</a>);
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}
		
		if ($num == 0)
		{
			$VAR{'TOP-RIGHT'}="";
			$VAR{'TOP-LEFT'}="<em>No Strengths are Associated with this Collection</em>";
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}
		
		$VAR{'BOT-MID'}=
			qq(<a href="$addDelEditFilename&action=add">[Add New Strength]</a>);
		push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-bot-xhtml.template", %VAR));
		
		$VAR{'DB-STRENGTH'} = "@rowdata";
	}
}

# Output Collection Languages
if ($criticalerror == 0)
{
	my ($setFilename) = 
		qq(setlanguages.cgi?recordID=$FORM{'recordID'});
	my ($sthLanguages) = $dbh->prepare("select Language.name, Language.iso639 from Language,CollectionLanguage where Language.id=CollectionLanguage.language and CollectionLanguage.collection=$FORM{'recordID'} order by Language.name");
	my (@rowdata) = ();

	if ($sthLanguages == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthLanguages->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		my ($num) = 0;
		while (my($language, $languagecode) =
			$sthLanguages->fetchrow_array())
		{
			$num++;

			$VAR{'TOP-RIGHT'}="";
			$VAR{'TOP-LEFT'} = "$language ($languagecode)";
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}
		
		if ($num == 0)
		{
			$VAR{'TOP-LEFT'} = qq(<em>No Languages are Associated with this Collection</em>);
			$VAR{'TOP-RIGHT'} = "";
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}
			
		$VAR{'BOT-MID'}=qq(<a href="$setFilename">[Select Languages]</a>);

		# Output results via templates
		$VAR{'DB-LANGUAGES'} = "@rowdata";
		$VAR{'DB-LANGUAGES'} .= &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-bot-xhtml.template", %VAR);
	}
}

# Output Collection Type String
if ($criticalerror == 0)
{
	my ($setFilename) = 
		qq(settypes.cgi?recordID=$FORM{'recordID'});
	my ($sthGetTypeIds) = $dbh->prepare("select type,curatorialenvironment,content,policy from Collection where id=$FORM{'recordID'}");

	if ($sthGetTypeIds == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetTypeIds->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		my ($tmpstr) = "";
	
		if (my($type, $typecuratorialenv, $typecontent, $typepolicy) =
				$sthGetTypeIds->fetchrow_array())
		{
			# Get TypeType string from database if set
			if ($type != NULL)
			{
				my ($sthGetTypeType) = $dbh->prepare("select TypeType.text from TypeType,Collection where Collection.type=TypeType.id and Collection.id=$FORM{'recordID'}");
				
				if ($sthGetTypeType == 0)
				{
					$criticalerror = 1;
					&dbdie("$DBI::errstr");
				}
				else
				{
					$sthGetTypeType->execute() or 
						(&dbdie("$DBI::errstr") and ($criticalerror = 1));
				}
				
				if ($criticalerror == 0)
				{
					if (my($typetext) = $sthGetTypeType->fetchrow_array())
					{
						$tmpstr .= qq($typetext.);
					}
					else
					{
						$criticalerror = 1;
						&dbdie("Could not get TypeType text for ID $type");
					}
				}
			}

			# Get Type Curatorial Environment string from database if set
			if ($typecuratorialenv != NULL)
			{
				my ($sthGetCurEnv) = $dbh->prepare("select TypeCuratorialEnvironment.text from TypeCuratorialEnvironment,Collection where Collection.curatorialEnvironment=TypeCuratorialEnvironment.id and Collection.id=$FORM{'recordID'}");
				
				if ($sthGetCurEnv == 0)
				{
					$criticalerror = 1;
					&dbdie("$DBI::errstr");
				}
				else
				{
					$sthGetCurEnv->execute() or 
						(&dbdie("$DBI::errstr") and ($criticalerror = 1));
				}
				
				if ($criticalerror == 0)
				{
					if (my($typetext) = $sthGetCurEnv->fetchrow_array())
					{
						$tmpstr .= qq($typetext.);
					}
					else
					{
						$criticalerror = 1;
						&dbdie("Could not get TypeCuratorialEnvironment text for ID $typecuratorialenv");
					}
				}
			}

			# Get TypeContent string from database if set
			if ($typecontent != NULL)
			{
				my ($sthGetTypeContent) = $dbh->prepare("select TypeContent.text from TypeContent,Collection where Collection.content=TypeContent.id and Collection.id=$FORM{'recordID'}");
				
				if ($sthGetTypeContent == 0)
				{
					$criticalerror = 1;
					&dbdie("$DBI::errstr");
				}
				else
				{
					$sthGetTypeContent->execute() or 
						(&dbdie("$DBI::errstr") and ($criticalerror = 1));
				}
				
				if ($criticalerror == 0)
				{
					if (my($typetext) = $sthGetTypeContent->fetchrow_array())
					{
						$tmpstr .= qq($typetext.);
					}
					else
					{
						$criticalerror = 1;
						&dbdie("Could not get TypeContent text for ID $typecontent");
					}
				}
			}

			# Get TypePolicy string from database if set
			if ($typepolicy != NULL)
			{
				my ($sthGetTypePolicy) = $dbh->prepare("select TypePolicy.text from TypePolicy,Collection where Collection.policy=TypePolicy.id and Collection.id=$FORM{'recordID'}");
				
				if ($sthGetTypePolicy == 0)
				{
					$criticalerror = 1;
					&dbdie("$DBI::errstr");
				}
				else
				{
					$sthGetTypePolicy->execute() or 
						(&dbdie("$DBI::errstr") and ($criticalerror = 1));
				}
				
				if ($criticalerror == 0)
				{
					if (my($typetext) = $sthGetTypePolicy->fetchrow_array())
					{
						$tmpstr .= qq($typetext.);
					}
					else
					{
						$criticalerror = 1;
						&dbdie("Could not get TypePolicy text for ID $typepolicy");
					}
				}
			}
		}
		else
		{
			$criticalerror = 1;
			&dbdie("Could not get Type information from Collection table");
		}
			

		# if all types were blank, then no type has been assigned
		if ($tmpstr eq "")
		{
			$tmpstr = qq(<em>No Type has been assigned to this Collection</em>);
		}
		else
		{
			# remove last full stop .
			chop($tmpstr);
		}
		
		$VAR{'TOP-LEFT'} = "$tmpstr";
			
		$VAR{'TOP-RIGHT'}=qq(<a href="$setFilename">[Select Collection Type]</a>);
		
		$VAR{'BOT-MID'} = "";

		# Output results via templates
		$VAR{'DB-COLLECTION-TYPE'} = &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR);
		$VAR{'DB-COLLECTION-TYPE'} .= &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-bot-xhtml.template", %VAR);
	}
}

# Output List of sub-collections associated with this collection
if ($criticalerror == 0)
{
	my ($sthGetSubCollections) = $dbh->prepare("select id, title from Collection where superCollection=$FORM{'recordID'} order by title");

	if ($sthGetSubCollections == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetSubCollections->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		my ($num) = 0;
		while (my($subrecid, $subrectitle) =
			$sthGetSubCollections->fetchrow_array())
		{
			$num++;
			if ($num == 1)
			{
				$VAR{'DB-SUB-COLLECTIONS'} = qq(<ul>);
			}
			
			$VAR{'DB-SUB-COLLECTIONS'} .= qq(<li><a href="recordmanager.cgi?recordID=$subrecid">$subrectitle</a></li>);
		}
		
		if ($num == 0)
		{
			$VAR{'TOP-RIGHT'} = "";
			$VAR{'TOP-LEFT'} = qq(<em>This collection contains no Sub-Collections</em>);

			# Output results via templates
			$VAR{'DB-SUB-COLLECTIONS'} = &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR);
		}
		else
		{
			$VAR{'DB-SUB-COLLECTIONS'} .= qq(</ul>);
		}
	}
}

############################################################################
# Output Advanced Information
############################################################################
if ($criticalerror == 0)
{
	my ($sthGetAdv) = $dbh->prepare("select accumulationDateRange, contentsDateRange, superCollection, associatedCollection, associatedPublication, accrualstatus, legalstatus, custodialhistory, accesscontrol, note from Collection where id=$FORM{'recordID'}");
	
	if ($sthGetAdv == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetAdv->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		# should be just one row cause recordID is primary
		if (my($accdaterange, $contentdaterange, $supercoll, $associatedcoll,
			$associatedpub, $accrualstatus, $legalstatus, $custodialhist,
			$accesscontrol, $note) = $sthGetAdv->fetchrow_array())
		{
			$VAR{'DB-ADV-ACC-DATE-RANGE'} = 
				&get_advanced_template($accdaterange, 
					"No accumulation date range is set");

			$VAR{'DB-ADV-CONTENTS-DATE-RANGE'} = 
				&get_advanced_template($contentdaterange, 
					"No contents date range is set");

			$VAR{'DB-ADV-ASSOCIATED-PUBLICATION'} = 
				&get_advanced_template($associatedpub, 
					"No publications are associated with collection");

			$VAR{'DB-ADV-ACCRUAL-STATUS'} = 
				&get_advanced_template($accrualstatus, 
					"No Accrual status is set");

			$VAR{'DB-ADV-LEGAL-STATUS'} = 
				&get_advanced_template($legalstatus, 
					"No Legal status is set");

			$VAR{'DB-ADV-CUSTODIAL-HIST'} = 
				&get_advanced_template($custodialhist, 
					"No custodial history is given");

			$VAR{'DB-ADV-ACCESS-CONTROL'} = 
				&get_advanced_template($accesscontrol, 
					"No access control information is given");

			$VAR{'DB-ADV-NOTE'} = 
				&get_advanced_template($note, 
					"No additional notes are given");

			# Super, associated collection are int ID's
			($VAR{'DB-ADV-SUPER-COLLECTION'}, $criticalerror) =
				&get_advanced_link_template($supercoll,
					"This collection is not part of another collection",
					$dbh);

			$VAR{'DB-ADV-ASSOCIATED-COLLECTION'} =
				&get_advanced_template($associatedcoll,
					"This collection is not associated with another collection");
			
			$VAR{'DB-ADV-UPDATE'} = qq(<a href=updateadvanced.cgi?recordID=$FORM{'recordID'}>[Update Additional Information]</a>);

		}
		else
		{
			$criticalerror = 1;
			&dbdie("Cannot get advanced data using record ID $FORM{'recordID'}");
		}
	}
}

# Output Catalogues
if ($criticalerror == 0)
{
	my ($addDelEditFilename) = 
		qq(addeditdelcatalogue.cgi?recordID=$FORM{'recordID'});
	my ($sthCatalogue) = $dbh->prepare("select catalogueid, catalogue from CollectionCatalogue where collection=$FORM{'recordID'} order by catalogue");

	my (@rowdata) = ();

	if ($sthCatalogue == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthCatalogue->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		my ($num) = 0;
		while (my($catalogueid, $cataloguetext) =
			$sthCatalogue->fetchrow_array())
		{
			$num++;

			$VAR{'TOP-LEFT'} = "$cataloguetext";
			
			$VAR{'TOP-RIGHT'}=qq(<a href="$addDelEditFilename&action=update&catalogueID=$catalogueid">[Update]</a>/<a href="$addDelEditFilename&action=delete&catalogueID=$catalogueid">[Delete]</a>);
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}
		
		if ($num == 0)
		{
			$VAR{'TOP-RIGHT'}="";
			$VAR{'TOP-LEFT'}="<em>No Catalogues are Associated with this Collection</em>";
			push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-xhtml.template", %VAR));
		}
		
		$VAR{'BOT-MID'}=
			qq(<a href="$addDelEditFilename&action=add">[Add New Catalogue]</a>);
		push(@rowdata, &parse_template("$CFG{'admin-templates-rel-dir'}/repeat-row-bot-xhtml.template", %VAR));
		
		$VAR{'DB-CATALOGUES'} = "@rowdata";
	}
}

############################################################################
# Output Collection Location details
############################################################################
if ($criticalerror == 0)
{
	my ($sthGetLocation) = $dbh->prepare("select name, address, postcode, country, url, accessconditions, seealso from CollectionLocation where collectionid=$FORM{'recordID'}");
	
	if ($sthGetLocation == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetLocation->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		# Initialise all element to blank
		my($locname, $locaddress, $locpostcode, $loccountry, $locurl,
			$locaccess, $localso) = ("", "", "", "", "", "", "");

		# should be just one row cause recordID is primary
		($locname, $locaddress, $locpostcode, $loccountry, 
			$locurl, $locaccess, $localso) = $sthGetLocation->fetchrow_array();

		# Note: The location record is handled differently from the Advanced
		# record in the DB. It has a separate record, which may or may not
		# exist. If there are no rows, then hide the fact there is no record
		# - so from the users view it is handled no differently.

		$VAR{'DB-LOCATION-NAME'} = 
			&get_advanced_template($locname, 
				"No location name is given");

		$VAR{'DB-LOCATION-ADDRESS'} = 
			&get_advanced_template($locaddress, 
				"No location address is given");

		$VAR{'DB-LOCATION-POSTCODE'} = 
			&get_advanced_template($locpostcode, 
				"No location postcode is given");

		$VAR{'DB-LOCATION-COUNTRY'} = 
			&get_advanced_template($loccountry, 
				"No location country is given");

		if (!($locurl eq ""))
		{
			$VAR{'DB-LOCATION-LOCATOR'} = qq(<a href="$locurl">$locurl</a>);
		}
		else
		{
			$VAR{'DB-LOCATION-LOCATOR'} = 
				&get_advanced_template($locurl, 
					"No home page URL is given");
		}

		$VAR{'DB-LOCATION-ACCESS-CONDITIONS'} = 
			&get_advanced_template($locaccess, 
				"No access conditions are specified");

		$VAR{'DB-LOCATION-SEE-ALSO'} = 
			&get_advanced_template($localso, 
				"No additional information is given");
			
		$VAR{'DB-LOCATION-UPDATE'} = qq(<a href=updatelocation.cgi?recordID=$FORM{'recordID'}>[Update Location Information]</a>);
	}
}

############################################################################
# Output Person Contact Details
############################################################################

# Find out which person details exist for this collection
my($collectperson, $ownerperson, $adminperson) = (0, 0, 0);

if ($criticalerror == 0)
{
	my ($sthGetPerson) = $dbh->prepare("select collectorperson, ownerperson, adminperson from CollectionPerson where collectionid=$FORM{'recordID'}");
	
	if ($sthGetPerson == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetPerson->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		# should be just one row cause recordID is primary
		($collectperson, $ownerperson, $adminperson) = 
			$sthGetPerson->fetchrow_array();
		if ($collectperson == NULL)
		{
			$collectperson = 0;
		}
		if ($ownerperson == NULL)
		{
			$ownerperson = 0;
		}
		if ($adminperson == NULL)
		{
			$adminperson = 0;
		}
	}
}

# NOTE: There are three possible Person types, Admin, Owner, and collector,
# and each references the same table and template, so build hash of 
# the three templates names, against the person record id as values.
my(%persontype) = ( 'DB-PERSON-COLLECTOR-RECORD'=>"$collectperson",
					'DB-PERSON-OWNER-RECORD'=>"$ownerperson",
					'DB-PERSON-ADMINISTRATOR-RECORD'=>"$adminperson" );

# Also store a hash of the record type, so we know what paramaters to pass
# to the script that will add a new person
my(%personrecordtype) = ( 'DB-PERSON-COLLECTOR-RECORD'=>"collector",
						  'DB-PERSON-OWNER-RECORD'=>"owner",
						  'DB-PERSON-ADMINISTRATOR-RECORD'=>"administrator" );

foreach my $persontemplate (keys %persontype)
{

	if ($criticalerror == 0)
	{
		my($ucfrectype) = ucfirst($personrecordtype{$persontemplate});
		my($linkstr) = qq(Update $ucfrectype Information);

		# Initialise database variables
		my($pername, $perorganisation, $perrole, $pertelnum, $perfaxnum,
			$peremail, $perbiography) = ("", "", "", "", "", "", "");

		# See if a person record exists, for this collection and
		# current type of person (from admin, owner, collector)
		if ($persontype{$persontemplate} != 0)
		{
			my ($sthGetPersonDetails) = $dbh->prepare("select name, organisation, role, telnum, faxnum, email, biography from Person where personid=$persontype{$persontemplate}");

			if ($sthGetPersonDetails == 0)
			{
				$criticalerror = 1;
				&dbdie("$DBI::errstr");
			}
	
			if ($criticalerror == 0)
			{
				$sthGetPersonDetails->execute() or 
					(&dbdie("$DBI::errstr") and ($criticalerror = 1));
			}
	
			# Fill in templates
			if ($criticalerror == 0)
			{

				# should be just one row cause recordID is primary
				if (($pername, $perorganisation, $perrole, $pertelnum, 
						$perfaxnum, $peremail, $perbiography) = 
						$sthGetPersonDetails->fetchrow_array())
				{

					$VAR{'DB-PERSON-UPDATE'} = qq(<a href="addupdateperson.cgi?recordID=$FORM{'recordID'}&action=update&personID=$persontype{$persontemplate}">[$linkstr]</a>);
				}
				else
				{
					&dbdie("Could not find Person ID \'$persontype{$persontemplate}\' for collection \'$FORM{'recordID'}\'. DB invalid state");
					$criticalerror = 1;
				}
			}
		}
		else
		{
			# No person record, so fill template with blank entry
			$VAR{'DB-PERSON-UPDATE'} = qq(<a href="addupdateperson.cgi?recordID=$FORM{'recordID'}&action=add&personType=$personrecordtype{$persontemplate}">[$linkstr]</a>);
		}

		# Whatever happened fill in standard templates and parse form
		$VAR{'DB-PERSON-NAME'} = 
			&get_advanced_template($pername, 
				"No $personrecordtype{$persontemplate} name is given");

		$VAR{'DB-PERSON-ORGANISATION'} = 
			&get_advanced_template($perorganisation, 
				"No organisation is given");

		$VAR{'DB-PERSON-ROLE'} = 
			&get_advanced_template($perrole, 
				"The role of the $personrecordtype{$persontemplate} is not given");

		$VAR{'DB-PERSON-TELNO'} = 
			&get_advanced_template($pertelnum, 
				"No telephone number is given");

		$VAR{'DB-PERSON-FAXNO'} = 
			&get_advanced_template($perfaxnum, 
				"No fax number is given");

		if (!($peremail eq ""))
		{
			$VAR{'DB-PERSON-EMAIL'} = qq(<a href="mailto:$peremail">$peremail</a>);
		}
		else
		{
			$VAR{'DB-PERSON-EMAIL'} = 
				&get_advanced_template($peremail, 
					"No email address is given");
		}

		$VAR{'DB-PERSON-HISTORY'} = 
			&get_advanced_template($perbiography, 
				"The biography of the $personrecordtype{$persontemplate} is not given");

		$VAR{$persontemplate} = &parse_template("$CFG{'admin-templates-rel-dir'}/person-record-xhtml.template", %VAR);
	}
}

# Display page
if ($criticalerror == 0)
{
	print &parse_template("$CFG{'admin-templates-rel-dir'}/recordmanager-xhtml.template", %VAR);
	
}

# Disconnect
if ($dbConnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}


# End Content
#print $cgi->end_table();
	
# Page Footer
print &parse_template("$CFG{'admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();

##################################################################
# HELPER FUNCTIONS
##################################################################
sub get_advanced_template()
{
	my($dbvalue, $nulltext) = @_;
	my($rettext);

	if ($dbvalue eq "")
	{
		$rettext = '<em>' . $nulltext . '</em>';
	}
	else
	{
		$rettext = $dbvalue;
		$rettext =~ s/\n/<br \/>/g;
	}
	
	return($rettext);
}

sub get_advanced_link_template()
{
	my($dbvalue, $nulltext, $dbh) = @_;
	my($rettext, $retresult) = ("", 0);

	if ($dbvalue == NULL)
	{
		$rettext =	qq(<em>$nulltext</em>);
	}
	else
	{
		my($sthGetTitle) = 
			$dbh->prepare("select title from Collection where id=$dbvalue");
		
		if ($sthGetTitle == 0)
		{
			$retresult = 1;
			&dbdie("$DBI::errstr");
		}
		else
		{
			$sthGetTitle->execute() or 
				(&dbdie("$DBI::errstr") and ($retresult = 1));
		}
		if ($retresult == 0)
		{
			if (my($linktitle) = $sthGetTitle->fetchrow_array())
			{
				$rettext = qq(<a href="recordmanager.cgi?recordID=$dbvalue">$linktitle</a>);
			}
			else
			{
				$retresult = 1;
				&dbdie("Advanced Data: Cannot get title for recordID $dbvalue");
			}
		}
	}
	
	return($rettext, $retresult);
}

sub output_header()
{
	my($titletext, $cfgref, $varref, $cgi) = @_;

	# Print Standard Header
	print $cgi->header();
	print $cgi->start_html({-title=>"Admin - Collection Manager - $titletext",
							-style=>"$cfgref->{'admin-stylesheet-rel-path'}"});

	print &parse_template("$cfgref->{'admin-templates-rel-dir'}/mainheader-xhtml.template", %{$varref});
}
