#!/bin/perl -wT

############################################################################
# setlanguages.cgi
#
# Description
#
# Allows the administrator to Associate languages with a Collection
#
# Change History
#
# 17/06/05 Ian Bolton
# Created 
#
# 01/07/07 Ian Bolton
# Updated links to record manager for ease of navigation.
#
# $Id: setlanguages.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# Standard help image used on the form
$VAR{'HELP-IMAGE'} = "$CFG{'image-dir'}/helpQ.gif";

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

# Find out what the previous link navbar should be
my ($prevlink) = "";
if (exists($FORM{'recordID'}))
{
	$prevlink = qq(<a href="recordmanager.cgi?recordID=$FORM{'recordID'}#language">Collection Record Manager</a> &gt;&gt;);
}

# Each script action expects this same script to be called twice
my ($iteration) = 1;
if (exists($FORM{'call'}))
{
	if ($FORM{'call'} == 2)
	{
		$iteration = 2;
	}
}

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; <a href="searchorbrowse.cgi">Search or Browse</a> &gt;&gt; $prevlink Set Collection Languages);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - Set Collection Languages",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});

print &parse_template("$CFG{'admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&dbdie("$DBI::errstr");
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}

# Build hash of which languages are already associated with this collection,
# if any! Alongwith their id's
my(%dblanguage);
if ($dberror == 0)
{
	my($sthGetLanguages) = $dbh->prepare("select Language.name, Language.id from Language,CollectionLanguage where Language.id=CollectionLanguage.language and CollectionLanguage.collection=$FORM{'recordID'}");
	
	if ($sthGetLanguages == 0)
	{
		$dberror = 1;
		&dbdie("$DBI::errstr");
	}
	else
	{
		$sthGetLanguages->execute() or 
			(($dberror=1) and (&dbdie("$DBI::errstr")));
	}
	
	if ($dberror == 0)
	{
	
		while( my($language, $languagecode) = 
			$sthGetLanguages->fetchrow_array())
		{
			$dblanguage{$language} = $languagecode;
		}
	}
}

# Iteration 1, display form
if ($dberror == 0 && $iteration == 1)
{
	# Get ALL language names from the database
	my($sthGetAllLanguages) = 
		$dbh->prepare("select name from Language order by name");
	if ($sthGetAllLanguages == 0)
	{
		$dberror = 1;
		&dbdie("$DBI::errstr");
	}
	else
	{
		$sthGetAllLanguages->execute() or 
			(($dberror=1) and (&dbdie("$DBI::errstr")));
	}

	# Populate language list box
	if ($dberror == 0)
	{
	
		while( my($language) = 
			$sthGetAllLanguages->fetchrow_array())
		{
			# Set option selected if it exists in DB for this record
			my($selected) = "";
			if (exists($dblanguage{$language}))
			{
				$selected = " selected";
			}
			$VAR{'LANGUAGE-SELECT-OPTIONS'} .= 
				qq(<option$selected>$language</option>); 
		}
	}

	if ($dberror == 0)
	{
		$VAR{'LANGUAGE-FORM-START'} = 
			$cgi->start_form({method=>'post', action=>'setlanguages.cgi'});
		$VAR{'LANGUAGE-FORM-HIDDEN'} = 
			$cgi->hidden({name=>'recordID', value=>"$FORM{'recordID'}"});
		$VAR{'LANGUAGE-FORM-HIDDEN'} .=
			$cgi->hidden({name=>'call', value=>"2"});

		print &parse_template("$CFG{'admin-templates-rel-dir'}/set-languages-xhtml.template", %VAR);
	}
}

# Else delete or add languages depending on list box selections
elsif ($dberror == 0 && $iteration == 2)
{
	# Add all the selected language entries from list box,
	# which are not already in the database
	my(%selected);
	my($totaladded) = 0;
	foreach my $selectedlang ($cgi->param(languagelist))
	{
		# Make into hash to save time later
		$selected{$selectedlang} = 1;
		
		# Add if selected does not exist in DB
		if (!exists($dblanguage{$selectedlang}))
		{
			# need to first get the language ID of this unknown langauge
			my ($langid);
			my ($sthGetLangID) = $dbh->prepare("select id from Language where name=\"$selectedlang\"");
			if ($sthGetLangID == 0)
			{
				$dberror = 1;
				&dbdie("$DBI::errstr");
			}
			else
			{
				$sthGetLangID->execute() or 
					(($dberror = 1) and &dbdie("$DBI::errstr"));
			}
			if ($dberror == 0)
			{
				if (!($langid = $sthGetLangID->fetchrow_array()))
				{
					$dberror = 1;
				}
			}
			
			my ($sthAdd) = 
				$dbh->prepare("insert into CollectionLanguage values(?,?)");

			if ($sthAdd == 0)
			{
				$dberror = 1;
				&dbdie("$DBI::errstr");
			}
			else
			{
				$totaladded += $sthAdd->execute($FORM{'recordID'}, $langid) or
								 (($dberror = 1) and &dbdie("$DBI::errstr"));
			}
		}
	}
	
	# Set the details of added records in the templates
	$VAR{'ADD-UPD-DEL-RESULT'} .= 
		qq(Added $totaladded row(s) to CollectionLanguage<br />);
	if ($dberror == 1)
	{
		$VAR{'ADD-UPD-DEL-RESULT'} .= 
			qq(Error(s) occurred so not all data may have been Added!<br />);
	}

	
	# Delete all language entries that are in DB, but not selected
	# from list in previous call of form
	my ($totaldeleted) = 0;
	$dberror = 0;
	foreach my $dblang (keys %dblanguage)
	{
		if (!exists($selected{$dblang}))
		{
			my ($sthDel) = $dbh->prepare("delete from CollectionLanguage where collection=$FORM{'recordID'} and language=$dblanguage{$dblang}");
			
			if ($sthDel == 0)
			{
				$dberror = 1;
				&dbdie("$DBI::errstr");
			}
			else
			{
				$totaldeleted += $sthDel->execute() or
								 (($dberror = 1) and &dbdie("$DBI::errstr"));
			}
		}
	}
	
	# Set the details of deleted records in the templates
	$VAR{'ADD-UPD-DEL-RESULT'} .= 
		qq(Deleted $totaldeleted row(s) from CollectionLanguage<br />);
	if ($dberror == 1)
	{
		$VAR{'ADD-UPD-DEL-RESULT'} .= 
			qq(Error(s) occurred so not all data may have been Deleted!<br />);
	}
	
	$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<a href="recordmanager.cgi?recordID=$FORM{'recordID'}#language">[Return to Collection Record Manager]</a>);
			
	# Output results template
	print &parse_template("$CFG{'admin-templates-rel-dir'}/add-upd-del-results-xhtml.template", %VAR);
}


# Disconnect
if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}

# Page Footer
print &parse_template("$CFG{'admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();
