#!/bin/perl -wT

############################################################################
# settimes.cgi
#
# Description
#
# Allows the administrator to associated Times with a particular 
# collection.
#
# Change History
#
# 21/06/05 Ian Bolton
# Created 
#
# 01/07/07 Ian Bolton
# Updated links to record manager for ease of navigation.
#
# 03/10/04 Ian Bolton
# Updated ordering
#
# $Id: settimes.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# Standard help image used on the form
$VAR{'HELP-IMAGE'} = "$CFG{'image-dir'}/helpQ.gif";

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

# Find out what the previous link navbar should be
my ($prevlink) = "";
if (exists($FORM{'recordID'}))
{
	$prevlink = qq(<a href="searchorbrowse.cgi">Search or Browse</a> &gt;&gt; <a href="recordmanager.cgi?recordID=$FORM{'recordID'}#time">Collection Record Manager</a> &gt;&gt;);
}

# Script just needs to Update list boxes
my ($action) = "Update";
#if (exists($FORM{'action'}))
#{
#	if ($FORM{'action'} eq "add")
#	{
#		$action = "Add";
#	}
#	elsif ($FORM{'action'} eq "delete")
#	{
#		$action = "Delete";
#	}
#	elsif ($FORM{'action'} eq "update")
#	{
#		$action = "Update";
#	}
#}

# Each script action expects this same script to be called twice
my ($iteration) = 1;
if (exists($FORM{'call'}))
{
	if ($FORM{'call'} == 2)
	{
		$iteration = 2;
	}
}

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; $prevlink
Select Times);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - Select Times",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});

print &parse_template("$CFG{'admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&dbdie("$DBI::errstr");
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}

# Build hash of which Times are already 
# associated with this collection, to save time later
my(%dbtimes) = ();
if ($dberror == 0)
{
	my ($sthGetTime) = $dbh->prepare("select Time.id, Time.name from Time,CollectionTime where Time.id=CollectionTime.time and CollectionTime.collection=$FORM{'recordID'}");

	if ($sthGetTime == 0)
	{
		$dberror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($dberror == 0)
	{
		$sthGetTime->execute() or 
			(&dbdie("$DBI::errstr") and ($dberror = 1));
	}
	
	# Get all records associated with this collection
	if ($dberror == 0)
	{
		while (my($timeid, $timename) =
				$sthGetTime->fetchrow_array())
		{
			$dbtimes{$timeid} = $timename;
		}
	}
}

# Are we Adding or Updating a record
if ($dberror == 0 && ($action eq "Add" || $action eq "Update"))
{
	my ($displayaddeditform) = 1;
	
	# Set submit button text template
	if ($action eq "Update")
	{
		$VAR{'TIMES-FORM-SUBMIT-VALUE'} = "Update Database";
	}

#	else
#	{
#		$VAR{'BASIC-FORM-SUBMIT-VALUE'} = "Add to Database";
#	}
	
	# On first iteration display the add/edit form with or without values
	# set from the database
	if ($iteration == 1)
	{
		# Fill in the listbox values

		# Fill in the Time list box with terms
		my($sthGetTimeNames) = $dbh->prepare("select id, name from Time order by sequence");
		if ($sthGetTimeNames == 0)
		{
			$dberror = 1;	
			&dbdie("$DBI::errstr");
		}
		if ($dberror == 0)
		{
			$sthGetTimeNames->execute() or 
				(&dbdie("$DBI::errstr") and ($dberror = 1));
		}
		if ($dberror == 0)
		{
			my ($numrows) = 0;
			while (my($recid,$rectitle) = 
				$sthGetTimeNames->fetchrow_array())
			{
				$numrows++;
				my($optstr) = qq(<option value="$recid" );

				# See if the record id should be selected
				# by default in the concept list
				if (exists($dbtimes{$recid}))
				{
					$optstr .= "selected>";
				}
				else
				{
					$optstr .= ">";
				}
				$optstr .= "$rectitle";

				$VAR{'TIMES-OPTIONS-TIMES'} .= 
					qq($optstr</option>);
			}

			if ($numrows == 0)
			{
				$dberror = 1;
				&dbdie("There are no Time Names loaded into the database! Please reset the database.");
			}
		}


# NO NEED TO LOAD ANY OTHER FORM VARIABLES, CAUSE ONLY CONTAINS LIST BOXES		
		# Need to load add/edit form template values from DB
#		if ($action eq "Update" && $dberror == 0)
#		{
			
#			my ($sthGetBasic) = $dbh->prepare("select title, description, #physicalCharacteristics, size from Collection where id=$FORM{recordID}");
	
#			if ($sthGetBasic == 0)
#			{
#				$dberror = 1;	
#				&dbdie("$DBI::errstr");
#			}
#
#			if ($dberror == 0)
#			{
#				$sthGetBasic->execute() or 
#					(&dbdie("$DBI::errstr") and ($dberror = 1));
#			}
#			
#			if ($dberror == 0)
#			{
#				# should be just one row cause recordID is primary
#				if (my($title, $description, $physchar, $size) = 
#						$sthGetBasic->fetchrow_array())
#				{
#					# Set the form templates
#					$VAR{'BASIC-VAL-TITLE'} = $title;
#					$VAR{'BASIC-VAL-DESCRIPTION'} = $description;
#
#					$VAR{'BASIC-VAL-PHYSCHAR'} = $physchar;
#					$VAR{'BASIC-VAL-SIZE'} = $size;
#			}
			
		# end $update
#		}

	# end $iteration1
	}
	
	# On second iteration, validate form values then add/update the DB OR
	# if not valid display the add/edit form with submitted values and red
	# error row to indicate error.
	elsif ($iteration == 2)
	{
		# Validate form fields
		my ($validform) = 1;
# NOTE: This form cannot be invalid, cause just contains list boxes
#		if (!exists($FORM{'title'}) || $FORM{'title'} eq "")
#		{
#			$validform = 0;
#			$VAR{'ERROR-ROW-TEXT'} = $CFG{'error-title'};
#			$VAR{'BASIC-ERRORROW-TITLE'} = &parse_template("$CFG{'admin-#templates-rel-dir'}/error-row-xhtml.template", %VAR);
#		}

#		if ( (my($numsel) = ($cgi->param(concepts))) <= 0 )
#		{
#			$validform = 0;
#			$VAR{'ERROR-ROW-TEXT'} = $CFG{'error-concept'};
#			$VAR{'BASIC-ERRORROW-CONCEPTS'} = &parse_template("$CFG{'admin-#templates-rel-dir'}/error-row-xhtml.template", %VAR);
#		}

		# FURTHER PROCESSING that is required if the form is valid or not
		# NOTE: none for this form, check addupdatebasic.cgi

		# If form valid add/update DB and do not display add/update form
		if ($validform == 1)
		{
			$displayaddeditform = 0;

			# Add/Update the Basic Collection fields in the Database
			
			# Prepare all submitted form values for database
#			my ($title, $description, $physchar, $size) = 
#					("", "", "", "");
#
#			$title = $FORM{'title'};
#			$description = $FORM{'description'};
#			$physchar = $FORM{'physchar'};
#			$size = $FORM{'size'};


			# Add/Update the Times associated with this collection in the DB

			# ADD all the selected entries from list box,
			# which are not already in the database
			my(%selected) = ();
			my($totaladded) = 0;
			foreach my $selectedid ($cgi->param('times'))
			{
				# Make into hash to save time later
				$selected{$selectedid} = 1;
		
				# Add if selected does not exist in DB
				if (!exists($dbtimes{$selectedid}))
				
				{
					my ($sthAdd) = 
					$dbh->prepare("insert into CollectionTime values(?,?)");

					if ($sthAdd == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr");
					}
					else
					{
						$totaladded += $sthAdd->execute($FORM{'recordID'},
													    $selectedid) or
								 (($dberror = 1) and &dbdie("$DBI::errstr"));
					}
				}
			}
	
			# Set the details of added records in the templates
			$VAR{'ADD-UPD-DEL-RESULT'} .= 
				qq(Added $totaladded row(s) to CollectionTime<br />);

	
			# Delete all Time Name IDs that are in DB, but not selected
			# from list in previous call of form
			my ($totaldeleted) = 0;
			foreach my $dbid (keys %dbtimes)
			{
				if (!exists($selected{$dbid}))
				{
					my ($sthDel) = $dbh->prepare("delete from CollectionTime where collection=$FORM{'recordID'} and time=$dbid");
			
					if ($sthDel == 0)
					{
						$dberror = 1;
						&dbdie("$DBI::errstr");
					}
					else
					{
						$totaldeleted += $sthDel->execute() or
								 (($dberror = 1) and &dbdie("$DBI::errstr"));
					}
				}
			}
	
			# Set the details of deleted records in the templates
			$VAR{'ADD-UPD-DEL-RESULT'} .= 
				qq(Deleted $totaldeleted row(s) from CollectionTime<br />);

			if ($dberror == 1)
			{
				if ($action eq "Update")
				{
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Error(s) occurred so not all data may have been Updated!</b><br />);
				}
#				else
#				{
#					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Error(s) occurred #so not all data may have been Added!</b><br />);
#				}
			}
			else
			{
					$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Record was Updated Successfully!</b><br />);
			}
			
			if ($action eq "Update")
			{
				$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<a href="recordmanager.cgi?recordID=$FORM{'recordID'}#time">[Return to Collection Record Manager]</a>);
			}
#			else
#			{
#				$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<a #href="recordmanager.cgi?recordID=$FORM{'recordID'}#time">[Proceed to Collection #Record Manager]</a>);
#			}
			
			# Output results template
			print &parse_template("$CFG{'admin-templates-rel-dir'}/add-upd-del-results-xhtml.template", %VAR);
		}
		
		# Form not valid, so save all fields by setting their values as before
		else
		{
			# This form cannot be invalid
		}
	# end iteration 2
	}
	
	if ($dberror)
	{
		$displayaddeditform = 0;
	}
	
	# Set standard template params and display add/edit form
	if ($displayaddeditform == 1)
	{
		my ($formstr) = "";
		$formstr = $cgi->start_form({action=>'settimes.cgi', 
									 method=>'post'});

		# Only set the recordID param if we are not Adding a collection
		if (!($action eq "Add"))
		{
			$formstr .= $cgi->hidden({name=>'recordID', 
									  value=>"$FORM{'recordID'}"});
		}

		$formstr .= $cgi->hidden({name=>'action', 
								  value=>"$FORM{'action'}"});
								  
		# Always indicate second iteration of script
		$formstr .= $cgi->hidden({name=>'call', value=>"2"});

		$VAR{'TIMES-FORM-START-HIDDEN'} = $formstr;

		# Field help texts
		$VAR{'TIMES-HELP-TIMES'} = $CFG{'help-times-form-time'};

		print &parse_template("$CFG{'admin-templates-rel-dir'}/set-times-xhtml.template", %VAR);
	}
# end add || update
}

# This element is to be deleted
elsif ($dberror == 0 && $action eq "Delete")
{
	# Do nothing for this script. See addeditdelstrength.cgi for impl.
}

# Disconnect
if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}

# Page Footer
print &parse_template("$CFG{'admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();
