#!/bin/perl -wT

############################################################################
# updateadvanced.cgi
#
# Description
#
# Allows the administrator update the advanced information associated with
# a particular collection.
#
# Change History
#
# 18/06/05 Ian Bolton
# Created 
#
# 01/07/05 Ian Bolton
# Title and Navbar should now say Additional, rather than Advanced.
# Also updated links to return to correct part of record manager.
#
# 03/10/04 Ian Bolton
# The associated collection is now a text field in the database.
#
# $Id: updateadvanced.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# Standard help image used on the form
$VAR{'HELP-IMAGE'} = "$CFG{'image-dir'}/helpQ.gif";

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

# Find out what the previous link navbar should be
my ($prevlink) = "";
if (exists($FORM{'recordID'}))
{
	$prevlink = qq(<a href="recordmanager.cgi?recordID=$FORM{'recordID'}#additional">Collection Record Manager</a> &gt;&gt;);
}

# Advanced data only ever needs to be updated, because collection
# already exists!
my ($action) = "Update";

# Each script action expects this same script to be called twice
my ($iteration) = 1;
if (exists($FORM{'call'}))
{
	if ($FORM{'call'} == 2)
	{
		$iteration = 2;
	}
}

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; <a href="searchorbrowse.cgi">Search or Browse</a> &gt;&gt; $prevlink $action Additional Information);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - $action Additional Information",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});

print &parse_template("$CFG{'admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&dbdie("$DBI::errstr");
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}


# Are we Adding or Updating a record
if ($dberror == 0 && ($action eq "Add" || $action eq "Update"))
{
	my ($displayaddeditform) = 1;
	
	# Set submit button text template
	if ($action eq "Update")
	{
		$VAR{'ADVANCED-FORM-SUBMIT-VALUE'} = "Update Database";
	}

#	else
#	{
#		$VAR{'STRENGTH-SUBMIT-VALUE'} = "Add to Database";
#	}
	
	# On first iteration display the add/edit form with or without values
	# set from the database
	if ($iteration == 1)
	{
		
		# Need to load add/edit form template values from DB
		if ($action eq "Update")
		{
			
			my ($sthGetAdv) = $dbh->prepare("select accumulationDateRange, contentsDateRange, superCollection, associatedCollection, associatedPublication, accrualstatus, legalstatus, custodialhistory, accesscontrol, note from Collection where id=$FORM{'recordID'}");
	
			if ($sthGetAdv == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");
			}

			if ($dberror == 0)
			{
				$sthGetAdv->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}
			
			if ($dberror == 0)
			{
				# should be just one row cause recordID is primary
				if (my($accdaterange, $contentdaterange, 
					$supercoll, $associatedcoll, $associatedpub, 
					$accrualstatus, $legalstatus, $custodialhist,
					$accesscontrol, $note) = $sthGetAdv->fetchrow_array())
				{
					# Set the form templates
					$VAR{'ADVANCED-VAL-ACC-DATE-RANGE'} = $accdaterange;
					$VAR{'ADVANCED-VAL-CNT-DATE-RANGE'} = $contentdaterange;

					$VAR{'ADVANCED-VAL-ASSOCIATED-PUB'} = $associatedpub;
					$VAR{'ADVANCED-VAL-ACCRUAL-STATUS'} = $accrualstatus;
					$VAR{'ADVANCED-VAL-LEGAL-STATUS'} = $legalstatus;
					$VAR{'ADVANCED-VAL-CUSTODIAL-HIST'} = $custodialhist;
					$VAR{'ADVANCED-VAL-ACCESS-CTRL'} = $accesscontrol;
					$VAR{'ADVANCED-VAL-NOTE'} = $note;
					$VAR{'ADVANCED-VAL-ASSOCIATEDCOLL'} = $associatedcoll;
					
					# Fill in the list boxes with collection titles
					my($sthGetCollections) = $dbh->prepare("select id, title from Collection order by title");
					if ($sthGetCollections == 0)
					{
						$dberror = 1;	
						&dbdie("$DBI::errstr");
					}
					if ($dberror == 0)
					{
						$sthGetCollections->execute() or 
							(&dbdie("$DBI::errstr") and ($dberror = 1));
					}
					if ($dberror == 0)
					{
						while (my($recid,$rectitle) = 
							$sthGetCollections->fetchrow_array())
						{
							$sizelimit = 35;
							my($basestr) = qq(<option value="$recid" );
							my($superstr) = $basestr;

							# Cannot associate record with itself
							if ($recid == $FORM{'recordID'})
							{
								next;
							}

							# See if the record id should be selected
							# by default in the super collection list
							if ($recid == $supercoll)
							{
								$superstr .= "selected>";
							}
							else
							{
								$superstr .= ">";
							}


							# Limit size of title when put into list box
							if ((length($rectitle) > $sizelimit) &&
								($sizelimit > 2))
							{
								$rectitle = substr($rectitle, 0, $sizelimit-3);
								$rectitle .= "..";
							}

							$superstr .= $rectitle . '</option>';

							$VAR{'ADVANCED-OPTIONS-SUPERCOLL'} .= $superstr;
						}
					}
				}
				else
				{
					$dberror = 1;
					&dbdie("Could not find advanced record with ID $FORM{'recordID'}");
				}
			}
			
		# end $update
		}

	# end $iteration1
	}
	
	# On second iteration, validate form values then add/update the DB OR
	# if not valid display the add/edit form with submitted values and red
	# error row to indicate error.
	elsif ($iteration == 2)
	{
		# Validate form: NB This particular form cannot be invalid
		my ($validform) = 1;

#		if (!exists($FORM{'accdaterange'}) || $FORM{'accdaterange'} eq "")
#		{
#			$validform = 0;
#			$VAR{'ERROR-ROW-TEXT'} = "acc date range required";
#			$VAR{'ADVANCED-ERRORROW-ACC-DATE-RANGE'} = #&parse_template("$CFG{'admin-templates-rel-dir'}/error-row-xhtml.template", #%VAR);
#		}
		
		# If form valid add/update DB and do not display add/update form
		if ($validform == 1)
		{
			$displayaddeditform = 0;

			# Prepare all submitted form values for database
			my ($accdaterange, $contentdaterange, $supercoll, $associatedcoll, 
				$associatedpub, $accrualstatus, $legalstatus, $custodialhist,
				$accesscontrol, $note) = (NULL, NULL, NULL, NULL, NULL,
											NULL, NULL, NULL, NULL, NULL);
			$accdaterange = $FORM{'accdaterange'};
			$contentdaterange = $FORM{'cntdaterange'};
			$associatedpub = $FORM{'associatedpub'};
			$accrualstatus = $FORM{'accrualstatus'};
			$legalstatus = $FORM{'legalstatus'};
			$custodialhist = $FORM{'custodialhist'};
			$accesscontrol = $FORM{'accessctrl'};
			$note = $FORM{'note'};
			$associatedcoll = $FORM{'associatedcoll'};
			
			if (!($FORM{'supercollection'} eq "none"))
			{
				$supercoll = $FORM{'supercollection'};
			}

			# Update
			my($sthAddUpd);
			if ($action eq "Update")
			{
				$sthAddUpd = $dbh->prepare("update Collection set accumulationDateRange=?, contentsDateRange=?, superCollection=?, associatedCollection=?, associatedPublication=?, accrualstatus=?, legalstatus=?, custodialhistory=?, accesscontrol=?, note=? where id=$FORM{'recordID'}");
			}
			# Add
			else
			{
				# Add N/A for this form
			}
			
			if ($sthAddUpd == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");	
			}
			
			my ($rows) = 0;
			if ($dberror == 0)
			{
				$rows = $sthAddUpd->execute($accdaterange, $contentdaterange,
							$supercoll, $associatedcoll, $associatedpub, 
							$accrualstatus, $legalstatus, $custodialhist,
							$accesscontrol, $note) or 
						(($dberror = 1) && &dbdie("$DBI::errstr"));
				if ($rows <= 0)
				{
					$rows = 0;
				}
			}
			
			# Set output result row template
			if ($dberror == 0)
			{
				if ($action eq "Update")
				{
					$VAR{'ADD-UPD-DEL-RESULT'} = qq(Updated $rows row(s) from Collection<br />);
				}
				else
				{
#$VAR{'ADD-UPD-DEL-RESULT'} = qq(Added $rows row to CollectionStrength<br />);
				}
			}
			else
			{
				$VAR{'ADD-UPD-DEL-RESULT'} = qq(Error(s) occurred so not all data may have been $action!<br />);
			}
			
			$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<a href="recordmanager.cgi?recordID=$FORM{'recordID'}#additional">[Return to Collection Record Manager]</a>);
			
			# Output results template
			print &parse_template("$CFG{'admin-templates-rel-dir'}/add-upd-del-results-xhtml.template", %VAR);
		}
		
		# Form not valid, so save all fields by setting their values as before
		else
		{
			# FORM CANNOT BE INVALID, OR WOULD HAVE TO SAVE VALUES HERE
			$VAR{'ADVANCED-VAL-ACC-DATE-RANGE'} = $FORM{'accdaterange'};
			$VAR{'ADVANCED-VAL-CNT-DATE-RANGE'} = $FORM{'cntdaterange'};
			
		}
	# end iteration 2
	}
	
	if ($dberror)
	{
		$displayaddeditform = 0;
	}
	
	# Set standard template params and display add/edit form
	if ($displayaddeditform == 1)
	{
		my ($formstr) = "";
		$formstr = $cgi->start_form({action=>'updateadvanced.cgi', 
									 method=>'post'});
		$formstr .= $cgi->hidden({name=>'recordID', 
								  value=>"$FORM{'recordID'}"});

#		$formstr .= $cgi->hidden({name=>'action', 
#								  value=>"$FORM{'action'}"});
								  
		# Always indicate second iteration of script
		$formstr .= $cgi->hidden({name=>'call', value=>"2"});

		$VAR{'ADVANCED-FORM-START-HIDDEN'} = $formstr;
		
		# TODO: Field help texts
		$VAR{'ADVANCED-HELP-ACC-DATE-RANGE'} = $CFG{'help-adv-acc-date-range'};
		$VAR{'ADVANCED-HELP-CNT-DATE-RANGE'} = $CFG{'help-adv-cnt-date-range'};
		$VAR{'ADVANCED-HELP-SUPERCOLL'} = $CFG{'help-adv-supercoll'};
		$VAR{'ADVANCED-HELP-ASSOCIATEDCOLL'} = $CFG{'help-adv-associatedcoll'};
		$VAR{'ADVANCED-HELP-ASSOCIATED-PUB'} = $CFG{'help-adv-associatedpub'};
		$VAR{'ADVANCED-HELP-ACCRUAL-STATUS'} = $CFG{'help-adv-accrualstatus'};
		$VAR{'ADVANCED-HELP-LEGAL-STATUS'} = $CFG{'help-adv-legalstatus'};
		$VAR{'ADVANCED-HELP-CUSTODIAL-HIST'} = $CFG{'help-adv-custodialhist'};
		$VAR{'ADVANCED-HELP-ACCESS-CTRL'} = $CFG{'help-adv-accessctrl'};
		$VAR{'ADVANCED-HELP-NOTE'} = $CFG{'help-adv-note'};

		print &parse_template("$CFG{'admin-templates-rel-dir'}/update-advanced-form-xhtml.template", %VAR);
	}
# end add || update
}

# This element is to be deleted
elsif ($dberror == 0 && $action eq "Delete")
{
	# Do nothing for this script. See addeditdelstrength.cgi for impl.
}

# Disconnect
if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}

# Page Footer
print &parse_template("$CFG{'admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();
