#!/bin/perl -wT

############################################################################
# updatelocation.cgi
#
# Description
#
# Allows the administrator to set the Location data for a collection.
#
# Cannot be deleted without removing the whole collection from the DB, once
# the table is created - it will be kept transparently, and filled with
# blanks to simulate its removal.
#
# Change History
#
# 22/06/05 Ian Bolton
# Created 
#
# 01/07/07 Ian Bolton
# Updated links to record manager for ease of navigation.
# Put verification of URL.
#
# $Id: updatelocation.cgi,v 1.1 2004/10/03 15:35:02 ibolton Exp $
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';

############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# Standard help image used on the form
$VAR{'HELP-IMAGE'} = "$CFG{'image-dir'}/helpQ.gif";

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

# Find out what the previous link navbar should be
my ($prevlink) = "";
if (exists($FORM{'recordID'}))
{
	$prevlink = qq(<a href="searchorbrowse.cgi">Search or Browse</a> &gt;&gt; <a href="recordmanager.cgi?recordID=$FORM{'recordID'}#location">Collection Record Manager</a> &gt;&gt;);
}


# This script does not need to distinguish between add and update because
# it can work it out for itself. Just set to the more common Update op.
my ($action) = "Update";

# Each script action expects this same script to be called twice
my ($iteration) = 1;
if (exists($FORM{'call'}))
{
	if ($FORM{'call'} == 2)
	{
		$iteration = 2;
	}
}

$VAR{'NAVBAR'} = qq(<a href="main.cgi">Main Menu</a> &gt;&gt; $prevlink $action Location Information);

# Print Standard Header
print $cgi->header();
print $cgi->start_html({-title=>"Admin - $action Location Information",
						-style=>"$CFG{'admin-stylesheet-rel-path'}"});

print &parse_template("$CFG{'admin-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

# Connect to DB
my ($dbh);
my ($dberror) = 1;
my ($dbconnected) = 0;
$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
if ($dbh == 0)
{
	&dbdie("$DBI::errstr");
}
else
{
	$dberror = 0;
	$dbconnected = 1;
}

# Are we Adding or Updating a record
if ($dberror == 0 && ($action eq "Add" || $action eq "Update"))
{
	my ($displayaddeditform) = 1;
	
	# Set submit button text template
	if ($action eq "Update")
	{
		$VAR{'LOCATION-FORM-SUBMIT-VALUE'} = "Update Database";
	}
	
	# On first iteration display the add/edit form with or without values
	# set from the database
	if ($iteration == 1)
	{
		
		# Need to load add/edit form template values from DB
		if ($action eq "Update" && $dberror == 0)
		{
			
			my ($sthGetLocation) = $dbh->prepare("select name, address, postcode, country, url, accessconditions, seealso from CollectionLocation where collectionid=$FORM{'recordID'}");
	
			if ($sthGetLocation == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");
			}

			if ($dberror == 0)
			{
				$sthGetLocation->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}
			
			if ($dberror == 0)
			{
				# should be just one row cause collectionID is primary
				if (my($locname, $locaddress, $locpostcode, $loccountry,
						$loclocator, $locaccess, $seealso) = 
							$sthGetLocation->fetchrow_array())
				{
					# Set the form templates
					$VAR{'LOCATION-VAL-NAME'} = $locname;
					$VAR{'LOCATION-VAL-ADDRESS'} = $locaddress;
					$VAR{'LOCATION-VAL-POSTCODE'} = $locpostcode;
					$VAR{'LOCATION-VAL-COUNTRY'} = $loccountry;
					$VAR{'LOCATION-VAL-LOCATOR'} = $loclocator;
					$VAR{'LOCATION-VAL-ACCESS'} = $locaccess;
					$VAR{'LOCATION-VAL-SEEALSO'} = $seealso;
				}
				else
				{
					# Record probably not created yet, so ignore
					# because the fields will be just filled blank
				}
			}
			
		# end $update
		}

	# end $iteration1
	}
	
	# On second iteration, validate form values then add/update the DB OR
	# if not valid display the add/edit form with submitted values and red
	# error row to indicate error.
	elsif ($iteration == 2)
	{
		# Validate form fields
		my ($validform) = 1;

		# If URL field was submitted, and not blank, then verify
		# it is a valid URL address.
		if (exists($FORM{'locator'}) && !($FORM{'locator'} eq ""))
		{
			if ($FORM{'locator'} !~ /^(http|https|ftp):\/\//)
			{
				$validform = 0;
				$VAR{'ERROR-ROW-TEXT'} = $CFG{'error-invalid-locator'};
				$VAR{'LOCATION-ERRORROW-LOCATOR'} = &parse_template("$CFG{'admin-templates-rel-dir'}/error-row-xhtml.template", %VAR);
			}
		}

		# If form valid add/update DB and do not display add/update form
		if ($validform == 1)
		{
			$displayaddeditform = 0;

			# Add/Update the Basic Collection fields in the Database
			
			# Prepare all submitted form values for database
			my ($locname, $locaddress, $locpostcode, $loccountry,
				$loclocator, $locaccess, $seealso) = 
					("", "", "", "", "", "", "");

			$locname = $FORM{'name'};
			$locaddress = $FORM{'address'};
			$locpostcode = $FORM{'postcode'};
			$loccountry = $FORM{'country'};
			$loclocator = $FORM{'locator'};
			$locaccess = $FORM{'access'};
			$seealso = $FORM{'seealso'};

			# DO AUTOMATIC CHECK, TO DECIDE IF TO UPDATE OR ADD BY 
			# CALLING FOLLOWING QUERY
			my ($sthAutoCheck) = $dbh->prepare("select name from CollectionLocation where collectionid=$FORM{'recordID'}");
	
			if ($sthAutoCheck == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");
			}

			if ($dberror == 0)
			{
				$sthAutoCheck->execute() or 
					(&dbdie("$DBI::errstr") and ($dberror = 1));
			}
			
			my($updatecolllocation) = 0;
			if ($dberror == 0)
			{
				# should be just one row cause collectionID is primary
				if (my($tmpthrowawayname) = 
							$sthAutoCheck->fetchrow_array())
				{
					$updatecolllocation = 1;
				}
				else
				{
					$updatecolllocation = 0;
				}
			}

			# Update
			my($sthAddUpd);
			if ($updatecolllocation == 1)
			{
				$sthAddUpd = $dbh->prepare("update CollectionLocation set name=?, address=?, postcode=?, country=?, url=?, accessconditions=?, seealso=? where collectionid=$FORM{'recordID'}");
			}
			# Add
			else
			{
				$sthAddUpd = $dbh->prepare("insert into CollectionLocation(collectionid, name, address, postcode, country, url, accessconditions, seealso) values($FORM{'recordID'},?,?,?,?,?,?,?)");
			}
			
			if ($sthAddUpd == 0)
			{
				$dberror = 1;	
				&dbdie("$DBI::errstr");	
			}
			
			my ($rows) = 0;
			if ($dberror == 0)
			{
				$rows = $sthAddUpd->execute($locname, $locaddress, $locpostcode,
											$loccountry, $loclocator, 
											$locaccess, $seealso) or 
						(($dberror = 1) && &dbdie("$DBI::errstr"));
				if ($rows <= 0)
				{
					$rows = 0;
				}
			}
			
			# Set output result row template
			if ($dberror == 0)
			{
				if ($updatecolllocation == 1)
				{
					$VAR{'ADD-UPD-DEL-RESULT'} = qq(Updated $rows row(s) from CollectionLocation<br />);
				}
				else
				{
					$VAR{'ADD-UPD-DEL-RESULT'} = qq(Added $rows row to CollectionLocation<br />);
				}
			}

			if ($dberror == 1)
			{
				$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Error(s) occurred so not all data may have been Updated!</b><br />);
			}
			else
			{
				$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<br /><b>Record was successfully Updated!</b><br />);
			}
			
			$VAR{'ADD-UPD-DEL-RESULT'} .= qq(<a href="recordmanager.cgi?recordID=$FORM{'recordID'}#location">[Return to Collection Record Manager]</a>);
			
			# Output results template
			print &parse_template("$CFG{'admin-templates-rel-dir'}/add-upd-del-results-xhtml.template", %VAR);
		}
		
		# Form not valid, so save all fields by setting their values as before
		else
		{
			# Save the text and text area fields
			$VAR{'LOCATION-VAL-NAME'} = $FORM{'name'};
			$VAR{'LOCATION-VAL-ADDRESS'} = $FORM{'address'};
			$VAR{'LOCATION-VAL-POSTCODE'} = $FORM{'postcode'};
			$VAR{'LOCATION-VAL-COUNTRY'} = $FORM{'country'};
			$VAR{'LOCATION-VAL-LOCATOR'} = $FORM{'locator'};
			$VAR{'LOCATION-VAL-ACCESS'} = $FORM{'access'};
			$VAR{'LOCATION-VAL-SEEALSO'} = $FORM{'seealso'};
		}
	# end iteration 2
	}
	
	if ($dberror)
	{
		$displayaddeditform = 0;
	}
	
	# Set standard template params and display add/edit form
	if ($displayaddeditform == 1)
	{
		my ($formstr) = "";
		$formstr = $cgi->start_form({action=>'updatelocation.cgi', 
									 method=>'post'});

		# Only set the recordID param if we are not Adding a collection
		$formstr .= $cgi->hidden({name=>'recordID', 
								  value=>"$FORM{'recordID'}"});
								  
		# Always indicate second iteration of script
		$formstr .= $cgi->hidden({name=>'call', value=>"2"});

		$VAR{'LOCATION-FORM-START-HIDDEN'} = "$formstr";

		# Field help texts
		$VAR{'LOCATION-HELP-NAME'} = $CFG{'help-location-name'};
		$VAR{'LOCATION-HELP-ADDRESS'} = $CFG{'help-location-address'};
		$VAR{'LOCATION-HELP-POSTCODE'} = $CFG{'help-location-postcode'};
		$VAR{'LOCATION-HELP-COUNTRY'} = $CFG{'help-location-country'};
		$VAR{'LOCATION-HELP-LOCATOR'} = $CFG{'help-location-locator'};
		$VAR{'LOCATION-HELP-ACCESS'} = $CFG{'help-location-access'};
		$VAR{'LOCATION-HELP-SEEALSO'} = $CFG{'help-location-seealso'};

		print &parse_template("$CFG{'admin-templates-rel-dir'}/update-location-form-xhtml.template", %VAR);
	}
# end add || update
}

# This element is to be deleted
elsif ($dberror == 0 && $action eq "Delete")
{
	# Do nothing for this script. See addeditdelstrength.cgi for impl.
}

# Disconnect
if ($dbconnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}

# Page Footer
print &parse_template("$CFG{'admin-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();
