#!/bin/perl -wT

############################################################################
# showbriefdetails.cgi
#
# Description
#
# Show just a couple of brief details about the collection.
#
# Change History
#
# 28/06/04 Ian Bolton
# Created, based on showdetails.cgi
#
# 08/07/04 Ian Bolton
# Updated so that collection title is displayed in browser title bar.
#
############################################################################

############################################################################
# MODULES
############################################################################

# Use the CGI module
use CGI;

# Comment out for final release
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);

# Display nice XHTML code
use CGI::Pretty;

# Uncomment at end to show compliance
#use strict;

use DBI;

############################################################################
# Library Includes
############################################################################

# Look in these locations for libraries
push(@INC, "../common/require");
push(@INC, "../common/cfg");

# Template parsing functions
require 'template.pl';

# Configuration
require 'config.pl';

# Utility function library
require 'utility.pl';


############################################################################
# Main Script
############################################################################

# Load Config
my (%CFG) = &get_config();

# Construct the CGI object
my $cgi = CGI->new;

# This contains the values to replace in each template
my (%VAR) = &get_template_values(%CFG);

# parse form variables
%FORM;
foreach my $par ($cgi->param())
{
	$FORM{$par} = $cgi->param($par);
}

# Find out what the previous link navbar should be
my ($prevlink) = "";
my ($dberror) = 1;
if (exists($FORM{'searchtype'}))
{
	if ($FORM{'searchtype'} eq "basic")
	{
		$prevlink = qq(<a href="search.cgi">Search</a> &gt;&gt; <a href="performsearch.cgi?searchID=$FORM{'searchID'}&searchtype=$FORM{'searchtype'}&action=retrieve&page=$FORM{'page'}&numresultsperpage=$FORM{'numresultsperpage'}">Search Results</a> &gt;&gt; );
	}
	elsif ($FORM{'searchtype'} eq "advanced")
	{
		$prevlink = qq(<a href="advancedsearch.cgi">Advanced Search</a> &gt;&gt; <a href="performsearch.cgi?searchID=$FORM{'searchID'}&searchtype=$FORM{'searchtype'}&action=retrieve&page=$FORM{'page'}&numresultsperpage=$FORM{'numresultsperpage'}">Search Results</a> &gt;&gt; );
	}
	else
	{
		&dienice("Unknow searchtype passed to script!");
		$dberror = 1;
	}
}
else
{
	&dienice("Invalid parameters passed into script!");
	$dberror = 1;
}

$VAR{'NAVBAR'} = qq($prevlink Brief Record Details&nbsp;&nbsp;<span class="small"><a href="showdetails.cgi?recordID=$FORM{'recordID'}&searchID=$FORM{'searchID'}&searchtype=$FORM{'searchtype'}&page=$FORM{'page'}&numresultsperpage=$FORM{'numresultsperpage'}">[Show Full Record]</a></span>);

# Print Standard Header
my ($recordtitle) = &get_record_title($FORM{'recordID'}, \%CFG);
print $cgi->header();
print $cgi->start_html({-title=>"Brief Record Details - $recordtitle",
						-style=>"$CFG{'public-stylesheet-rel-path'}"});


print &parse_template("$CFG{'public-templates-rel-dir'}/mainheader-xhtml.template", %VAR);

my ($criticalerror) = 0;
my ($dbConnected) = 0;
my ($dbh);

# Check Record ID
if (!exists($FORM{'recordID'}))
{
	$criticalerror = 1;	
	print $cgi->h2("Error: No Collection Record ID given to script!");
	print $cgi->br();
}
elsif ($FORM{'recordID'} eq "")
{
	$criticalerror = 1;
	print $cgi->h2("Error: A Blank Collection Record ID was given to script!");
	print $cgi->br();
}
else
{
	# Connect to MySQL Database
#	print $CFG{'database-file-path'};
	$dbh = DBI->connect("dbi:mysql:$CFG{'database-name'}" . ";mysql_read_default_file=$CFG{'database-file-path'}");
	if ($dbh == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	else
	{
		$dbConnected = 1;
	}
}

# Check if ID passed into the script is valid
if ($criticalerror == 0)
{
	my($sthGetID) = $dbh->prepare("select id from Collection where id=?");
	
	if ($sthGetID == 0)
	{
		$criticalerror = 1;	
		&dbdie("$DBI::errstr");
	}
		
	if ($criticalerror == 0)
	{
		$sthGetID->execute($FORM{recordID}) or (&dbdie() and ($criticalerror = 1));

		if ($criticalerror == 0)
		{
			if (!(my($id) = $sthGetID->fetchrow_array()))
			{
				$criticalerror = 1;
				print $cgi->h2("Error: An Invalid Collection Record ID was passed into the Script!");
				print $cgi->br();
			}
		}
	}	
}


# Output Title and Description
if ($criticalerror == 0)
{
	my ($sthGetBasic) = $dbh->prepare("select title, description from Collection where id=$FORM{recordID}");
	
	if ($sthGetBasic == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetBasic->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in Basic fields templates
	if ($criticalerror == 0)
	{
		my($title, $desc) = $sthGetBasic->fetchrow_array();
		

		$VAR{'DB-TITLE'}=$title;
		$VAR{'DB-DESCRIPTION'}=$desc;
	}
}

my ($callresult);

# Output List of sub-collections associated with this collection
if ($criticalerror == 0)
{
	my ($sthGetSubCollections) = $dbh->prepare("select id, title from Collection where superCollection=$FORM{'recordID'} order by title");

	if ($sthGetSubCollections == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetSubCollections->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		my ($num) = 0;
		while (my($subrecid, $subrectitle) =
			$sthGetSubCollections->fetchrow_array())
		{
			$num++;
			if ($num == 1)
			{
				$VAR{'DB-SUB-COLLECTIONS'} = qq(<ul>);
			}
			
			$VAR{'DB-SUB-COLLECTIONS'} .= qq(<li><a href="showbriefdetails.cgi?recordID=$subrecid&searchID=$FORM{'searchID'}&searchtype=$FORM{'searchtype'}&page=$FORM{'page'}&numresultsperpage=$FORM{'numresultsperpage'}">$subrectitle</a></li>);
		}
		
		if ($num == 0)
		{
			$VAR{'TOP-RIGHT'} = "";
			$VAR{'TOP-LEFT'} = qq(<em>This collection contains no Sub-Collections</em>);

			# Output results via templates
			$VAR{'DB-SUB-COLLECTIONS'} = &parse_template("$CFG{'public-templates-rel-dir'}/repeat-row-xhtml.template", %VAR);
		}
		else
		{
			$VAR{'DB-SUB-COLLECTIONS'} .= qq(</ul>);
		}
	}
}

# Output Super collection and accesscontrol
if ($criticalerror == 0)
{
	my ($sthGetAdv) = $dbh->prepare("select superCollection, accesscontrol from Collection where id=$FORM{'recordID'}");
	
	if ($sthGetAdv == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetAdv->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		# should be just one row cause recordID is primary
		if (my($supercoll, $accesscontrol) = $sthGetAdv->fetchrow_array())
		{
			$VAR{'DB-ADV-ACCESS-CONTROL'} = 
				&get_advanced_template($accesscontrol, 
					"No access control information is given");

			# Super, associated collection are int ID's
			($VAR{'DB-ADV-SUPER-COLLECTION'}, $criticalerror) =
				&get_advanced_link_template($supercoll,
					"This collection is not part of another collection",
					$dbh);
		}
		else
		{
			$criticalerror = 1;
			&dbdie("Cannot get advanced data using record ID $FORM{'recordID'}");
		}
	}
}

# Output postal address, locator, and access conditions
if ($criticalerror == 0)
{
	my ($sthGetLocation) = $dbh->prepare("select address, url, accessconditions from CollectionLocation where collectionid=$FORM{'recordID'}");
	
	if ($sthGetLocation == 0)
	{
		$criticalerror = 1;
		&dbdie("$DBI::errstr");
	}
	
	if ($criticalerror == 0)
	{
		$sthGetLocation->execute() or 
			(&dbdie("$DBI::errstr") and ($criticalerror = 1));
	}
	
	# Fill in templates
	if ($criticalerror == 0)
	{
		# Initialise all element to blank
		my($locaddress, $locurl, $locaccess) = ("", "", "");

		# should be just one row cause recordID is primary
		($locaddress, $locurl, $locaccess) = $sthGetLocation->fetchrow_array();

		# Note: The location record is handled differently from the Advanced
		# record in the DB. It has a separate record, which may or may not
		# exist. If there are no rows, then hide the fact there is no record
		# - so from the users view it is handled no differently.

		$VAR{'DB-LOCATION-ADDRESS'} = 
			&get_advanced_template($locaddress, 
				"No location address is given");

		if (!($locurl eq ""))
		{
			$VAR{'DB-LOCATION-LOCATOR'} = qq(<a href="$locurl">$locurl</a>);
		}
		else
		{
			$VAR{'DB-LOCATION-LOCATOR'} = 
				&get_advanced_template($locurl, 
					"No home page URL is given");
		}

		$VAR{'DB-LOCATION-ACCESS-CONDITIONS'} = 
			&get_advanced_template($locaccess, 
				"No access conditions are specified");
	}
}

# Display page
if ($criticalerror == 0)
{
	print &parse_template("$CFG{'public-templates-rel-dir'}/showbriefdetails-xhtml.template", %VAR);
	
}

# Disconnect
if ($dbConnected == 1)
{
	$dbh->disconnect() || &dbdie("Can't Disconnect!");
}
	
# Page Footer
print &parse_template("$CFG{'public-templates-rel-dir'}/mainfooter-xhtml.template", %VAR);

print $cgi->end_html();


##################################################################
# HELPER FUNCTIONS
##################################################################
sub get_advanced_template()
{
	my($dbvalue, $nulltext) = @_;
	my($rettext);

	if ($dbvalue eq "")
	{
		$rettext = '<em>' . $nulltext . '</em>';
	}
	else
	{
		$rettext = $dbvalue;
		$rettext =~ s/\n/<br \/>/g;
	}
	
	return($rettext);
}

sub get_advanced_link_template()
{
	my($dbvalue, $nulltext, $dbh) = @_;
	my($rettext, $retresult) = ("", 0);

	if ($dbvalue == NULL)
	{
		$rettext =	qq(<em>$nulltext</em>);
	}
	else
	{
		my($sthGetTitle) = 
			$dbh->prepare("select title from Collection where id=$dbvalue");
		
		if ($sthGetTitle == 0)
		{
			$retresult = 1;
			&dbdie("$DBI::errstr");
		}
		else
		{
			$sthGetTitle->execute() or 
				(&dbdie("$DBI::errstr") and ($retresult = 1));
		}
		if ($retresult == 0)
		{
			if (my($linktitle) = $sthGetTitle->fetchrow_array())
			{
				$rettext = qq(<a href="showbriefdetails.cgi?recordID=$dbvalue&searchID=$FORM{'searchID'}&searchtype=$FORM{'searchtype'}&page=$FORM{'page'}&numresultsperpage=$FORM{'numresultsperpage'}">$linktitle</a>);
			}
			else
			{
				$retresult = 1;
				&dbdie("Advanced Data: Cannot get title for recordID $dbvalue");
			}
		}
	}
	
	return($rettext, $retresult);
}
